@@ -20,16 +20,6 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
- *
- * NOTE
- * 
- * This code was audited for completeness against the documented features
- * of Comctl32.dll version 6.0 on Sep. 9, 2002, by Dimitrie O. Paun.
- * 
- * Unless otherwise noted, we believe this code to be complete, as per
- * the specification mentioned above.
- * If you discover missing features, or bugs, please note them below.
- * 
  */
 
 #include <ctype.h>
@@ -48,8 +38,8 @@
 #include "uxtheme.h"
 #include "vsstyle.h"
 #include "vssym32.h"
-#include "wine/unicode.h"
 #include "wine/debug.h"
+#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(ipaddress);
 
@@ -92,12 +82,12 @@ static void IPADDRESS_UpdateText (const IPADDRESS_INFO *infoPtr)
 
     for (i = 0; i < 4; i++) {
         if (GetWindowTextW (infoPtr->Part[i].EditHwnd, field, 4))
-            strcatW(ip, field);
+            lstrcatW(ip, field);
         else
             /* empty edit treated as zero */
-            strcatW(ip, zero);
+            lstrcatW(ip, zero);
         if (i != 3)
-            strcatW(ip, dot);
+            lstrcatW(ip, dot);
     }
 
     SetWindowTextW(infoPtr->Self, ip);
@@ -228,7 +218,7 @@ static LRESULT IPADDRESS_Create (HWND hwnd, const CREATESTRUCTA *lpCreate)
     SetWindowLongW (hwnd, GWL_STYLE,
 		    GetWindowLongW(hwnd, GWL_STYLE) & ~WS_BORDER);
 
-    infoPtr = Alloc (sizeof(IPADDRESS_INFO));
+    infoPtr = heap_alloc_zero (sizeof(*infoPtr));
     if (!infoPtr) return -1;
     SetWindowLongPtrW (hwnd, 0, (DWORD_PTR)infoPtr);
 
@@ -246,7 +236,7 @@ static LRESULT IPADDRESS_Create (HWND hwnd, const CREATESTRUCTA *lpCreate)
     hSysFont = GetStockObject(ANSI_VAR_FONT);
     GetObjectW(hSysFont, sizeof(LOGFONTW), &logSysFont);
     SystemParametersInfoW(SPI_GETICONTITLELOGFONT, 0, &logFont, 0);
-    strcpyW(logFont.lfFaceName, logSysFont.lfFaceName);
+    lstrcpyW(logFont.lfFaceName, logSysFont.lfFaceName);
     hFont = CreateFontIndirectW(&logFont);
 
     for (i = 0; i < 4; i++) {
@@ -287,7 +277,7 @@ static LRESULT IPADDRESS_Destroy (IPADDRESS_INFO *infoPtr)
     }
 
     SetWindowLongPtrW (infoPtr->Self, 0, 0);
-    Free (infoPtr);
+    heap_free (infoPtr);
     return 0;
 }
 
@@ -345,7 +335,7 @@ static int IPADDRESS_GetAddress (const IPADDRESS_INFO *infoPtr, LPDWORD ip_addre
     for (i = 0; i < 4; i++) {
         ip_addr *= 256;
         if (GetWindowTextW (infoPtr->Part[i].EditHwnd, field, 4))
-  	    ip_addr += atolW(field);
+	    ip_addr += wcstol(field, NULL, 10);
 	else
 	    invalid++;
     }
@@ -427,7 +417,7 @@ static BOOL IPADDRESS_ConstrainField (const IPADDRESS_INFO *infoPtr, int current
     part = &infoPtr->Part[currentfield];
     if (!GetWindowTextW (part->EditHwnd, field, 4)) return FALSE;
 
-    curValue = atoiW(field);
+    curValue = wcstol(field, NULL, 10);
     TRACE("  curValue=%d\n", curValue);
 
     newValue = IPADDRESS_IPNotify(infoPtr, currentfield, curValue);
