@@ -22,6 +22,7 @@
 
 #include "wine/test.h"
 #include "msg.h"
+#include "v6util.h"
 
 #define expect(EXPECTED, GOT) ok((GOT)==(EXPECTED), "Expected %d, got %ld\n", (EXPECTED), (GOT))
 
@@ -30,6 +31,8 @@
 #define NUM_MSG_SEQUENCES   1
 #define DATETIME_SEQ_INDEX    0
 
+static BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
+
 static struct msg_sequence *sequences[NUM_MSG_SEQUENCES];
 
 static const struct message test_dtm_set_format_seq[] = {
@@ -718,6 +721,46 @@ static void test_dtm_set_and_get_systemtime_with_limits(void)
     DestroyWindow(hWnd);
 }
 
+static void test_dtm_get_ideal_size(void)
+{
+    HWND hwnd;
+    HDC hdc;
+    HFONT hfont;
+    LOGFONTA lf;
+    TEXTMETRICA tm;
+    SIZE size;
+    BOOL r;
+
+    hwnd = create_datetime_control(0);
+    r = SendMessageA(hwnd, DTM_GETIDEALSIZE, 0, (LPARAM)&size);
+    if (!r)
+    {
+        win_skip("DTM_GETIDEALSIZE is not available\n");
+        DestroyWindow(hwnd);
+        return;
+    }
+
+    /* Set font so that the test is consistent on Wine and Windows */
+    ZeroMemory(&lf, sizeof(lf));
+    lf.lfWeight = FW_NORMAL;
+    lf.lfHeight = 20;
+    lstrcpyA(lf.lfFaceName, "Tahoma");
+    hfont = CreateFontIndirectA(&lf);
+    SendMessageA(hwnd, WM_SETFONT, (WPARAM)hfont, (LPARAM)TRUE);
+
+    hdc = GetDC(hwnd);
+    GetTextMetricsA(hdc, &tm);
+    ReleaseDC(hwnd, hdc);
+
+    r = SendMessageA(hwnd, DTM_GETIDEALSIZE, 0, (LPARAM)&size);
+    ok(r, "Expect DTM_GETIDEALSIZE message to return true\n");
+    ok(size.cx > 0 && size.cy >= tm.tmHeight,
+       "Expect size.cx > 0 and size.cy >= %d, got cx:%d cy:%d\n", tm.tmHeight, size.cx, size.cy);
+
+    DestroyWindow(hwnd);
+    DeleteObject(hfont);
+}
+
 static void test_wm_set_get_text(void)
 {
     static const CHAR a_str[] = "a";
@@ -777,19 +820,23 @@ static void test_dts_shownone(void)
     DestroyWindow(hwnd);
 }
 
+static void init_functions(void)
+{
+    HMODULE hComCtl32 = LoadLibraryA("comctl32.dll");
+
+#define X(f) p##f = (void*)GetProcAddress(hComCtl32, #f);
+    X(InitCommonControlsEx);
+#undef X
+}
+
 START_TEST(datetime)
 {
-    HMODULE hComctl32;
-    BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
     INITCOMMONCONTROLSEX iccex;
+    ULONG_PTR cookie;
+    HANDLE ctxt;
+
+    init_functions();
 
-    hComctl32 = GetModuleHandleA("comctl32.dll");
-    pInitCommonControlsEx = (void*)GetProcAddress(hComctl32, "InitCommonControlsEx");
-    if (!pInitCommonControlsEx)
-    {
-        win_skip("InitCommonControlsEx() is missing. Skipping the tests\n");
-        return;
-    }
     iccex.dwSize = sizeof(iccex);
     iccex.dwICC  = ICC_DATE_CLASSES;
     pInitCommonControlsEx(&iccex);
@@ -806,4 +853,17 @@ START_TEST(datetime)
     test_dtm_set_and_get_systemtime_with_limits();
     test_wm_set_get_text();
     test_dts_shownone();
+
+    if (!load_v6_module(&cookie, &ctxt))
+        return;
+
+    test_dtm_set_format();
+    test_dtm_set_and_get_mccolor();
+    test_dtm_set_and_get_mcfont();
+    test_dtm_get_monthcal();
+    test_dtm_get_ideal_size();
+    test_wm_set_get_text();
+    test_dts_shownone();
+
+    unload_v6_module(cookie, ctxt);
 }
