@@ -28,8 +28,12 @@
 #include <commctrl.h>
 #include <uxtheme.h>
 
+#include "wine/heap.h"
 #include "wine/test.h"
 
+static BOOL (WINAPI *pImageList_Destroy)(HIMAGELIST);
+static HIMAGELIST (WINAPI *pImageList_LoadImageA)(HINSTANCE, LPCSTR, int, int, COLORREF, UINT, UINT);
+
 static RECT height_change_notify_rect;
 static HWND hMainWnd;
 static int system_font_height;
@@ -217,9 +221,9 @@ static rbsize_result_t rbsize_init(int cleft, int ctop, int cright, int cbottom,
     SetRect(&ret.rcClient, cleft, ctop, cright, cbottom);
     ret.cyBarHeight = cyBarHeight;
     ret.nRows = 0;
-    ret.cyRowHeights = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, nRows*sizeof(int));
+    ret.cyRowHeights = heap_alloc_zero(nRows * sizeof(int));
     ret.nBands = 0;
-    ret.bands = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, nBands*sizeof(rbband_result_t));
+    ret.bands = heap_alloc_zero(nBands * sizeof(*ret.bands));
 
     return ret;
 }
@@ -243,7 +247,7 @@ static rbsize_result_t *rbsize_results;
 
 static void rbsize_results_init(void)
 {
-    rbsize_results = HeapAlloc(GetProcessHeap(), 0, rbsize_results_num*sizeof(rbsize_result_t));
+    rbsize_results = heap_alloc(rbsize_results_num * sizeof(*rbsize_results));
 
     rbsize_results[0] = rbsize_init(0, 0, 672, 0, 0, 0, 0);
 
@@ -430,10 +434,10 @@ static void rbsize_results_free(void)
     int i;
 
     for (i = 0; i < rbsize_results_num; i++) {
-        HeapFree(GetProcessHeap(), 0, rbsize_results[i].cyRowHeights);
-        HeapFree(GetProcessHeap(), 0, rbsize_results[i].bands);
+        heap_free(rbsize_results[i].cyRowHeights);
+        heap_free(rbsize_results[i].bands);
     }
-    HeapFree(GetProcessHeap(), 0, rbsize_results);
+    heap_free(rbsize_results);
     rbsize_results = NULL;
 }
 
@@ -495,6 +499,7 @@ static void test_layout(void)
     REBARBANDINFOA rbi;
     HIMAGELIST himl;
     REBARINFO ri;
+    int count;
 
     rbsize_results_init();
 
@@ -577,7 +582,7 @@ static void test_layout(void)
     check_sizes();
 
     /* an image will increase the band height */
-    himl = ImageList_LoadImageA(GetModuleHandleA("comctl32"), MAKEINTRESOURCEA(121), 24, 2,
+    himl = pImageList_LoadImageA(GetModuleHandleA("comctl32"), MAKEINTRESOURCEA(121), 24, 2,
             CLR_NONE, IMAGE_BITMAP, LR_DEFAULTCOLOR);
     ri.cbSize = sizeof(ri);
     ri.fMask = RBIM_IMAGELIST;
@@ -656,9 +661,27 @@ static void test_layout(void)
     SendMessageA(hRebar, RB_INSERTBANDA, -1, (LPARAM)&rbi);
     check_sizes();
 
-    rbsize_results_free();
     DestroyWindow(hRebar);
-    ImageList_Destroy(himl);
+    pImageList_Destroy(himl);
+
+    /* One hidden band. */
+    hRebar = create_rebar_control();
+
+    rbi.cbSize = REBARBANDINFOA_V6_SIZE;
+    rbi.fMask = RBBIM_STYLE | RBBIM_SIZE | RBBIM_CHILDSIZE | RBBIM_CHILD;
+    rbi.fStyle = RBBS_HIDDEN;
+    rbi.cx = 200;
+    rbi.cxMinChild = 100;
+    rbi.cyMinChild = 30;
+    rbi.hwndChild = NULL;
+
+    SendMessageA(hRebar, RB_INSERTBANDA, -1, (LPARAM)&rbi);
+    count = SendMessageA(hRebar, RB_GETROWCOUNT, 0, 0);
+    ok(!count, "Unexpected row count %d.\n", count);
+
+    DestroyWindow(hRebar);
+
+    rbsize_results_free();
 }
 
 #if 0       /* use this to generate more tests */
@@ -822,7 +845,7 @@ static DWORD resize_numtests = 0;
         RECT r; \
         int value; \
         const rbresize_test_result_t *res = &resize_results[resize_numtests++]; \
-        assert(resize_numtests <= sizeof(resize_results)/sizeof(resize_results[0])); \
+        assert(resize_numtests <= ARRAY_SIZE(resize_results)); \
         GetWindowRect(hRebar, &r); \
         MapWindowPoints(HWND_DESKTOP, hMainWnd, (LPPOINT)&r, 2); \
         if ((dwStyles[i] & (CCS_NOPARENTALIGN|CCS_NODIVIDER)) == CCS_NOPARENTALIGN) {\
@@ -849,7 +872,7 @@ static void test_resize(void)
         CCS_TOP | WS_BORDER, CCS_NOPARENTALIGN | CCS_NODIVIDER | WS_BORDER, CCS_NORESIZE | WS_BORDER,
         CCS_NOMOVEY | WS_BORDER};
 
-    const int styles_count = sizeof(dwStyles) / sizeof(dwStyles[0]);
+    const int styles_count = ARRAY_SIZE(dwStyles);
     int i;
 
     for (i = 0; i < styles_count; i++)
@@ -1125,26 +1148,22 @@ static void test_notification(void)
     DestroyWindow(rebar);
 }
 
+static void init_functions(void)
+{
+    HMODULE hComCtl32 = LoadLibraryA("comctl32.dll");
+
+#define X(f) p##f = (void*)GetProcAddress(hComCtl32, #f);
+    X(ImageList_Destroy);
+    X(ImageList_LoadImageA);
+#undef X
+}
+
 START_TEST(rebar)
 {
-    HMODULE hComctl32;
-    BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
-    INITCOMMONCONTROLSEX iccex;
     MSG msg;
 
     init_system_font_height();
-
-    /* LoadLibrary is needed. This file has no reference to functions in comctl32 */
-    hComctl32 = LoadLibraryA("comctl32.dll");
-    pInitCommonControlsEx = (void*)GetProcAddress(hComctl32, "InitCommonControlsEx");
-    if (!pInitCommonControlsEx)
-    {
-        win_skip("InitCommonControlsEx() is missing. Skipping the tests\n");
-        return;
-    }
-    iccex.dwSize = sizeof(iccex);
-    iccex.dwICC = ICC_COOL_CLASSES;
-    pInitCommonControlsEx(&iccex);
+    init_functions();
 
     hMainWnd = create_parent_window();
 
@@ -1169,6 +1188,4 @@ out:
         DispatchMessageA(&msg);
     }
     DestroyWindow(hMainWnd);
-
-    FreeLibrary(hComctl32);
 }
