@@ -19,9 +19,6 @@
  *
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdarg.h>
 
 #define COBJMACROS
@@ -54,7 +51,7 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
  */
 BOOL WINAPI D3DX10CheckVersion(UINT d3dsdkvers, UINT d3dxsdkvers)
 {
-    if ((d3dsdkvers == D3D10_SDK_VERSION) && (d3dxsdkvers == 38))
+    if ((d3dsdkvers == D3D10_SDK_VERSION) && (d3dxsdkvers == 40))
         return TRUE;
 
     return FALSE;
