@@ -18,8 +18,6 @@
  *
  */
 
-#include "config.h"
-#include "wine/port.h"
 
 #include "d3dx9_private.h"
 
@@ -27,6 +25,8 @@
 #include "ole2.h"
 #include "wincodec.h"
 
+#include "txc_dxtn.h"
+
 WINE_DEFAULT_DEBUG_CHANNEL(d3dx);
 
 HRESULT WINAPI WICCreateImagingFactory_Proxy(UINT, IWICImagingFactory**);
@@ -200,35 +200,47 @@ static const struct {
     { 32, 0x000000ff, 0x0000ff00, 0x00ff0000, 0x00000000, D3DFMT_X8B8G8R8 },
 };
 
-HRESULT lock_surface(IDirect3DSurface9 *surface, D3DLOCKED_RECT *lock,
+HRESULT lock_surface(IDirect3DSurface9 *surface, const RECT *surface_rect, D3DLOCKED_RECT *lock,
         IDirect3DSurface9 **temp_surface, BOOL write)
 {
+    unsigned int width, height;
     IDirect3DDevice9 *device;
     D3DSURFACE_DESC desc;
     DWORD lock_flag;
     HRESULT hr;
 
-    lock_flag = write ? D3DLOCK_DISCARD : D3DLOCK_READONLY;
+    lock_flag = write ? 0 : D3DLOCK_READONLY;
     *temp_surface = NULL;
-    if (FAILED(hr = IDirect3DSurface9_LockRect(surface, lock, NULL, lock_flag)))
+    if (FAILED(hr = IDirect3DSurface9_LockRect(surface, lock, surface_rect, lock_flag)))
     {
         IDirect3DSurface9_GetDevice(surface, &device);
         IDirect3DSurface9_GetDesc(surface, &desc);
 
-        hr = write ? IDirect3DDevice9_CreateOffscreenPlainSurface(device, desc.Width, desc.Height,
+        if (surface_rect)
+        {
+            width = surface_rect->right - surface_rect->left;
+            height = surface_rect->bottom - surface_rect->top;
+        }
+        else
+        {
+            width = desc.Width;
+            height = desc.Height;
+        }
+
+        hr = write ? IDirect3DDevice9_CreateOffscreenPlainSurface(device, width, height,
                 desc.Format, D3DPOOL_SYSTEMMEM, temp_surface, NULL)
-                : IDirect3DDevice9_CreateRenderTarget(device, desc.Width, desc.Height,
+                : IDirect3DDevice9_CreateRenderTarget(device, width, height,
                 desc.Format, D3DMULTISAMPLE_NONE, 0, TRUE, temp_surface, NULL);
         if (FAILED(hr))
         {
             WARN("Failed to create temporary surface, surface %p, format %#x,"
                     " usage %#x, pool %#x, write %#x, width %u, height %u.\n",
-                    surface, desc.Format, desc.Usage, desc.Pool, write, desc.Width, desc.Height);
+                    surface, desc.Format, desc.Usage, desc.Pool, write, width, height);
             IDirect3DDevice9_Release(device);
             return hr;
         }
 
-        if (write || SUCCEEDED(hr = IDirect3DDevice9_StretchRect(device, surface, NULL,
+        if (write || SUCCEEDED(hr = IDirect3DDevice9_StretchRect(device, surface, surface_rect,
                 *temp_surface, NULL, D3DTEXF_NONE)))
             hr = IDirect3DSurface9_LockRect(*temp_surface, lock, NULL, lock_flag);
 
@@ -246,20 +258,34 @@ HRESULT lock_surface(IDirect3DSurface9 *surface, D3DLOCKED_RECT *lock,
     return hr;
 }
 
-HRESULT unlock_surface(IDirect3DSurface9 *surface, D3DLOCKED_RECT *lock,
+HRESULT unlock_surface(IDirect3DSurface9 *surface, const RECT *surface_rect,
         IDirect3DSurface9 *temp_surface, BOOL update)
 {
     IDirect3DDevice9 *device;
+    POINT surface_point;
     HRESULT hr;
 
     if (!temp_surface)
-        return IDirect3DSurface9_UnlockRect(surface);
+    {
+        hr = IDirect3DSurface9_UnlockRect(surface);
+        return hr;
+    }
 
     hr = IDirect3DSurface9_UnlockRect(temp_surface);
     if (update)
     {
+        if (surface_rect)
+        {
+            surface_point.x = surface_rect->left;
+            surface_point.y = surface_rect->top;
+        }
+        else
+        {
+            surface_point.x = 0;
+            surface_point.y = 0;
+        }
         IDirect3DSurface9_GetDevice(surface, &device);
-        if (FAILED(hr = IDirect3DDevice9_UpdateSurface(device, temp_surface, NULL, surface, NULL)))
+        if (FAILED(hr = IDirect3DDevice9_UpdateSurface(device, temp_surface, NULL, surface, &surface_point)))
             WARN("Updating surface failed, hr %#x, surface %p, temp_surface %p.\n",
                     hr, surface, temp_surface);
         IDirect3DDevice9_Release(device);
@@ -571,7 +597,7 @@ static HRESULT save_dds_surface_to_memory(ID3DXBuffer **dst_buffer, IDirect3DSur
         return hr;
     }
 
-    hr = lock_surface(src_surface, &locked_rect, &temp_surface, FALSE);
+    hr = lock_surface(src_surface, NULL, &locked_rect, &temp_surface, FALSE);
     if (FAILED(hr))
     {
         ID3DXBuffer_Release(buffer);
@@ -584,7 +610,7 @@ static HRESULT save_dds_surface_to_memory(ID3DXBuffer **dst_buffer, IDirect3DSur
     copy_pixels(locked_rect.pBits, locked_rect.Pitch, 0, pixels, dst_pitch, 0,
         &volume, pixel_format);
 
-    unlock_surface(src_surface, &locked_rect, temp_surface, FALSE);
+    unlock_surface(src_surface, NULL, temp_surface, FALSE);
 
     *dst_buffer = buffer;
     return D3D_OK;
@@ -1142,7 +1168,7 @@ HRESULT WINAPI D3DXLoadSurfaceFromFileInMemory(IDirect3DSurface9 *pDestSurface,
         wicrect.Height = imginfo.Height;
     }
 
-    SetRect(&rect, 0, 0, wicrect.Width, wicrect.Height);
+    SetRect(&rect, wicrect.X, wicrect.Y, wicrect.X + wicrect.Width, wicrect.Y + wicrect.Height);
 
     if (imginfo.ImageFileFormat == D3DXIFF_DDS)
     {
@@ -1595,6 +1621,11 @@ void convert_argb_pixels(const BYTE *src, UINT src_row_pitch, UINT src_slice_pit
     UINT min_width, min_height, min_depth;
     UINT x, y, z;
 
+    TRACE("src %p, src_row_pitch %u, src_slice_pitch %u, src_size %p, src_format %p, dst %p, "
+            "dst_row_pitch %u, dst_slice_pitch %u, dst_size %p, dst_format %p, color_key 0x%08x, palette %p.\n",
+            src, src_row_pitch, src_slice_pitch, src_size, src_format, dst, dst_row_pitch, dst_slice_pitch, dst_size,
+            dst_format, color_key, palette);
+
     ZeroMemory(channels, sizeof(channels));
     init_argb_conversion_info(src_format, dst_format, &conv_info);
 
@@ -1697,6 +1728,11 @@ void point_filter_argb_pixels(const BYTE *src, UINT src_row_pitch, UINT src_slic
     DWORD channels[4];
     UINT x, y, z;
 
+    TRACE("src %p, src_row_pitch %u, src_slice_pitch %u, src_size %p, src_format %p, dst %p, "
+            "dst_row_pitch %u, dst_slice_pitch %u, dst_size %p, dst_format %p, color_key 0x%08x, palette %p.\n",
+            src, src_row_pitch, src_slice_pitch, src_size, src_format, dst, dst_row_pitch, dst_slice_pitch, dst_size,
+            dst_format, color_key, palette);
+
     ZeroMemory(channels, sizeof(channels));
     init_argb_conversion_info(src_format, dst_format, &conv_info);
 
@@ -1812,10 +1848,11 @@ HRESULT WINAPI D3DXLoadSurfaceFromMemory(IDirect3DSurface9 *dst_surface,
         DWORD filter, D3DCOLOR color_key)
 {
     const struct pixel_format_desc *srcformatdesc, *destformatdesc;
+    struct volume src_size, dst_size, dst_size_aligned;
+    RECT dst_rect_temp, dst_rect_aligned;
     IDirect3DSurface9 *surface;
     D3DSURFACE_DESC surfdesc;
     D3DLOCKED_RECT lockrect;
-    struct volume src_size, dst_size;
     HRESULT hr;
 
     TRACE("(%p, %p, %s, %p, %#x, %u, %p, %s, %#x, 0x%08x)\n",
@@ -1835,18 +1872,26 @@ HRESULT WINAPI D3DXLoadSurfaceFromMemory(IDirect3DSurface9 *dst_surface,
         return E_FAIL;
     }
 
-    if (filter == D3DX_DEFAULT)
-        filter = D3DX_FILTER_TRIANGLE | D3DX_FILTER_DITHER;
-
-    IDirect3DSurface9_GetDesc(dst_surface, &surfdesc);
+    srcformatdesc = get_format_info(src_format);
+    if (srcformatdesc->type == FORMAT_UNKNOWN)
+    {
+        FIXME("Unsupported format %#x.\n", src_format);
+        return E_NOTIMPL;
+    }
 
     src_size.width = src_rect->right - src_rect->left;
     src_size.height = src_rect->bottom - src_rect->top;
     src_size.depth = 1;
+
+    IDirect3DSurface9_GetDesc(dst_surface, &surfdesc);
+    destformatdesc = get_format_info(surfdesc.Format);
     if (!dst_rect)
     {
-        dst_size.width = surfdesc.Width;
-        dst_size.height = surfdesc.Height;
+        dst_rect = &dst_rect_temp;
+        dst_rect_temp.left = 0;
+        dst_rect_temp.top = 0;
+        dst_rect_temp.right = surfdesc.Width;
+        dst_rect_temp.bottom = surfdesc.Height;
     }
     else
     {
@@ -1857,58 +1902,149 @@ HRESULT WINAPI D3DXLoadSurfaceFromMemory(IDirect3DSurface9 *dst_surface,
             WARN("Invalid dst_rect specified.\n");
             return D3DERR_INVALIDCALL;
         }
-        dst_size.width = dst_rect->right - dst_rect->left;
-        dst_size.height = dst_rect->bottom - dst_rect->top;
-        if (!dst_size.width || !dst_size.height)
+        if (dst_rect->left == dst_rect->right || dst_rect->top == dst_rect->bottom)
+        {
+            WARN("Empty dst_rect specified.\n");
             return D3D_OK;
+        }
     }
+
+    dst_rect_aligned = *dst_rect;
+    if (dst_rect_aligned.left & (destformatdesc->block_width - 1))
+        dst_rect_aligned.left = dst_rect_aligned.left & ~(destformatdesc->block_width - 1);
+    if (dst_rect_aligned.top & (destformatdesc->block_height - 1))
+        dst_rect_aligned.top = dst_rect_aligned.top & ~(destformatdesc->block_height - 1);
+    if (dst_rect_aligned.right & (destformatdesc->block_width - 1) && dst_rect_aligned.right != surfdesc.Width)
+        dst_rect_aligned.right = min((dst_rect_aligned.right + destformatdesc->block_width - 1)
+                & ~(destformatdesc->block_width - 1), surfdesc.Width);
+    if (dst_rect_aligned.bottom & (destformatdesc->block_height - 1) && dst_rect_aligned.bottom != surfdesc.Height)
+        dst_rect_aligned.bottom = min((dst_rect_aligned.bottom + destformatdesc->block_height - 1)
+                & ~(destformatdesc->block_height - 1), surfdesc.Height);
+
+    dst_size.width = dst_rect->right - dst_rect->left;
+    dst_size.height = dst_rect->bottom - dst_rect->top;
     dst_size.depth = 1;
+    dst_size_aligned.width = dst_rect_aligned.right - dst_rect_aligned.left;
+    dst_size_aligned.height = dst_rect_aligned.bottom - dst_rect_aligned.top;
+    dst_size_aligned.depth = 1;
 
-    srcformatdesc = get_format_info(src_format);
-    destformatdesc = get_format_info(surfdesc.Format);
-    if (srcformatdesc->type == FORMAT_UNKNOWN || destformatdesc->type == FORMAT_UNKNOWN)
-    {
-        FIXME("Unsupported pixel format conversion %#x -> %#x\n", src_format, surfdesc.Format);
-        return E_NOTIMPL;
-    }
+    if (filter == D3DX_DEFAULT)
+        filter = D3DX_FILTER_TRIANGLE | D3DX_FILTER_DITHER;
 
-    if (FAILED(hr = lock_surface(dst_surface, &lockrect, &surface, TRUE)))
+    if (FAILED(hr = lock_surface(dst_surface, &dst_rect_aligned, &lockrect, &surface, TRUE)))
         return hr;
 
+    src_memory = (BYTE *)src_memory + src_rect->top / srcformatdesc->block_height * src_pitch
+            + src_rect->left / srcformatdesc->block_width * srcformatdesc->block_byte_count;
+
     if (src_format == surfdesc.Format
             && dst_size.width == src_size.width
             && dst_size.height == src_size.height
-            && color_key == 0) /* Simple copy. */
+            && color_key == 0
+            && !(src_rect->left & (srcformatdesc->block_width - 1))
+            && !(src_rect->top & (srcformatdesc->block_height - 1))
+            && !(dst_rect->left & (destformatdesc->block_width - 1))
+            && !(dst_rect->top & (destformatdesc->block_height - 1)))
     {
-        if (src_rect->left & (srcformatdesc->block_width - 1)
-                || src_rect->top & (srcformatdesc->block_height - 1)
-                || (src_rect->right & (srcformatdesc->block_width - 1)
-                    && src_size.width != surfdesc.Width)
-                || (src_rect->bottom & (srcformatdesc->block_height - 1)
-                    && src_size.height != surfdesc.Height))
-        {
-            WARN("Source rect %s is misaligned.\n", wine_dbgstr_rect(src_rect));
-            unlock_surface(dst_surface, &lockrect, surface, FALSE);
-            return D3DXERR_INVALIDDATA;
-        }
-
+        TRACE("Simple copy.\n");
         copy_pixels(src_memory, src_pitch, 0, lockrect.pBits, lockrect.Pitch, 0,
                 &src_size, srcformatdesc);
     }
     else /* Stretching or format conversion. */
     {
+        const struct pixel_format_desc *dst_format;
+        DWORD *src_uncompressed = NULL;
+        BYTE *dst_uncompressed = NULL;
+        unsigned int dst_pitch;
+        BYTE *dst_mem;
+
         if (!is_conversion_from_supported(srcformatdesc)
                 || !is_conversion_to_supported(destformatdesc))
         {
             FIXME("Unsupported format conversion %#x -> %#x.\n", src_format, surfdesc.Format);
-            unlock_surface(dst_surface, &lockrect, surface, FALSE);
+            unlock_surface(dst_surface, &dst_rect_aligned, surface, FALSE);
             return E_NOTIMPL;
         }
 
+        if (srcformatdesc->type == FORMAT_DXT)
+        {
+            void (*fetch_dxt_texel)(int srcRowStride, const BYTE *pixdata,
+                    int i, int j, void *texel);
+            unsigned int x, y;
+
+            src_pitch = src_pitch * srcformatdesc->block_width / srcformatdesc->block_byte_count;
+
+            src_uncompressed = heap_alloc(src_size.width * src_size.height * sizeof(DWORD));
+            if (!src_uncompressed)
+            {
+                unlock_surface(dst_surface, &dst_rect_aligned, surface, FALSE);
+                return E_OUTOFMEMORY;
+            }
+
+            switch(src_format)
+            {
+                case D3DFMT_DXT1:
+                    fetch_dxt_texel = fetch_2d_texel_rgba_dxt1;
+                    break;
+                case D3DFMT_DXT2:
+                case D3DFMT_DXT3:
+                    fetch_dxt_texel = fetch_2d_texel_rgba_dxt3;
+                    break;
+                case D3DFMT_DXT4:
+                case D3DFMT_DXT5:
+                    fetch_dxt_texel = fetch_2d_texel_rgba_dxt5;
+                    break;
+                default:
+                    FIXME("Unexpected compressed texture format %u.\n", src_format);
+                    fetch_dxt_texel = NULL;
+            }
+
+            TRACE("Uncompressing DXTn surface.\n");
+            for (y = 0; y < src_size.height; ++y)
+            {
+                DWORD *ptr = &src_uncompressed[y * src_size.width];
+                for (x = 0; x < src_size.width; ++x)
+                {
+                    fetch_dxt_texel(src_pitch, src_memory, x + src_rect->left, y + src_rect->top, ptr);
+                    ++ptr;
+                }
+            }
+            src_memory = src_uncompressed;
+            src_pitch = src_size.width * sizeof(DWORD);
+            srcformatdesc = get_format_info(D3DFMT_A8B8G8R8);
+        }
+
+        if (destformatdesc->type == FORMAT_DXT)
+        {
+            BOOL dst_misaligned = dst_rect->left != dst_rect_aligned.left
+                    || dst_rect->top != dst_rect_aligned.top
+                    || dst_rect->right != dst_rect_aligned.right
+                    || dst_rect->bottom != dst_rect_aligned.bottom;
+
+            dst_uncompressed = HeapAlloc(GetProcessHeap(), dst_misaligned ? HEAP_ZERO_MEMORY : 0,
+                    dst_size_aligned.width * dst_size_aligned.height * sizeof(DWORD));
+            if (!dst_uncompressed)
+            {
+                heap_free(src_uncompressed);
+                unlock_surface(dst_surface, &dst_rect_aligned, surface, FALSE);
+                return E_OUTOFMEMORY;
+            }
+            dst_pitch = dst_size_aligned.width * sizeof(DWORD);
+            dst_format = get_format_info(D3DFMT_A8B8G8R8);
+            dst_mem = dst_uncompressed + (dst_rect->top - dst_rect_aligned.top) * dst_pitch
+                    + (dst_rect->left - dst_rect_aligned.left) * sizeof(DWORD);
+        }
+        else
+        {
+            dst_mem = lockrect.pBits;
+            dst_pitch = lockrect.Pitch;
+            dst_format = destformatdesc;
+        }
+
         if ((filter & 0xf) == D3DX_FILTER_NONE)
         {
             convert_argb_pixels(src_memory, src_pitch, 0, &src_size, srcformatdesc,
-                    lockrect.pBits, lockrect.Pitch, 0, &dst_size, destformatdesc, color_key, src_palette);
+                    dst_mem, dst_pitch, 0, &dst_size, dst_format, color_key, src_palette);
         }
         else /* if ((filter & 0xf) == D3DX_FILTER_POINT) */
         {
@@ -1918,11 +2054,40 @@ HRESULT WINAPI D3DXLoadSurfaceFromMemory(IDirect3DSurface9 *dst_surface,
             /* Always apply a point filter until D3DX_FILTER_LINEAR,
              * D3DX_FILTER_TRIANGLE and D3DX_FILTER_BOX are implemented. */
             point_filter_argb_pixels(src_memory, src_pitch, 0, &src_size, srcformatdesc,
-                    lockrect.pBits, lockrect.Pitch, 0, &dst_size, destformatdesc, color_key, src_palette);
+                    dst_mem, dst_pitch, 0, &dst_size, dst_format, color_key, src_palette);
+        }
+
+        heap_free(src_uncompressed);
+
+        if (dst_uncompressed)
+        {
+            GLenum gl_format = 0;
+
+            TRACE("Compressing DXTn surface.\n");
+            switch(surfdesc.Format)
+            {
+                case D3DFMT_DXT1:
+                    gl_format = GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
+                    break;
+                case D3DFMT_DXT2:
+                case D3DFMT_DXT3:
+                    gl_format = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
+                    break;
+                case D3DFMT_DXT4:
+                case D3DFMT_DXT5:
+                    gl_format = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
+                    break;
+                default:
+                    ERR("Unexpected destination compressed format %u.\n", surfdesc.Format);
+            }
+            tx_compress_dxtn(4, dst_size_aligned.width, dst_size_aligned.height,
+                    dst_uncompressed, gl_format, lockrect.pBits,
+                    lockrect.Pitch * destformatdesc->block_width / destformatdesc->block_byte_count);
+            heap_free(dst_uncompressed);
         }
     }
 
-    return unlock_surface(dst_surface, &lockrect, surface, TRUE);
+    return unlock_surface(dst_surface, &dst_rect_aligned, surface, TRUE);
 }
 
 /************************************************************
@@ -2006,13 +2171,13 @@ HRESULT WINAPI D3DXLoadSurfaceFromSurface(IDirect3DSurface9 *dst_surface,
         src_rect = &s;
     }
 
-    if (FAILED(lock_surface(src_surface, &lock, &temp_surface, FALSE)))
+    if (FAILED(lock_surface(src_surface, NULL, &lock, &temp_surface, FALSE)))
         return D3DXERR_INVALIDDATA;
 
     hr = D3DXLoadSurfaceFromMemory(dst_surface, dst_palette, dst_rect, lock.pBits,
             src_desc.Format, lock.Pitch, src_palette, src_rect, filter, color_key);
 
-    if (FAILED(unlock_surface(src_surface, &lock, temp_surface, FALSE)))
+    if (FAILED(unlock_surface(src_surface, NULL, temp_surface, FALSE)))
         return D3DXERR_INVALIDDATA;
 
     return hr;
@@ -2188,12 +2353,12 @@ HRESULT WINAPI D3DXSaveSurfaceToFileInMemory(ID3DXBuffer **dst_buffer, D3DXIMAGE
         TRACE("Using pixel format %s %#x\n", debugstr_guid(&wic_pixel_format), d3d_pixel_format);
         if (src_surface_desc.Format == d3d_pixel_format) /* Simple copy */
         {
-            if (FAILED(hr = lock_surface(src_surface, &locked_rect, &temp_surface, FALSE)))
+            if (FAILED(hr = lock_surface(src_surface, src_rect, &locked_rect, &temp_surface, FALSE)))
                 goto cleanup;
 
             IWICBitmapFrameEncode_WritePixels(frame, height,
                 locked_rect.Pitch, height * locked_rect.Pitch, locked_rect.pBits);
-            unlock_surface(src_surface, &locked_rect, temp_surface, FALSE);
+            unlock_surface(src_surface, src_rect, temp_surface, FALSE);
         }
         else /* Pixel format conversion */
         {
@@ -2223,14 +2388,14 @@ HRESULT WINAPI D3DXSaveSurfaceToFileInMemory(ID3DXBuffer **dst_buffer, D3DXIMAGE
                 hr = E_OUTOFMEMORY;
                 goto cleanup;
             }
-            if (FAILED(hr = lock_surface(src_surface, &locked_rect, &temp_surface, FALSE)))
+            if (FAILED(hr = lock_surface(src_surface, src_rect, &locked_rect, &temp_surface, FALSE)))
             {
                 HeapFree(GetProcessHeap(), 0, dst_data);
                 goto cleanup;
             }
             convert_argb_pixels(locked_rect.pBits, locked_rect.Pitch, 0, &size, src_format_desc,
                 dst_data, dst_pitch, 0, &size, dst_format_desc, 0, NULL);
-            unlock_surface(src_surface, &locked_rect, temp_surface, FALSE);
+            unlock_surface(src_surface, src_rect, temp_surface, FALSE);
 
             IWICBitmapFrameEncode_WritePixels(frame, height, dst_pitch, dst_pitch * height, dst_data);
             HeapFree(GetProcessHeap(), 0, dst_data);
