@@ -114,8 +114,7 @@ void*    hash_table_iter_up(struct hash_table_iter* hti) DECLSPEC_HIDDEN;
 
 
 extern unsigned dbghelp_options DECLSPEC_HIDDEN;
-/* some more Wine extensions */
-#define SYMOPT_WINE_WITH_NATIVE_MODULES 0x40000000
+extern BOOL     dbghelp_opt_native DECLSPEC_HIDDEN;
 
 enum location_kind {loc_error,          /* reg is the error code */
                     loc_unavailable,    /* location is not available */
@@ -471,6 +470,12 @@ struct dump_memory
     ULONG                               rva;
 };
 
+struct dump_memory64
+{
+    ULONG64                             base;
+    ULONG64                             size;
+};
+
 struct dump_module
 {
     unsigned                            is_elf;
@@ -509,6 +514,9 @@ struct dump_context
     struct dump_memory*                 mem;
     unsigned                            num_mem;
     unsigned                            alloc_mem;
+    struct dump_memory64*               mem64;
+    unsigned                            num_mem64;
+    unsigned                            alloc_mem64;
     /* callback information */
     MINIDUMP_CALLBACK_INFORMATION*      cb;
 };
@@ -579,8 +587,6 @@ extern BOOL         pcs_callback(const struct process* pcs, ULONG action, void*
 extern void*        fetch_buffer(struct process* pcs, unsigned size) DECLSPEC_HIDDEN;
 extern const char*  wine_dbgstr_addr(const ADDRESS64* addr) DECLSPEC_HIDDEN;
 extern struct cpu*  cpu_find(DWORD) DECLSPEC_HIDDEN;
-
-/* crc32.c */
 extern DWORD calc_crc32(int fd) DECLSPEC_HIDDEN;
 
 typedef BOOL (*enum_modules_cb)(const WCHAR*, unsigned long addr, void* user);
