@@ -33,7 +33,6 @@ typedef struct IDirectMusicTempoTrack {
     IDirectMusicTrack8 IDirectMusicTrack8_iface;
     struct dmobject dmobj;  /* IPersistStream only */
     LONG ref;
-    BOOL enabled;
     struct list Items;
 } IDirectMusicTempoTrack;
 
@@ -154,63 +153,72 @@ static HRESULT WINAPI tempo_track_Play(IDirectMusicTrack8 *iface, void *pStateDa
   return S_OK;
 }
 
-static HRESULT WINAPI tempo_track_GetParam(IDirectMusicTrack8 *iface, REFGUID rguidType,
-        MUSIC_TIME mtTime, MUSIC_TIME *pmtNext, void *pParam)
+static HRESULT WINAPI tempo_track_GetParam(IDirectMusicTrack8 *iface, REFGUID type, MUSIC_TIME time,
+        MUSIC_TIME *next, void *param)
 {
-  IDirectMusicTempoTrack *This = impl_from_IDirectMusicTrack8(iface);
+    IDirectMusicTempoTrack *This = impl_from_IDirectMusicTrack8(iface);
+    DMUS_PRIVATE_TEMPO_ITEM *item = NULL;
+    DMUS_TEMPO_PARAM *prm = param;
 
-  HRESULT hr = S_OK;
-  struct list* pEntry = NULL;
-  LPDMUS_PRIVATE_TEMPO_ITEM pIt = NULL;
-  DMUS_TEMPO_PARAM* prm = pParam;
+    TRACE("(%p, %s, %d, %p, %p)\n", This, debugstr_dmguid(type), time, next, param);
 
-  FIXME("(%p, %s, %d, %p, %p): almost stub\n", This, debugstr_dmguid(rguidType), mtTime, pmtNext, pParam);
+    if (!param)
+        return E_POINTER;
+    if (!IsEqualGUID(type, &GUID_TempoParam))
+        return DMUS_E_GET_UNSUPPORTED;
 
-  if (NULL == pParam) {
-    return E_POINTER;
-  }
+    FIXME("Partial support for GUID_TempoParam\n");
 
-  hr = IDirectMusicTrack_IsParamSupported (iface, rguidType);
-  if (FAILED(hr)) {
-    return hr;
-  }
-  if (FALSE == This->enabled) {
-    return DMUS_E_TYPE_DISABLED;
-  }
+    if (next)
+        *next = 0;
+    prm->mtTime = 0;
+    prm->dblTempo = 0.123456;
 
-  if (NULL != pmtNext) *pmtNext = 0;
-  prm->mtTime = 0;
-  prm->dblTempo = 0.123456;
-
-  LIST_FOR_EACH (pEntry, &This->Items) {
-    pIt = LIST_ENTRY(pEntry, DMUS_PRIVATE_TEMPO_ITEM, entry);
-    /*TRACE(" - %p -> 0x%lx,%p\n", pIt, pIt->item.lTime, pIt->item.dblTempo);*/
-    if (pIt->item.lTime <= mtTime) {
-      MUSIC_TIME ofs = pIt->item.lTime - mtTime;
-      if (ofs > prm->mtTime) {
-	prm->mtTime = ofs;
-	prm->dblTempo = pIt->item.dblTempo;
-      }	
-      if (NULL != pmtNext && pIt->item.lTime > mtTime) {
-	if (pIt->item.lTime < *pmtNext) {
-	  *pmtNext = pIt->item.lTime;
-	}
-      }
+    LIST_FOR_EACH_ENTRY(item, &This->Items, DMUS_PRIVATE_TEMPO_ITEM, entry) {
+        if (item->item.lTime <= time) {
+            MUSIC_TIME ofs = item->item.lTime - time;
+            if (ofs > prm->mtTime) {
+                prm->mtTime = ofs;
+                prm->dblTempo = item->item.dblTempo;
+            }
+            if (next && item->item.lTime > time && item->item.lTime < *next)
+                *next = item->item.lTime;
+        }
     }
-  }
-  
-  if (0.123456 == prm->dblTempo) {
-    return DMUS_E_NOT_FOUND;
-  }
-  return S_OK;
+
+    if (0.123456 == prm->dblTempo)
+        return DMUS_E_NOT_FOUND;
+
+    return S_OK;
 }
 
-static HRESULT WINAPI tempo_track_SetParam(IDirectMusicTrack8 *iface, REFGUID rguidType,
-        MUSIC_TIME mtTime, void *pParam)
+static HRESULT WINAPI tempo_track_SetParam(IDirectMusicTrack8 *iface, REFGUID type, MUSIC_TIME time,
+        void *param)
 {
-        IDirectMusicTempoTrack *This = impl_from_IDirectMusicTrack8(iface);
-	FIXME("(%p, %s, %d, %p): stub\n", This, debugstr_dmguid(rguidType), mtTime, pParam);
-	return S_OK;
+    IDirectMusicTempoTrack *This = impl_from_IDirectMusicTrack8(iface);
+
+    TRACE("(%p, %s, %d, %p)\n", This, debugstr_dmguid(type), time, param);
+
+    if (IsEqualGUID(type, &GUID_DisableTempo)) {
+        if (!param)
+            return DMUS_E_TYPE_DISABLED;
+        FIXME("GUID_DisableTempo not handled yet\n");
+        return S_OK;
+    }
+    if (IsEqualGUID(type, &GUID_EnableTempo)) {
+        if (!param)
+            return DMUS_E_TYPE_DISABLED;
+        FIXME("GUID_EnableTempo not handled yet\n");
+        return S_OK;
+    }
+    if (IsEqualGUID(type, &GUID_TempoParam)) {
+        if (!param)
+            return E_POINTER;
+        FIXME("GUID_TempoParam not handled yet\n");
+        return S_OK;
+    }
+
+    return DMUS_E_SET_UNSUPPORTED;
 }
 
 static HRESULT WINAPI tempo_track_IsParamSupported(IDirectMusicTrack8 *iface, REFGUID rguidType)
@@ -224,9 +232,6 @@ static HRESULT WINAPI tempo_track_IsParamSupported(IDirectMusicTrack8 *iface, RE
     TRACE("param supported\n");
     return S_OK;
   }
-  if (FALSE == This->enabled) {
-    return DMUS_E_TYPE_DISABLED;
-  }
   TRACE("param unsupported\n");
   return DMUS_E_TYPE_UNSUPPORTED;
 }
@@ -405,7 +410,6 @@ HRESULT WINAPI create_dmtempotrack(REFIID lpcGUID, void **ppobj)
     dmobject_init(&track->dmobj, &CLSID_DirectMusicTempoTrack,
                   (IUnknown *)&track->IDirectMusicTrack8_iface);
     track->dmobj.IPersistStream_iface.lpVtbl = &persiststream_vtbl;
-    track->enabled = TRUE;
     list_init(&track->Items);
 
     DMIME_LockModule();
