@@ -34,13 +34,13 @@
 
 #include "wine/debug.h"
 #include "wine/list.h"
-#include "wine/unicode.h"
 #include "winreg.h"
 #include "objbase.h"
 
 #include "dmusici.h"
 #include "dmusicf.h"
 #include "dmusics.h"
+#include "dmobject.h"
 
 #define ICOM_THIS_MULTI(impl,field,iface) impl* const This=(impl*)((char*)(iface) - offsetof(impl,field))
 
