@@ -25,22 +25,257 @@
 #include "objbase.h"
 #include "dmusici.h"
 #include "dmusicf.h"
+#include "dmusics.h"
 #include "dmobject.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(dmobj);
 WINE_DECLARE_DEBUG_CHANNEL(dmfile);
 
+/* Debugging helpers */
+const char *debugstr_dmguid(const GUID *id) {
+    unsigned int i;
+#define X(guid) { &guid, #guid }
+    static const struct {
+        const GUID *guid;
+        const char *name;
+    } guids[] = {
+        /* CLSIDs */
+        X(CLSID_AudioVBScript),
+        X(CLSID_DirectMusic),
+        X(CLSID_DirectMusicAudioPathConfig),
+        X(CLSID_DirectMusicAuditionTrack),
+        X(CLSID_DirectMusicBand),
+        X(CLSID_DirectMusicBandTrack),
+        X(CLSID_DirectMusicChordMapTrack),
+        X(CLSID_DirectMusicChordMap),
+        X(CLSID_DirectMusicChordTrack),
+        X(CLSID_DirectMusicCollection),
+        X(CLSID_DirectMusicCommandTrack),
+        X(CLSID_DirectMusicComposer),
+        X(CLSID_DirectMusicContainer),
+        X(CLSID_DirectMusicGraph),
+        X(CLSID_DirectMusicLoader),
+        X(CLSID_DirectMusicLyricsTrack),
+        X(CLSID_DirectMusicMarkerTrack),
+        X(CLSID_DirectMusicMelodyFormulationTrack),
+        X(CLSID_DirectMusicMotifTrack),
+        X(CLSID_DirectMusicMuteTrack),
+        X(CLSID_DirectMusicParamControlTrack),
+        X(CLSID_DirectMusicPatternTrack),
+        X(CLSID_DirectMusicPerformance),
+        X(CLSID_DirectMusicScript),
+        X(CLSID_DirectMusicScriptAutoImpSegment),
+        X(CLSID_DirectMusicScriptAutoImpPerformance),
+        X(CLSID_DirectMusicScriptAutoImpSegmentState),
+        X(CLSID_DirectMusicScriptAutoImpAudioPathConfig),
+        X(CLSID_DirectMusicScriptAutoImpAudioPath),
+        X(CLSID_DirectMusicScriptAutoImpSong),
+        X(CLSID_DirectMusicScriptSourceCodeLoader),
+        X(CLSID_DirectMusicScriptTrack),
+        X(CLSID_DirectMusicSection),
+        X(CLSID_DirectMusicSegment),
+        X(CLSID_DirectMusicSegmentState),
+        X(CLSID_DirectMusicSegmentTriggerTrack),
+        X(CLSID_DirectMusicSegTriggerTrack),
+        X(CLSID_DirectMusicSeqTrack),
+        X(CLSID_DirectMusicSignPostTrack),
+        X(CLSID_DirectMusicSong),
+        X(CLSID_DirectMusicStyle),
+        X(CLSID_DirectMusicStyleTrack),
+        X(CLSID_DirectMusicSynth),
+        X(CLSID_DirectMusicSynthSink),
+        X(CLSID_DirectMusicSysExTrack),
+        X(CLSID_DirectMusicTemplate),
+        X(CLSID_DirectMusicTempoTrack),
+        X(CLSID_DirectMusicTimeSigTrack),
+        X(CLSID_DirectMusicWaveTrack),
+        X(CLSID_DirectSoundWave),
+        /* IIDs */
+        X(IID_IDirectMusic),
+        X(IID_IDirectMusic2),
+        X(IID_IDirectMusic8),
+        X(IID_IDirectMusicAudioPath),
+        X(IID_IDirectMusicBand),
+        X(IID_IDirectMusicBuffer),
+        X(IID_IDirectMusicChordMap),
+        X(IID_IDirectMusicCollection),
+        X(IID_IDirectMusicComposer),
+        X(IID_IDirectMusicContainer),
+        X(IID_IDirectMusicDownload),
+        X(IID_IDirectMusicDownloadedInstrument),
+        X(IID_IDirectMusicGetLoader),
+        X(IID_IDirectMusicGraph),
+        X(IID_IDirectMusicInstrument),
+        X(IID_IDirectMusicLoader),
+        X(IID_IDirectMusicLoader8),
+        X(IID_IDirectMusicObject),
+        X(IID_IDirectMusicPatternTrack),
+        X(IID_IDirectMusicPerformance),
+        X(IID_IDirectMusicPerformance2),
+        X(IID_IDirectMusicPerformance8),
+        X(IID_IDirectMusicPort),
+        X(IID_IDirectMusicPortDownload),
+        X(IID_IDirectMusicScript),
+        X(IID_IDirectMusicSegment),
+        X(IID_IDirectMusicSegment2),
+        X(IID_IDirectMusicSegment8),
+        X(IID_IDirectMusicSegmentState),
+        X(IID_IDirectMusicSegmentState8),
+        X(IID_IDirectMusicStyle),
+        X(IID_IDirectMusicStyle8),
+        X(IID_IDirectMusicSynth),
+        X(IID_IDirectMusicSynth8),
+        X(IID_IDirectMusicSynthSink),
+        X(IID_IDirectMusicThru),
+        X(IID_IDirectMusicTool),
+        X(IID_IDirectMusicTool8),
+        X(IID_IDirectMusicTrack),
+        X(IID_IDirectMusicTrack8),
+        X(IID_IUnknown),
+        X(IID_IPersistStream),
+        X(IID_IStream),
+        X(IID_IClassFactory),
+        /* GUIDs */
+        X(GUID_DirectMusicAllTypes),
+        X(GUID_NOTIFICATION_CHORD),
+        X(GUID_NOTIFICATION_COMMAND),
+        X(GUID_NOTIFICATION_MEASUREANDBEAT),
+        X(GUID_NOTIFICATION_PERFORMANCE),
+        X(GUID_NOTIFICATION_RECOMPOSE),
+        X(GUID_NOTIFICATION_SEGMENT),
+        X(GUID_BandParam),
+        X(GUID_ChordParam),
+        X(GUID_CommandParam),
+        X(GUID_CommandParam2),
+        X(GUID_CommandParamNext),
+        X(GUID_IDirectMusicBand),
+        X(GUID_IDirectMusicChordMap),
+        X(GUID_IDirectMusicStyle),
+        X(GUID_MuteParam),
+        X(GUID_Play_Marker),
+        X(GUID_RhythmParam),
+        X(GUID_TempoParam),
+        X(GUID_TimeSignature),
+        X(GUID_Valid_Start_Time),
+        X(GUID_Clear_All_Bands),
+        X(GUID_ConnectToDLSCollection),
+        X(GUID_Disable_Auto_Download),
+        X(GUID_DisableTempo),
+        X(GUID_DisableTimeSig),
+        X(GUID_Download),
+        X(GUID_DownloadToAudioPath),
+        X(GUID_Enable_Auto_Download),
+        X(GUID_EnableTempo),
+        X(GUID_EnableTimeSig),
+        X(GUID_IgnoreBankSelectForGM),
+        X(GUID_SeedVariations),
+        X(GUID_StandardMIDIFile),
+        X(GUID_Unload),
+        X(GUID_UnloadFromAudioPath),
+        X(GUID_Variations),
+        X(GUID_PerfMasterTempo),
+        X(GUID_PerfMasterVolume),
+        X(GUID_PerfMasterGrooveLevel),
+        X(GUID_PerfAutoDownload),
+        X(GUID_DefaultGMCollection),
+        X(GUID_Synth_Default),
+        X(GUID_Buffer_Reverb),
+        X(GUID_Buffer_EnvReverb),
+        X(GUID_Buffer_Stereo),
+        X(GUID_Buffer_3D_Dry),
+        X(GUID_Buffer_Mono),
+        X(GUID_DMUS_PROP_GM_Hardware),
+        X(GUID_DMUS_PROP_GS_Capable),
+        X(GUID_DMUS_PROP_GS_Hardware),
+        X(GUID_DMUS_PROP_DLS1),
+        X(GUID_DMUS_PROP_DLS2),
+        X(GUID_DMUS_PROP_Effects),
+        X(GUID_DMUS_PROP_INSTRUMENT2),
+        X(GUID_DMUS_PROP_LegacyCaps),
+        X(GUID_DMUS_PROP_MemorySize),
+        X(GUID_DMUS_PROP_SampleMemorySize),
+        X(GUID_DMUS_PROP_SamplePlaybackRate),
+        X(GUID_DMUS_PROP_SetSynthSink),
+        X(GUID_DMUS_PROP_SinkUsesDSound),
+        X(GUID_DMUS_PROP_SynthSink_DSOUND),
+        X(GUID_DMUS_PROP_SynthSink_WAVE),
+        X(GUID_DMUS_PROP_Volume),
+        X(GUID_DMUS_PROP_WavesReverb),
+        X(GUID_DMUS_PROP_WriteLatency),
+        X(GUID_DMUS_PROP_WritePeriod),
+        X(GUID_DMUS_PROP_XG_Capable),
+        X(GUID_DMUS_PROP_XG_Hardware)
+    };
+#undef X
+
+    if (!id)
+        return "(null)";
+
+    for (i = 0; i < ARRAY_SIZE(guids); i++)
+        if (IsEqualGUID(id, guids[i].guid))
+            return guids[i].name;
+
+    return debugstr_guid(id);
+}
+
+void dump_DMUS_OBJECTDESC(DMUS_OBJECTDESC *desc)
+{
+    if (!desc || !TRACE_ON(dmfile))
+        return;
+
+    TRACE_(dmfile)("DMUS_OBJECTDESC (%p):", desc);
+    TRACE_(dmfile)(" - dwSize = %u\n", desc->dwSize);
+
+#define X(flag) if (desc->dwValidData & flag) TRACE_(dmfile)(#flag " ")
+    TRACE_(dmfile)(" - dwValidData = %#08x ( ", desc->dwValidData);
+    X(DMUS_OBJ_OBJECT);
+    X(DMUS_OBJ_CLASS);
+    X(DMUS_OBJ_NAME);
+    X(DMUS_OBJ_CATEGORY);
+    X(DMUS_OBJ_FILENAME);
+    X(DMUS_OBJ_FULLPATH);
+    X(DMUS_OBJ_URL);
+    X(DMUS_OBJ_VERSION);
+    X(DMUS_OBJ_DATE);
+    X(DMUS_OBJ_LOADED);
+    X(DMUS_OBJ_MEMORY);
+    X(DMUS_OBJ_STREAM);
+    TRACE_(dmfile)(")\n");
+#undef X
+
+    if (desc->dwValidData & DMUS_OBJ_CLASS)
+        TRACE_(dmfile)(" - guidClass = %s\n", debugstr_dmguid(&desc->guidClass));
+    if (desc->dwValidData & DMUS_OBJ_OBJECT)
+        TRACE_(dmfile)(" - guidObject = %s\n", debugstr_guid(&desc->guidObject));
+
+    if (desc->dwValidData & DMUS_OBJ_DATE) {
+        SYSTEMTIME time;
+        FileTimeToSystemTime(&desc->ftDate, &time);
+        TRACE_(dmfile)(" - ftDate = \'%04u-%02u-%02u %02u:%02u:%02u\'\n",
+                time.wYear, time.wMonth, time.wDay, time.wHour, time.wMinute, time.wSecond);
+    }
+    if (desc->dwValidData & DMUS_OBJ_VERSION)
+        TRACE_(dmfile)(" - vVersion = \'%u,%u,%u,%u\'\n",
+                HIWORD(desc->vVersion.dwVersionMS), LOWORD(desc->vVersion.dwVersionMS),
+                HIWORD(desc->vVersion.dwVersionLS), LOWORD(desc->vVersion.dwVersionLS));
+    if (desc->dwValidData & DMUS_OBJ_NAME)
+        TRACE_(dmfile)(" - wszName = %s\n", debugstr_w(desc->wszName));
+    if (desc->dwValidData & DMUS_OBJ_CATEGORY)
+        TRACE_(dmfile)(" - wszCategory = %s\n", debugstr_w(desc->wszCategory));
+    if (desc->dwValidData & DMUS_OBJ_FILENAME)
+        TRACE_(dmfile)(" - wszFileName = %s\n", debugstr_w(desc->wszFileName));
+    if (desc->dwValidData & DMUS_OBJ_MEMORY)
+        TRACE_(dmfile)(" - llMemLength = 0x%s - pbMemData = %p\n",
+                wine_dbgstr_longlong(desc->llMemLength), desc->pbMemData);
+    if (desc->dwValidData & DMUS_OBJ_STREAM)
+        TRACE_(dmfile)(" - pStream = %p\n", desc->pStream);
+}
+
+
 /* RIFF format parsing */
 #define CHUNK_HDR_SIZE (sizeof(FOURCC) + sizeof(DWORD))
 
-static inline const char *debugstr_fourcc(DWORD fourcc)
-{
-    if (!fourcc) return "''";
-    return wine_dbg_sprintf("'%c%c%c%c'", (char)(fourcc), (char)(fourcc >> 8),
-            (char)(fourcc >> 16), (char)(fourcc >> 24));
-}
-
 const char *debugstr_chunk(const struct chunk_entry *chunk)
 {
     const char *type = "";
