@@ -81,7 +81,7 @@ static ULONG WINAPI IDirectSoundNotifyImpl_Release(IDirectSoundNotify *iface)
     return ref;
 }
 
-static int notify_compar(const void *l, const void *r)
+static int __cdecl notify_compar(const void *l, const void *r)
 {
     const DSBPOSITIONNOTIFY *left = l;
     const DSBPOSITIONNOTIFY *right = r;
@@ -695,6 +695,23 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetFrequency(IDirectSoundBuffer8 *i
 	return DS_OK;
 }
 
+static const char* dump_DSFX_guid(const DSEFFECTDESC *desc)
+{
+#define FE(guid) if (IsEqualGUID(&guid, &desc->guidDSFXClass)) return #guid
+    FE(GUID_DSFX_STANDARD_GARGLE);
+    FE(GUID_DSFX_STANDARD_CHORUS);
+    FE(GUID_DSFX_STANDARD_FLANGER);
+    FE(GUID_DSFX_STANDARD_ECHO);
+    FE(GUID_DSFX_STANDARD_DISTORTION);
+    FE(GUID_DSFX_STANDARD_COMPRESSOR);
+    FE(GUID_DSFX_STANDARD_PARAMEQ);
+    FE(GUID_DSFX_STANDARD_I3DL2REVERB);
+    FE(GUID_DSFX_WAVES_REVERB);
+#undef FE
+
+    return debugstr_guid(&desc->guidDSFXClass);
+}
+
 static HRESULT WINAPI IDirectSoundBufferImpl_SetFX(IDirectSoundBuffer8 *iface, DWORD dwEffectsCount,
         LPDSEFFECTDESC pDSFXDesc, DWORD *pdwResultCodes)
 {
@@ -767,6 +784,8 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetFX(IDirectSoundBuffer8 *iface, D
 	dmt.pbFormat = (BYTE*)&wfx;
 
 	for (u = 0; u < dwEffectsCount; u++) {
+		TRACE("%d: 0x%08x, %s\n", u, pDSFXDesc[u].dwFlags, dump_DSFX_guid(&pDSFXDesc[u]));
+
 		hr2 = CoCreateInstance(&pDSFXDesc[u].guidDSFXClass, NULL, CLSCTX_INPROC_SERVER, &IID_IMediaObject, (LPVOID*)&filters[u].obj);
 
 		if (SUCCEEDED(hr2)) {
@@ -923,11 +942,16 @@ static HRESULT WINAPI IDirectSoundBufferImpl_QueryInterface(IDirectSoundBuffer8
                 return S_OK;
 	}
 
-	if ( IsEqualGUID( &IID_IDirectSoundNotify, riid ) ) {
+        if ( IsEqualGUID( &IID_IDirectSoundNotify, riid ) ) {
+            if(This->dsbd.dwFlags & DSBCAPS_CTRLPOSITIONNOTIFY) {
                 IDirectSoundNotify_AddRef(&This->IDirectSoundNotify_iface);
                 *ppobj = &This->IDirectSoundNotify_iface;
                 return S_OK;
-	}
+            }
+
+            TRACE( "App requested IDirectSoundNotify without DSBCAPS_CTRLPOSITIONNOTIFY flag.\n");
+            return E_NOINTERFACE;
+        }
 
 	if ( IsEqualGUID( &IID_IDirectSound3DBuffer, riid ) ) {
             if(This->dsbd.dwFlags & DSBCAPS_CTRL3D){
