@@ -105,10 +105,12 @@ static void _dump_DSBCAPS(DWORD xmask) {
         FE(DSBCAPS_CTRLPAN)
         FE(DSBCAPS_CTRLVOLUME)
         FE(DSBCAPS_CTRLPOSITIONNOTIFY)
+        FE(DSBCAPS_CTRLFX)
         FE(DSBCAPS_STICKYFOCUS)
         FE(DSBCAPS_GLOBALFOCUS)
         FE(DSBCAPS_GETCURRENTPOSITION2)
         FE(DSBCAPS_MUTE3DATMAXDISTANCE)
+        FE(DSBCAPS_LOCDEFER)
 #undef FE
     };
     unsigned int     i;
