@@ -274,8 +274,8 @@ void get_mono_dc_colors( DC *dc, int color_table_size, BITMAPINFO *info, int cou
  *           null driver fallback implementations
  */
 
-BOOL nulldrv_StretchBlt( PHYSDEV dst_dev, struct bitblt_coords *dst,
-                         PHYSDEV src_dev, struct bitblt_coords *src, DWORD rop )
+BOOL CDECL nulldrv_StretchBlt( PHYSDEV dst_dev, struct bitblt_coords *dst,
+                               PHYSDEV src_dev, struct bitblt_coords *src, DWORD rop )
 {
     DC *dc_src = get_physdev_dc( src_dev ), *dc_dst = get_nulldrv_dc( dst_dev );
     char src_buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
@@ -326,8 +326,8 @@ BOOL nulldrv_StretchBlt( PHYSDEV dst_dev, struct bitblt_coords *dst,
 }
 
 
-BOOL nulldrv_AlphaBlend( PHYSDEV dst_dev, struct bitblt_coords *dst,
-                         PHYSDEV src_dev, struct bitblt_coords *src, BLENDFUNCTION func )
+BOOL CDECL nulldrv_AlphaBlend( PHYSDEV dst_dev, struct bitblt_coords *dst,
+                               PHYSDEV src_dev, struct bitblt_coords *src, BLENDFUNCTION func )
 {
     DC *dc_src = get_physdev_dc( src_dev ), *dc_dst = get_nulldrv_dc( dst_dev );
     char src_buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
@@ -365,8 +365,8 @@ done:
 }
 
 
-DWORD nulldrv_BlendImage( PHYSDEV dev, BITMAPINFO *info, const struct gdi_image_bits *bits,
-                          struct bitblt_coords *src, struct bitblt_coords *dst, BLENDFUNCTION blend )
+DWORD CDECL nulldrv_BlendImage( PHYSDEV dev, BITMAPINFO *info, const struct gdi_image_bits *bits,
+                                struct bitblt_coords *src, struct bitblt_coords *dst, BLENDFUNCTION blend )
 {
     char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
     BITMAPINFO *dst_info = (BITMAPINFO *)buffer;
@@ -414,8 +414,8 @@ update_format:
     return ERROR_BAD_FORMAT;
 }
 
-BOOL nulldrv_GradientFill( PHYSDEV dev, TRIVERTEX *vert_array, ULONG nvert,
-                           void * grad_array, ULONG ngrad, ULONG mode )
+BOOL CDECL nulldrv_GradientFill( PHYSDEV dev, TRIVERTEX *vert_array, ULONG nvert,
+                                 void * grad_array, ULONG ngrad, ULONG mode )
 {
     DC *dc = get_nulldrv_dc( dev );
     char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
@@ -499,7 +499,7 @@ done:
     return ret;
 }
 
-COLORREF nulldrv_GetPixel( PHYSDEV dev, INT x, INT y )
+COLORREF CDECL nulldrv_GetPixel( PHYSDEV dev, INT x, INT y )
 {
     DC *dc = get_nulldrv_dc( dev );
     char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
