@@ -462,9 +462,9 @@ fail:
 
 
 
-INT nulldrv_StretchDIBits( PHYSDEV dev, INT xDst, INT yDst, INT widthDst, INT heightDst,
-                           INT xSrc, INT ySrc, INT widthSrc, INT heightSrc, const void *bits,
-                           BITMAPINFO *src_info, UINT coloruse, DWORD rop )
+INT CDECL nulldrv_StretchDIBits( PHYSDEV dev, INT xDst, INT yDst, INT widthDst, INT heightDst,
+                                 INT xSrc, INT ySrc, INT widthSrc, INT heightSrc, const void *bits,
+                                 BITMAPINFO *src_info, UINT coloruse, DWORD rop )
 {
     DC *dc = get_nulldrv_dc( dev );
     char dst_buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
@@ -766,9 +766,9 @@ done:
 }
 
 
-INT nulldrv_SetDIBitsToDevice( PHYSDEV dev, INT x_dst, INT y_dst, DWORD cx, DWORD cy,
-                               INT x_src, INT y_src, UINT startscan, UINT lines,
-                               const void *bits, BITMAPINFO *src_info, UINT coloruse )
+INT CDECL nulldrv_SetDIBitsToDevice( PHYSDEV dev, INT x_dst, INT y_dst, DWORD cx, DWORD cy,
+                                     INT x_src, INT y_src, UINT startscan, UINT lines,
+                                     const void *bits, BITMAPINFO *src_info, UINT coloruse )
 {
     DC *dc = get_nulldrv_dc( dev );
     char dst_buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
@@ -1627,7 +1627,7 @@ NTSTATUS WINAPI D3DKMTCreateDCFromMemory( D3DKMT_CREATEDCFROMMEMORY *desc )
 
     if (!desc->pMemory) return STATUS_INVALID_PARAMETER;
 
-    for (i = 0; i < sizeof(format_info) / sizeof(*format_info); ++i)
+    for (i = 0; i < ARRAY_SIZE( format_info ); ++i)
     {
         if (format_info[i].format == desc->Format)
         {
@@ -1831,5 +1831,6 @@ static BOOL DIB_DeleteObject( HGDIOBJ handle )
     else VirtualFree( bmp->dib.dsBm.bmBits, 0, MEM_RELEASE );
 
     HeapFree(GetProcessHeap(), 0, bmp->color_table);
-    return HeapFree( GetProcessHeap(), 0, bmp );
+    HeapFree( GetProcessHeap(), 0, bmp );
+    return TRUE;
 }
