@@ -189,7 +189,7 @@ X(EMR_CREATECOLORSPACEW)
 static const char *get_emr_name(DWORD type)
 {
     unsigned int i;
-    for(i = 0; i < sizeof(emr_names) / sizeof(emr_names[0]); i++)
+    for(i = 0; i < ARRAY_SIZE(emr_names); i++)
         if(type == emr_names[i].type) return emr_names[i].name;
     TRACE("Unknown record type %d\n", type);
    return NULL;
@@ -297,7 +297,8 @@ static BOOL EMF_Delete_HENHMETAFILE( HENHMETAFILE hmf )
         UnmapViewOfFile( metaObj->emh );
     else
         HeapFree( GetProcessHeap(), 0, metaObj->emh );
-    return HeapFree( GetProcessHeap(), 0, metaObj );
+    HeapFree( GetProcessHeap(), 0, metaObj );
+    return TRUE;
 }
 
 /******************************************************************
@@ -479,7 +480,9 @@ HENHMETAFILE WINAPI SetEnhMetaFileBits(UINT bufsize, const BYTE *buf)
 {
     ENHMETAHEADER *emh = HeapAlloc( GetProcessHeap(), 0, bufsize );
     HENHMETAFILE hmf;
-    memmove(emh, buf, bufsize);
+
+    if (!emh) return 0;
+    memcpy(emh, buf, bufsize);
     hmf = EMF_Create_HENHMETAFILE( emh, bufsize, FALSE );
     if (!hmf)
         HeapFree( GetProcessHeap(), 0, emh );
