@@ -32,7 +32,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(enhmetafile);
 
-static BOOL EMFDRV_DeleteDC( PHYSDEV dev );
+static BOOL CDECL EMFDRV_DeleteDC( PHYSDEV dev );
 
 static const struct gdi_dc_funcs emfdrv_driver =
 {
@@ -73,6 +73,7 @@ static const struct gdi_dc_funcs emfdrv_driver =
     NULL,                            /* pGetCharABCWidths */
     NULL,                            /* pGetCharABCWidthsI */
     NULL,                            /* pGetCharWidth */
+    NULL,                            /* pGetCharWidthInfo */
     EMFDRV_GetDeviceCaps,            /* pGetDeviceCaps */
     NULL,                            /* pGetDeviceGammaRamp */
     NULL,                            /* pGetFontData */
@@ -162,6 +163,8 @@ static const struct gdi_dc_funcs emfdrv_driver =
     EMFDRV_StrokePath,               /* pStrokePath */
     NULL,                            /* pUnrealizePalette */
     EMFDRV_WidenPath,                /* pWidenPath */
+    NULL,                            /* pD3DKMTCheckVidPnExclusiveOwnership */
+    NULL,                            /* pD3DKMTSetVidPnSourceOwner */
     NULL,                            /* wine_get_wgl_driver */
     NULL,                            /* wine_get_vulkan_driver */
     GDI_PRIORITY_GRAPHICS_DRV        /* priority */
@@ -171,7 +174,7 @@ static const struct gdi_dc_funcs emfdrv_driver =
 /**********************************************************************
  *	     EMFDRV_DeleteDC
  */
-static BOOL EMFDRV_DeleteDC( PHYSDEV dev )
+static BOOL CDECL EMFDRV_DeleteDC( PHYSDEV dev )
 {
     EMFDRV_PDEVICE *physDev = get_emf_physdev( dev );
     UINT index;
@@ -378,7 +381,7 @@ HDC WINAPI CreateEnhMetaFileW(
         ref_dc = CreateDCW( displayW, NULL, NULL, NULL );
 
     memset( physDev->dev_caps, 0, sizeof(physDev->dev_caps) );
-    for (cap = 0; cap < sizeof(physDev->dev_caps) / sizeof(physDev->dev_caps[0]); cap++)
+    for (cap = 0; cap < ARRAY_SIZE( physDev->dev_caps ); cap++)
         if (devcap_is_valid( cap ))
             physDev->dev_caps[cap] = GetDeviceCaps( ref_dc, cap );
 
