@@ -743,7 +743,6 @@ void __gl_meshCheckMesh( GLUmesh *mesh )
   GLUvertex *v, *vPrev;
   GLUhalfEdge *e, *ePrev;
 
-  fPrev = fHead;
   for( fPrev = fHead ; (f = fPrev->next) != fHead; fPrev = f) {
     assert( f->prev == fPrev );
     e = f->anEdge;
@@ -758,7 +757,6 @@ void __gl_meshCheckMesh( GLUmesh *mesh )
   }
   assert( f->prev == fPrev && f->anEdge == NULL && f->data == NULL );
 
-  vPrev = vHead;
   for( vPrev = vHead ; (v = vPrev->next) != vHead; vPrev = v) {
     assert( v->prev == vPrev );
     e = v->anEdge;
@@ -773,7 +771,6 @@ void __gl_meshCheckMesh( GLUmesh *mesh )
   }
   assert( v->prev == vPrev && v->anEdge == NULL && v->data == NULL );
 
-  ePrev = eHead;
   for( ePrev = eHead ; (e = ePrev->next) != eHead; ePrev = e) {
     assert( e->Sym->next == ePrev->Sym );
     assert( e->Sym != e );
