@@ -96,7 +96,7 @@ static HRESULT do_regexp_match_next(script_ctx_t *ctx, RegExpInstance *regexp,
     }
 
     if(!(rem_flags & REM_NO_CTX_UPDATE)) {
-        DWORD i, n = min(sizeof(ctx->match_parens)/sizeof(ctx->match_parens[0]), ret->paren_count);
+        DWORD i, n = min(ARRAY_SIZE(ctx->match_parens), ret->paren_count);
 
         for(i=0; i < n; i++) {
             if(ret->parens[i].index == -1) {
@@ -108,7 +108,7 @@ static HRESULT do_regexp_match_next(script_ctx_t *ctx, RegExpInstance *regexp,
             }
         }
 
-        if(n < sizeof(ctx->match_parens)/sizeof(ctx->match_parens[0]))
+        if(n < ARRAY_SIZE(ctx->match_parens))
             memset(ctx->match_parens+n, 0, sizeof(ctx->match_parens) - n*sizeof(ctx->match_parens[0]));
     }
 
@@ -256,12 +256,6 @@ static HRESULT RegExp_get_source(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r
     return S_OK;
 }
 
-static HRESULT RegExp_set_source(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t value)
-{
-    FIXME("\n");
-    return E_NOTIMPL;
-}
-
 static HRESULT RegExp_get_global(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
     TRACE("\n");
@@ -270,12 +264,6 @@ static HRESULT RegExp_get_global(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r
     return S_OK;
 }
 
-static HRESULT RegExp_set_global(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t value)
-{
-    FIXME("\n");
-    return E_NOTIMPL;
-}
-
 static HRESULT RegExp_get_ignoreCase(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
     TRACE("\n");
@@ -284,12 +272,6 @@ static HRESULT RegExp_get_ignoreCase(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_
     return S_OK;
 }
 
-static HRESULT RegExp_set_ignoreCase(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t value)
-{
-    FIXME("\n");
-    return E_NOTIMPL;
-}
-
 static HRESULT RegExp_get_multiline(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
     TRACE("\n");
@@ -298,12 +280,6 @@ static HRESULT RegExp_get_multiline(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t
     return S_OK;
 }
 
-static HRESULT RegExp_set_multiline(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t value)
-{
-    FIXME("\n");
-    return E_NOTIMPL;
-}
-
 static INT index_from_val(script_ctx_t *ctx, jsval_t v)
 {
     double n;
@@ -607,11 +583,11 @@ static void RegExp_destructor(jsdisp_t *dispex)
 
 static const builtin_prop_t RegExp_props[] = {
     {execW,                  RegExp_exec,                  PROPF_METHOD|1},
-    {globalW,                NULL,0,                       RegExp_get_global,     RegExp_set_global},
-    {ignoreCaseW,            NULL,0,                       RegExp_get_ignoreCase, RegExp_set_ignoreCase},
+    {globalW,                NULL,0,                       RegExp_get_global},
+    {ignoreCaseW,            NULL,0,                       RegExp_get_ignoreCase},
     {lastIndexW,             NULL,0,                       RegExp_get_lastIndex,  RegExp_set_lastIndex},
-    {multilineW,             NULL,0,                       RegExp_get_multiline,  RegExp_set_multiline},
-    {sourceW,                NULL,0,                       RegExp_get_source,     RegExp_set_source},
+    {multilineW,             NULL,0,                       RegExp_get_multiline},
+    {sourceW,                NULL,0,                       RegExp_get_source},
     {testW,                  RegExp_test,                  PROPF_METHOD|1},
     {toStringW,              RegExp_toString,              PROPF_METHOD}
 };
@@ -619,24 +595,24 @@ static const builtin_prop_t RegExp_props[] = {
 static const builtin_info_t RegExp_info = {
     JSCLASS_REGEXP,
     {NULL, RegExp_value, 0},
-    sizeof(RegExp_props)/sizeof(*RegExp_props),
+    ARRAY_SIZE(RegExp_props),
     RegExp_props,
     RegExp_destructor,
     NULL
 };
 
 static const builtin_prop_t RegExpInst_props[] = {
-    {globalW,                NULL,0,                       RegExp_get_global,     RegExp_set_global},
-    {ignoreCaseW,            NULL,0,                       RegExp_get_ignoreCase, RegExp_set_ignoreCase},
+    {globalW,                NULL,0,                       RegExp_get_global},
+    {ignoreCaseW,            NULL,0,                       RegExp_get_ignoreCase},
     {lastIndexW,             NULL,0,                       RegExp_get_lastIndex,  RegExp_set_lastIndex},
-    {multilineW,             NULL,0,                       RegExp_get_multiline,  RegExp_set_multiline},
-    {sourceW,                NULL,0,                       RegExp_get_source,     RegExp_set_source}
+    {multilineW,             NULL,0,                       RegExp_get_multiline},
+    {sourceW,                NULL,0,                       RegExp_get_source}
 };
 
 static const builtin_info_t RegExpInst_info = {
     JSCLASS_REGEXP,
     {NULL, RegExp_value, 0},
-    sizeof(RegExpInst_props)/sizeof(*RegExpInst_props),
+    ARRAY_SIZE(RegExpInst_props),
     RegExpInst_props,
     RegExp_destructor,
     NULL
@@ -671,12 +647,12 @@ HRESULT create_regexp(script_ctx_t *ctx, jsstr_t *src, DWORD flags, jsdisp_t **r
     const WCHAR *str;
     HRESULT hres;
 
-    TRACE("%s %x\n", debugstr_jsstr(src), flags);
-
     str = jsstr_flatten(src);
     if(!str)
         return E_OUTOFMEMORY;
 
+    TRACE("%s %x\n", debugstr_wn(str, jsstr_length(src)), flags);
+
     hres = alloc_regexp(ctx, NULL, &regexp);
     if(FAILED(hres))
         return hres;
@@ -1019,7 +995,7 @@ static const builtin_prop_t RegExpConstr_props[] = {
 static const builtin_info_t RegExpConstr_info = {
     JSCLASS_FUNCTION,
     DEFAULT_FUNCTION_VALUE,
-    sizeof(RegExpConstr_props)/sizeof(*RegExpConstr_props),
+    ARRAY_SIZE(RegExpConstr_props),
     RegExpConstr_props,
     NULL,
     NULL
