@@ -25,6 +25,8 @@
 #include <winternl.h>
 #include <winreg.h>
 #include "wine/test.h"
+#include "wine/heap.h"
+#include "wine/rbtree.h"
 
 #ifndef STATUS_DEBUGGER_INACTIVE
 #define STATUS_DEBUGGER_INACTIVE         ((NTSTATUS) 0xC0000354)
@@ -36,9 +38,6 @@ static int    myARGC;
 static char** myARGV;
 
 static BOOL (WINAPI *pCheckRemoteDebuggerPresent)(HANDLE,PBOOL);
-static BOOL (WINAPI *pDebugActiveProcessStop)(DWORD);
-static BOOL (WINAPI *pDebugSetProcessKillOnExit)(BOOL);
-static BOOL (WINAPI *pIsDebuggerPresent)(void);
 
 static void (WINAPI *pDbgBreakPoint)(void);
 
@@ -46,7 +45,7 @@ static LONG child_failures;
 
 static HMODULE ntdll;
 
-static void WINAPIV WINETEST_PRINTF_ATTR(2, 3) test_child_ok(int condition, const char *msg, ...)
+static void WINAPIV __WINE_PRINTF_ATTR(2, 3) test_child_ok(int condition, const char *msg, ...)
 {
     __ms_va_list valist;
 
@@ -168,16 +167,11 @@ static void run_background_thread(void)
 {
     DWORD tid;
     HANDLE thread = CreateThread(NULL, 0, thread_proc, NULL, 0, &tid);
-    ok(thread != NULL, "CreateThrad failed\n");
+    ok(thread != NULL, "CreateThread failed\n");
     CloseHandle(thread);
 }
 
-typedef struct
-{
-    DWORD pid;
-} crash_blackbox_t;
-
-static void doCrash(int argc,  char** argv)
+static void doCrash(void)
 {
     volatile char* p;
 
@@ -187,13 +181,6 @@ static void doCrash(int argc,  char** argv)
 
     run_background_thread();
 
-    if (argc >= 4)
-    {
-        crash_blackbox_t blackbox;
-        blackbox.pid=GetCurrentProcessId();
-        save_blackbox(argv[3], &blackbox, sizeof(blackbox), NULL);
-    }
-
     /* Just crash */
     trace("child: crashing...\n");
     p=NULL;
@@ -218,19 +205,117 @@ typedef struct
 struct debugger_context
 {
     DWORD pid;
-    DWORD main_thread;
     DEBUG_EVENT ev;
     unsigned process_cnt;
-    unsigned thread_cnt;
     unsigned dll_cnt;
     void *image_base;
+    DWORD thread_tag;
+    unsigned thread_cnt;
+    struct wine_rb_tree threads;
+    struct debuggee_thread *current_thread;
+    struct debuggee_thread *main_thread;
 };
 
+struct debuggee_thread
+{
+    DWORD tid;
+    DWORD tag;
+    HANDLE handle;
+    CONTEXT ctx;
+    struct wine_rb_entry entry;
+};
+
+int debuggee_thread_compare(const void *key, const struct wine_rb_entry *entry)
+{
+    struct debuggee_thread *thread = WINE_RB_ENTRY_VALUE(entry, struct debuggee_thread, entry);
+    return memcmp(key, &thread->tid, sizeof(thread->tid));
+}
+
+static void add_thread(struct debugger_context *ctx, DWORD tid)
+{
+    struct debuggee_thread *thread;
+    if (!ctx->thread_cnt++) wine_rb_init(&ctx->threads, debuggee_thread_compare);
+    thread = heap_alloc(sizeof(*thread));
+    thread->tid = tid;
+    thread->tag = ctx->thread_tag;
+    thread->handle = NULL;
+    wine_rb_put(&ctx->threads, &tid, &thread->entry);
+    if (!ctx->main_thread) ctx->main_thread = thread;
+}
+
+static struct debuggee_thread *get_debuggee_thread(struct debugger_context *ctx, DWORD tid)
+{
+    struct wine_rb_entry *entry = wine_rb_get(&ctx->threads, &tid);
+    ok(entry != NULL, "unknown thread %x\n", tid);
+    return WINE_RB_ENTRY_VALUE(entry, struct debuggee_thread, entry);
+}
+
+static void remove_thread(struct debugger_context *ctx, DWORD tid)
+{
+    struct debuggee_thread *thread = get_debuggee_thread(ctx, tid);
+
+    wine_rb_remove(&ctx->threads, &thread->entry);
+    if (thread->handle) CloseHandle(thread->handle);
+    heap_free(thread);
+}
+
+static void *get_ip(const CONTEXT *ctx)
+{
+#ifdef __i386__
+    return (void *)ctx->Eip;
+#elif defined(__x86_64__)
+    return (void *)ctx->Rip;
+#else
+    return NULL;
+#endif
+}
+
+#define fetch_thread_context(a) fetch_thread_context_(__LINE__,a)
+static void fetch_thread_context_(unsigned line, struct debuggee_thread *thread)
+{
+    BOOL ret;
+
+    if (!thread->handle)
+    {
+        thread->handle = OpenThread(THREAD_GET_CONTEXT | THREAD_SET_CONTEXT | THREAD_QUERY_INFORMATION,
+                                    FALSE, thread->tid);
+        ok_(__FILE__,line)(thread->handle != NULL, "OpenThread failed: %u\n", GetLastError());
+    }
+
+    memset(&thread->ctx, 0xaa, sizeof(thread->ctx));
+    thread->ctx.ContextFlags = CONTEXT_FULL;
+    ret = GetThreadContext(thread->handle, &thread->ctx);
+    ok_(__FILE__,line)(ret, "GetThreadContext failed: %u\n", GetLastError());
+}
+
+#define set_thread_context(a,b) set_thread_context_(__LINE__,a,b)
+static void set_thread_context_(unsigned line, struct debugger_context *ctx, struct debuggee_thread *thread)
+{
+    BOOL ret;
+    ret = SetThreadContext(thread->handle, &thread->ctx);
+    ok_(__FILE__,line)(ret, "SetThreadContext failed: %u\n", GetLastError());
+}
+
+static void fetch_process_context(struct debugger_context *ctx)
+{
+    struct debuggee_thread *thread;
+
+    WINE_RB_FOR_EACH_ENTRY(thread, &ctx->threads, struct debuggee_thread, entry)
+    {
+        fetch_thread_context(thread);
+    }
+}
+
+#define WAIT_EVENT_TIMEOUT 20000
+#define POLL_EVENT_TIMEOUT 200
+
 #define next_event(a,b) next_event_(__LINE__,a,b)
 static void next_event_(unsigned line, struct debugger_context *ctx, unsigned timeout)
 {
     BOOL ret;
 
+    ctx->current_thread = NULL;
+
     for (;;)
     {
         if (ctx->process_cnt && ctx->ev.dwDebugEventCode != -1)
@@ -263,15 +348,18 @@ static void next_event_(unsigned line, struct debugger_context *ctx, unsigned ti
     switch (ctx->ev.dwDebugEventCode)
     {
     case CREATE_PROCESS_DEBUG_EVENT:
-        ctx->thread_cnt++;
+        add_thread(ctx, ctx->ev.dwThreadId);
         ctx->image_base = ctx->ev.u.CreateProcessInfo.lpBaseOfImage;
         break;
+    case EXIT_PROCESS_DEBUG_EVENT:
+        remove_thread(ctx, ctx->ev.dwThreadId);
+        return;
     case CREATE_THREAD_DEBUG_EVENT:
-        ctx->thread_cnt++;
+        add_thread(ctx, ctx->ev.dwThreadId);
         break;
     case EXIT_THREAD_DEBUG_EVENT:
-        ctx->thread_cnt--;
-        break;
+        remove_thread(ctx, ctx->ev.dwThreadId);
+        return;
     case LOAD_DLL_DEBUG_EVENT:
         ok(ctx->ev.u.LoadDll.lpBaseOfDll != ctx->image_base, "process image reported as DLL load event\n");
         ctx->dll_cnt++;
@@ -280,10 +368,27 @@ static void next_event_(unsigned line, struct debugger_context *ctx, unsigned ti
         ctx->dll_cnt--;
         break;
     }
+
+    ctx->current_thread = get_debuggee_thread(ctx, ctx->ev.dwThreadId);
 }
 
-static void process_attach_events(struct debugger_context *ctx)
+#define wait_for_breakpoint(a) wait_for_breakpoint_(__LINE__,a)
+static void wait_for_breakpoint_(unsigned line, struct debugger_context *ctx)
 {
+    do next_event_(line, ctx, WAIT_EVENT_TIMEOUT);
+    while (ctx->ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT || ctx->ev.dwDebugEventCode == UNLOAD_DLL_DEBUG_EVENT
+           || ctx->ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT);
+
+    ok_(__FILE__,line)(ctx->ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx->ev.dwDebugEventCode);
+    ok_(__FILE__,line)(ctx->ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
+                       ctx->ev.u.Exception.ExceptionRecord.ExceptionCode);
+}
+
+static void process_attach_events(struct debugger_context *ctx, BOOL pass_exception)
+{
+    DEBUG_EVENT ev;
+    BOOL ret;
+
     ctx->ev.dwDebugEventCode = -1;
     next_event(ctx, 0);
     ok(ctx->ev.dwDebugEventCode == CREATE_PROCESS_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx->ev.dwDebugEventCode);
@@ -301,7 +406,11 @@ static void process_attach_events(struct debugger_context *ctx)
 
     do
     {
-        next_event(ctx, 2000);
+        /* even when there are more pending events, they are not reported until current event is continued */
+        ret = WaitForDebugEvent(&ev, 10);
+        ok(GetLastError() == ERROR_SEM_TIMEOUT, "WaitForDebugEvent returned %x(%u)\n", ret, GetLastError());
+
+        next_event(ctx, WAIT_EVENT_TIMEOUT);
         if (ctx->ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT)
             ok(ctx->ev.u.LoadDll.lpBaseOfDll != ntdll, "ntdll.dll reported out of order\n");
     } while (ctx->ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT || ctx->ev.dwDebugEventCode == UNLOAD_DLL_DEBUG_EVENT);
@@ -312,17 +421,24 @@ static void process_attach_events(struct debugger_context *ctx)
     if (ctx->ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT)
     {
         DWORD last_thread = ctx->ev.dwThreadId;
-        next_event(ctx, 2000);
+        next_event(ctx, WAIT_EVENT_TIMEOUT);
         ok(ctx->ev.dwThreadId == last_thread, "unexpected thread\n");
     }
 
     ok(ctx->ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx->ev.dwDebugEventCode);
     ok(ctx->ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
        ctx->ev.u.Exception.ExceptionRecord.ExceptionCode);
-    ok(ctx->ev.u.Exception.ExceptionRecord.ExceptionAddress == pDbgBreakPoint, "ExceptionAddres != DbgBreakPoint\n");
+    ok(ctx->ev.u.Exception.ExceptionRecord.ExceptionAddress == pDbgBreakPoint, "ExceptionAddress != DbgBreakPoint\n");
+
+    if (pass_exception)
+    {
+        ret = ContinueDebugEvent(ctx->ev.dwProcessId, ctx->ev.dwThreadId, DBG_EXCEPTION_NOT_HANDLED);
+        ok(ret, "ContinueDebugEvent failed, last error %d.\n", GetLastError());
+        ctx->ev.dwDebugEventCode = -1;
+    }
 
     /* flush debug events */
-    do next_event(ctx, 200);
+    do next_event(ctx, POLL_EVENT_TIMEOUT);
     while (ctx->ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT || ctx->ev.dwDebugEventCode == UNLOAD_DLL_DEBUG_EVENT
            || ctx->ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT || ctx->ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT);
     ok(ctx->ev.dwDebugEventCode == -1, "dwDebugEventCode = %d\n", ctx->ev.dwDebugEventCode);
@@ -353,7 +469,7 @@ static void doDebugger(int argc, char** argv)
     if (strstr(myARGV[2], "process"))
     {
         strcat(buf, "processing debug messages\n");
-        process_attach_events(&ctx);
+        process_attach_events(&ctx, FALSE);
     }
 
     debug_event=(argc >= 6 ? (HANDLE)(INT_PTR)atol(argv[5]) : NULL);
@@ -370,7 +486,7 @@ static void doDebugger(int argc, char** argv)
 
     if (strstr(myARGV[2], "process"))
     {
-        next_event(&ctx, 2000);
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
         ok(ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
         ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionCode == STATUS_ACCESS_VIOLATION, "ExceptionCode = %x\n",
            ctx.ev.u.Exception.ExceptionRecord.ExceptionCode);
@@ -390,7 +506,7 @@ static void doDebugger(int argc, char** argv)
     blackbox.nokill_err=0;
     if (strstr(myARGV[2], "nokill"))
     {
-        blackbox.nokill_rc=pDebugSetProcessKillOnExit(FALSE);
+        blackbox.nokill_rc = DebugSetProcessKillOnExit(FALSE);
         if (!blackbox.nokill_rc)
             blackbox.nokill_err=GetLastError();
     }
@@ -400,7 +516,7 @@ static void doDebugger(int argc, char** argv)
     blackbox.detach_err=0;
     if (strstr(myARGV[2], "detach"))
     {
-        blackbox.detach_rc=pDebugActiveProcessStop(blackbox.pid);
+        blackbox.detach_rc = DebugActiveProcessStop(blackbox.pid);
         if (!blackbox.detach_rc)
             blackbox.detach_err=GetLastError();
     }
@@ -436,11 +552,9 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
     HANDLE start_event, done_event;
     char* cmd;
     char dbglog[MAX_PATH];
-    char childlog[MAX_PATH];
     PROCESS_INFORMATION	info;
     STARTUPINFOA startup;
     DWORD exit_code;
-    crash_blackbox_t crash_blackbox;
     debugger_blackbox_t dbg_blackbox;
     DWORD wait_code;
 
@@ -468,9 +582,8 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
     ok(ret == ERROR_SUCCESS, "unable to set AeDebug/debugger: ret=%d\n", ret);
     HeapFree(GetProcessHeap(), 0, cmd);
 
-    get_file_name(childlog);
-    cmd=HeapAlloc(GetProcessHeap(), 0, strlen(argv0)+16+strlen(dbglog)+2+1);
-    sprintf(cmd, "%s debugger crash \"%s\"", argv0, childlog);
+    cmd = HeapAlloc(GetProcessHeap(), 0, strlen(argv0) + 16);
+    sprintf(cmd, "%s debugger crash", argv0);
 
     trace("running %s...\n", dbgtasks);
     memset(&startup, 0, sizeof(startup));
@@ -494,7 +607,6 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
         WaitForSingleObject(info.hProcess, 5000);
         CloseHandle(info.hProcess);
         DeleteFileA(dbglog);
-        DeleteFileA(childlog);
         win_skip("Giving up on child process\n");
         return;
     }
@@ -530,18 +642,16 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
     if (skip_crash_and_debug)
     {
         DeleteFileA(dbglog);
-        DeleteFileA(childlog);
         win_skip("Giving up on debugger\n");
         return;
     }
 #endif
     ok(wait_code == WAIT_OBJECT_0, "Timed out waiting for the debugger\n");
 
-    ok(load_blackbox(childlog, &crash_blackbox, sizeof(crash_blackbox)), "failed to open: %s\n", childlog);
     ok(load_blackbox(dbglog, &dbg_blackbox, sizeof(dbg_blackbox)), "failed to open: %s\n", dbglog);
 
     ok(dbg_blackbox.argc == 6, "wrong debugger argument count: %d\n", dbg_blackbox.argc);
-    ok(dbg_blackbox.pid == crash_blackbox.pid, "the child and debugged pids don't match: %d != %d\n", crash_blackbox.pid, dbg_blackbox.pid);
+    ok(dbg_blackbox.pid == info.dwProcessId, "the child and debugged pids don't match: %d != %d\n", info.dwProcessId, dbg_blackbox.pid);
     ok(dbg_blackbox.debug_rc, "debugger: SetEvent(debug_event) failed err=%d\n", dbg_blackbox.debug_err);
     ok(dbg_blackbox.attach_rc, "DebugActiveProcess(%d) failed err=%d\n", dbg_blackbox.pid, dbg_blackbox.attach_err);
     ok(dbg_blackbox.nokill_rc, "DebugSetProcessKillOnExit(FALSE) failed err=%d\n", dbg_blackbox.nokill_err);
@@ -549,7 +659,6 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
     ok(!dbg_blackbox.failures, "debugger reported %u failures\n", dbg_blackbox.failures);
 
     DeleteFileA(dbglog);
-    DeleteFileA(childlog);
 }
 
 static void crash_and_winedbg(HKEY hkey, const char* argv0)
@@ -652,17 +761,9 @@ static void test_ExitCode(void)
     ok(disposition == REG_OPENED_EXISTING_KEY, "expected REG_OPENED_EXISTING_KEY, got %d\n", disposition);
     crash_and_debug(hkey, test_exe, "dbg,event,order");
     crash_and_debug(hkey, test_exe, "dbg,attach,event,code2");
-    if (pDebugSetProcessKillOnExit)
-        crash_and_debug(hkey, test_exe, "dbg,attach,event,nokill");
-    else
-        win_skip("DebugSetProcessKillOnExit is not available\n");
-    if (pDebugActiveProcessStop)
-    {
-        crash_and_debug(hkey, test_exe, "dbg,attach,event,detach");
-        crash_and_debug(hkey, test_exe, "dbg,attach,detach,late");
-    }
-    else
-        win_skip("DebugActiveProcessStop is not available\n");
+    crash_and_debug(hkey, test_exe, "dbg,attach,event,nokill");
+    crash_and_debug(hkey, test_exe, "dbg,attach,event,detach");
+    crash_and_debug(hkey, test_exe, "dbg,attach,detach,late");
     crash_and_debug(hkey, test_exe, "dbg,attach,process,event,detach");
 
     if (disposition == REG_CREATED_NEW_KEY)
@@ -739,7 +840,7 @@ static void doChild(int argc, char **argv)
     child_ok(ret, "CheckRemoteDebuggerPresent failed, last error %#x.\n", GetLastError());
     child_ok(debug, "Expected debug != 0, got %#x.\n", debug);
 
-    ret = pDebugActiveProcessStop(ppid);
+    ret = DebugActiveProcessStop(ppid);
     child_ok(ret, "DebugActiveProcessStop failed, last error %#x.\n", GetLastError());
 
     ret = pCheckRemoteDebuggerPresent(parent, &debug);
@@ -749,7 +850,7 @@ static void doChild(int argc, char **argv)
     ret = CloseHandle(parent);
     child_ok(ret, "CloseHandle failed, last error %#x.\n", GetLastError());
 
-    ret = pIsDebuggerPresent();
+    ret = IsDebuggerPresent();
     child_ok(ret, "Expected ret != 0, got %#x.\n", ret);
     ret = pCheckRemoteDebuggerPresent(GetCurrentProcess(), &debug);
     child_ok(ret, "CheckRemoteDebuggerPresent failed, last error %#x.\n", GetLastError());
@@ -757,7 +858,7 @@ static void doChild(int argc, char **argv)
 
     NtCurrentTeb()->Peb->BeingDebugged = FALSE;
 
-    ret = pIsDebuggerPresent();
+    ret = IsDebuggerPresent();
     child_ok(!ret, "Expected ret != 0, got %#x.\n", ret);
     ret = pCheckRemoteDebuggerPresent(GetCurrentProcess(), &debug);
     child_ok(ret, "CheckRemoteDebuggerPresent failed, last error %#x.\n", GetLastError());
@@ -781,9 +882,9 @@ static void test_debug_loop(int argc, char **argv)
     char *cmd;
     BOOL ret;
 
-    if (!pDebugActiveProcessStop || !pCheckRemoteDebuggerPresent)
+    if (!pCheckRemoteDebuggerPresent)
     {
-        win_skip("DebugActiveProcessStop or CheckRemoteDebuggerPresent not available, skipping test.\n");
+        win_skip("CheckRemoteDebuggerPresent not available, skipping test.\n");
         return;
     }
 
@@ -899,7 +1000,7 @@ static void doChildren(int argc, char **argv)
     HeapFree(GetProcessHeap(), 0, cmd);
 }
 
-static void test_debug_children(char *name, DWORD flag, BOOL debug_child)
+static void test_debug_children(const char *name, DWORD flag, BOOL debug_child, BOOL pass_exception)
 {
     const char *arguments = "debugger children";
     struct child_blackbox blackbox;
@@ -912,9 +1013,9 @@ static void test_debug_children(char *name, DWORD flag, BOOL debug_child)
     BOOL debug, ret;
     struct debugger_context ctx = { 0 };
 
-    if (!pDebugActiveProcessStop || !pCheckRemoteDebuggerPresent)
+    if (!pCheckRemoteDebuggerPresent)
     {
-        win_skip("DebugActiveProcessStop or CheckRemoteDebuggerPresent not available, skipping test.\n");
+        win_skip("CheckRemoteDebuggerPresent not available, skipping test.\n");
         return;
     }
 
@@ -958,51 +1059,52 @@ static void test_debug_children(char *name, DWORD flag, BOOL debug_child)
 
     if (flag)
     {
-        next_event(&ctx, 2000);
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
         ok(ctx.ev.dwDebugEventCode == CREATE_PROCESS_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
         ok(ctx.pid == pi.dwProcessId, "unexpected dwProcessId %x\n", ctx.ev.dwProcessId == ctx.pid);
 
-        next_event(&ctx, 2000);
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
         ok(ctx.ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
 
-        do next_event(&ctx, 2000);
-        while (ctx.ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT || ctx.ev.dwDebugEventCode == UNLOAD_DLL_DEBUG_EVENT
-               || ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT);
+        wait_for_breakpoint(&ctx);
         ok(ctx.dll_cnt > 2, "dll_cnt = %d\n", ctx.dll_cnt);
-
-        ok(ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
-        ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
-           ctx.ev.u.Exception.ExceptionRecord.ExceptionCode);
     }
     else
     {
         DWORD last_thread;
 
-        process_attach_events(&ctx);
+        process_attach_events(&ctx, pass_exception);
         ok(ctx.pid == pi.dwProcessId, "unexpected dwProcessId %x\n", ctx.pid);
 
         ret = DebugBreakProcess(pi.hProcess);
         ok(ret, "BreakProcess failed: %u\n", GetLastError());
 
-        /* a new thread, which executes DbgDebugBreak, is created */
-        next_event(&ctx, 2000);
+        /* a new thread, which executes DbgBreakPoint, is created */
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
         ok(ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
         last_thread = ctx.ev.dwThreadId;
 
         if (ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT)
-            next_event(&ctx, 2000);
+            next_event(&ctx, WAIT_EVENT_TIMEOUT);
 
         ok(ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
         ok(ctx.ev.dwThreadId == last_thread, "unexpected thread\n");
         ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
            ctx.ev.u.Exception.ExceptionRecord.ExceptionCode);
-        ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress == pDbgBreakPoint, "ExceptionAddres != DbgBreakPoint\n");
+        ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress == pDbgBreakPoint, "ExceptionAddress != DbgBreakPoint\n");
 
         ret = SetEvent(event_attach);
         ok(ret, "SetEvent failed, last error %d.\n", GetLastError());
+
+        if (pass_exception)
+        {
+            ret = ContinueDebugEvent(ctx.ev.dwProcessId, ctx.ev.dwThreadId, DBG_EXCEPTION_NOT_HANDLED);
+            ok(ret, "ContinueDebugEvent failed, last error %d.\n", GetLastError());
+            ctx.ev.dwDebugEventCode = -1;
+        }
     }
 
-    do next_event(&ctx, 2000);
+    do next_event(&ctx, WAIT_EVENT_TIMEOUT);
     while (ctx.ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT || ctx.ev.dwDebugEventCode == UNLOAD_DLL_DEBUG_EVENT
            || ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT || ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT);
 
@@ -1029,15 +1131,316 @@ static void test_debug_children(char *name, DWORD flag, BOOL debug_child)
     ok(ret, "DeleteFileA failed, last error %d.\n", GetLastError());
 }
 
+static void wait_debugger(HANDLE event)
+{
+    WaitForSingleObject(event, INFINITE);
+    ExitProcess(0);
+}
+
+#define expect_event(a,b) expect_event_(__LINE__,a,b)
+static void expect_event_(unsigned line, struct debugger_context *ctx, DWORD event_code)
+{
+    next_event(ctx, WAIT_EVENT_TIMEOUT);
+    ok_(__FILE__,line)(ctx->ev.dwDebugEventCode == event_code, "dwDebugEventCode = %d expected %d\n",
+                       ctx->ev.dwDebugEventCode, event_code);
+}
+
+#define expect_exception(a,b) expect_exception_(__LINE__,a,b)
+static void expect_exception_(unsigned line, struct debugger_context *ctx, DWORD exception_code)
+{
+    expect_event_(line, ctx, EXCEPTION_DEBUG_EVENT);
+    ok_(__FILE__,line)(ctx->ev.u.Exception.ExceptionRecord.ExceptionCode == exception_code, "ExceptionCode = %x expected %x\n",
+                       ctx->ev.u.Exception.ExceptionRecord.ExceptionCode, exception_code);
+}
+
+#define expect_breakpoint_exception(a,b) expect_breakpoint_exception_(__LINE__,a,b)
+static void expect_breakpoint_exception_(unsigned line, struct debugger_context *ctx, const void *expect_addr)
+{
+    struct debuggee_thread *thread;
+    expect_exception_(line, ctx, EXCEPTION_BREAKPOINT);
+    if (!expect_addr) return;
+    ok_(__FILE__,line)(ctx->ev.u.Exception.ExceptionRecord.ExceptionAddress == expect_addr,
+                       "ExceptionAddress = %p expected %p\n", ctx->ev.u.Exception.ExceptionRecord.ExceptionAddress, expect_addr);
+    thread = get_debuggee_thread(ctx, ctx->ev.dwThreadId);
+    fetch_thread_context(thread);
+    ok_(__FILE__,line)(get_ip(&thread->ctx) == (char*)expect_addr + 1, "unexpected instruction pointer %p expected %p\n",
+                       get_ip(&thread->ctx), expect_addr);
+}
+
+#define single_step(a,b,c) single_step_(__LINE__,a,b,c)
+static void single_step_(unsigned line, struct debugger_context *ctx, struct debuggee_thread *thread, void *expect_addr)
+{
+#if defined(__i386__) || defined(__x86_64__)
+    fetch_thread_context(thread);
+    thread->ctx.EFlags |= 0x100;
+    set_thread_context(ctx, thread);
+    expect_exception_(line, ctx, EXCEPTION_SINGLE_STEP);
+    ok_(__FILE__,line)(ctx->ev.u.Exception.ExceptionRecord.ExceptionAddress == expect_addr,
+                       "ExceptionAddress = %p expected %p\n", ctx->ev.u.Exception.ExceptionRecord.ExceptionAddress, expect_addr);
+    fetch_thread_context(thread);
+    ok_(__FILE__,line)(get_ip(&thread->ctx) == expect_addr, "unexpected instruction pointer %p expected %p\n",
+                       get_ip(&thread->ctx), expect_addr);
+    ok_(__FILE__,line)(!(thread->ctx.EFlags & 0x100), "EFlags = %x\n", thread->ctx.EFlags);
+#endif
+}
+
+static const BYTE loop_code[] = {
+#if defined(__i386__) || defined(__x86_64__)
+    0x90,                         /* nop */
+    0x90,                         /* nop */
+    0x90,                         /* nop */
+    0xe9, 0xf8, 0xff, 0xff, 0xff  /* jmp $-8 */
+#endif
+};
+
+static const BYTE call_debug_service_code[] = {
+#ifdef __i386__
+    0x53,                         /* pushl %ebx */
+    0x57,                         /* pushl %edi */
+    0x8b, 0x44, 0x24, 0x0c,       /* movl 12(%esp),%eax */
+    0xb9, 0x11, 0x11, 0x11, 0x11, /* movl $0x11111111,%ecx */
+    0xba, 0x22, 0x22, 0x22, 0x22, /* movl $0x22222222,%edx */
+    0xbb, 0x33, 0x33, 0x33, 0x33, /* movl $0x33333333,%ebx */
+    0xbf, 0x44, 0x44, 0x44, 0x44, /* movl $0x44444444,%edi */
+    0xcd, 0x2d,                   /* int $0x2d */
+    0xeb,                         /* jmp $+17 */
+    0x0f, 0x1f, 0x00,             /* nop */
+    0x31, 0xc0,                   /* xorl %eax,%eax */
+    0xeb, 0x0c,                   /* jmp $+14 */
+    0x90, 0x90, 0x90, 0x90,       /* nop */
+    0x90, 0x90, 0x90, 0x90,
+    0x90,
+    0x31, 0xc0,                   /* xorl %eax,%eax */
+    0x40,                         /* incl %eax */
+    0x5f,                         /* popl %edi */
+    0x5b,                         /* popl %ebx */
+    0xc3,                         /* ret */
+#elif defined(__x86_64__)
+    0x53,                         /* push %rbx */
+    0x57,                         /* push %rdi */
+    0x48, 0x89, 0xc8,             /* movl %rcx,%rax */
+    0x48, 0xb9, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, /* movabs $0x1111111111111111,%rcx */
+    0x48, 0xba, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, /* movabs $0x2222222222222222,%rdx */
+    0x48, 0xbb, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, /* movabs $0x3333333333333333,%rbx */
+    0x48, 0xbf, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, /* movabs $0x4444444444444444,%rdi */
+    0xcd, 0x2d,                   /* int $0x2d */
+    0xeb,                         /* jmp $+17 */
+    0x0f, 0x1f, 0x00,             /* nop */
+    0x48, 0x31, 0xc0,             /* xor %rax,%rax */
+    0xeb, 0x0e,                   /* jmp $+16 */
+    0x90, 0x90, 0x90, 0x90,       /* nop */
+    0x90, 0x90, 0x90, 0x90,
+    0x48, 0x31, 0xc0,             /* xor %rax,%rax */
+    0x48, 0xff, 0xc0,             /* inc %rax */
+    0x5f,                         /* pop %rdi */
+    0x5b,                         /* pop %rbx */
+    0xc3,                         /* ret */
+#endif
+};
+
+#if defined(__i386__) || defined(__x86_64__)
+#define OP_BP 0xcc
+#else
+#define OP_BP 0
+#endif
+
+static void test_debugger(const char *argv0)
+{
+    static const char arguments[] = " debugger wait ";
+    SECURITY_ATTRIBUTES sa = { sizeof(sa), NULL, TRUE };
+    struct debugger_context ctx = { 0 };
+    PROCESS_INFORMATION pi;
+    STARTUPINFOA si;
+    HANDLE event, thread;
+    BYTE *mem, buf[4096], *proc_code, *thread_proc, byte;
+    unsigned int i, worker_cnt, exception_cnt;
+    struct debuggee_thread *debuggee_thread;
+    char *cmd;
+    BOOL ret;
+
+    event = CreateEventW(&sa, TRUE, FALSE, NULL);
+    ok(event != NULL, "CreateEvent failed: %u\n", GetLastError());
+
+    cmd = heap_alloc(strlen(argv0) + strlen(arguments) + 16);
+    sprintf(cmd, "%s%s%x\n", argv0, arguments, (DWORD)(DWORD_PTR)event);
+
+    memset(&si, 0, sizeof(si));
+    si.cb = sizeof(si);
+    ret = CreateProcessA(NULL, cmd, NULL, NULL, TRUE, DEBUG_PROCESS, NULL, NULL, &si, &pi);
+    ok(ret, "CreateProcess failed, last error %#x.\n", GetLastError());
+    heap_free(cmd);
+
+    next_event(&ctx, WAIT_EVENT_TIMEOUT);
+    ok(ctx.ev.dwDebugEventCode == CREATE_PROCESS_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+    wait_for_breakpoint(&ctx);
+    do next_event(&ctx, POLL_EVENT_TIMEOUT);
+    while(ctx.ev.dwDebugEventCode != -1);
+
+    mem = VirtualAllocEx(pi.hProcess, NULL, sizeof(buf), MEM_COMMIT, PAGE_EXECUTE_READWRITE);
+    ok(mem != NULL, "VirtualAllocEx failed: %u\n", GetLastError());
+    proc_code   = buf + 1024;
+    thread_proc = mem + 1024;
+
+    if (sizeof(loop_code) > 1)
+    {
+        /* test single-step exceptions */
+        memset(buf, OP_BP, sizeof(buf));
+        memcpy(proc_code, &loop_code, sizeof(loop_code));
+        proc_code[0] = OP_BP; /* set a breakpoint */
+        ret = WriteProcessMemory(pi.hProcess, mem, buf, sizeof(buf), NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        thread = CreateRemoteThread(pi.hProcess, NULL, 0, (void*)thread_proc, NULL, 0, NULL);
+        ok(thread != NULL, "CreateRemoteThread failed: %u\n", GetLastError());
+
+        expect_event(&ctx, CREATE_THREAD_DEBUG_EVENT);
+        debuggee_thread = get_debuggee_thread(&ctx, ctx.ev.dwThreadId);
+
+        wait_for_breakpoint(&ctx);
+        fetch_thread_context(debuggee_thread);
+        ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress == thread_proc,
+           "ExceptionAddress = %p\n", ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress);
+        ok(get_ip(&debuggee_thread->ctx) == thread_proc + 1, "unexpected instruction pointer %p\n",
+           get_ip(&debuggee_thread->ctx));
+
+        single_step(&ctx, debuggee_thread, thread_proc + 2);
+        single_step(&ctx, debuggee_thread, thread_proc + 3);
+        single_step(&ctx, debuggee_thread, thread_proc);
+
+        byte = 0xc3; /* ret */
+        ret = WriteProcessMemory(pi.hProcess, thread_proc, &byte, 1, NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        expect_event(&ctx, EXIT_THREAD_DEBUG_EVENT);
+    }
+    else win_skip("loop_code not supported on this architecture\n");
+
+    if (sizeof(call_debug_service_code) > 1)
+    {
+        /* test debug service exceptions */
+        memset(buf, OP_BP, sizeof(buf));
+        memcpy(proc_code, call_debug_service_code, sizeof(call_debug_service_code));
+        ret = WriteProcessMemory(pi.hProcess, mem, buf, sizeof(buf), NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        /* BREAKPOINT_PRINT */
+        thread = CreateRemoteThread(pi.hProcess, NULL, 0, (void*)thread_proc, (void*)2, 0, NULL);
+        ok(thread != NULL, "CreateRemoteThread failed: %u\n", GetLastError());
+        expect_event(&ctx, CREATE_THREAD_DEBUG_EVENT);
+        expect_breakpoint_exception(&ctx, NULL);
+        expect_event(&ctx, EXIT_THREAD_DEBUG_EVENT);
+
+        /* BREAKPOINT_PROMPT */
+        thread = CreateRemoteThread(pi.hProcess, NULL, 0, (void*)thread_proc, (void*)1, 0, NULL);
+        ok(thread != NULL, "CreateRemoteThread failed: %u\n", GetLastError());
+        expect_event(&ctx, CREATE_THREAD_DEBUG_EVENT);
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        /* some 32-bit Windows versions report exception to the debugger */
+        if (sizeof(void *) == 4 && ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT) next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        ok(ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT, "unexpected debug event %u\n", ctx.ev.dwDebugEventCode);
+    }
+    else win_skip("call_debug_service_code not supported on this architecture\n");
+
+    if (sizeof(loop_code) > 1)
+    {
+        struct debuggee_thread *prev_thread;
+
+        memset(buf, OP_BP, sizeof(buf));
+        memcpy(proc_code, &loop_code, sizeof(loop_code));
+        ret = WriteProcessMemory(pi.hProcess, mem, buf, sizeof(buf), NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        ctx.thread_tag = 1;
+
+        worker_cnt = 20;
+        for (i = 0; i < worker_cnt; i++)
+        {
+            thread = CreateRemoteThread(pi.hProcess, NULL, 0, (void*)thread_proc, NULL, 0, NULL);
+            ok(thread != NULL, "CreateRemoteThread failed: %u\n", GetLastError());
+
+            next_event(&ctx, WAIT_EVENT_TIMEOUT);
+            ok(ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+
+            ret = CloseHandle(thread);
+            ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+        }
+
+        byte = OP_BP;
+        ret = WriteProcessMemory(pi.hProcess, thread_proc + 1, &byte, 1, NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        expect_breakpoint_exception(&ctx, thread_proc + 1);
+        exception_cnt = 1;
+
+        prev_thread = ctx.current_thread;
+        fetch_process_context(&ctx);
+
+        byte = 0xc3; /* ret */
+        ret = WriteProcessMemory(pi.hProcess, thread_proc + 1, &byte, 1, NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        for (;;)
+        {
+            DEBUG_EVENT ev;
+
+            /* even when there are more pending events, they are not reported until current event is continued */
+            ret = WaitForDebugEvent(&ev, 10);
+            ok(GetLastError() == ERROR_SEM_TIMEOUT, "WaitForDebugEvent returned %x(%u)\n", ret, GetLastError());
+
+            next_event(&ctx, POLL_EVENT_TIMEOUT);
+            if (ctx.ev.dwDebugEventCode != EXCEPTION_DEBUG_EVENT) break;
+            trace("exception at %p in thread %04x\n", ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress, ctx.ev.dwThreadId);
+            ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
+               ctx.ev.u.Exception.ExceptionRecord.ExceptionCode);
+            ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress == thread_proc + 1,
+               "ExceptionAddress = %p\n", ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress);
+            ok(get_ip(&prev_thread->ctx) == thread_proc + 2
+               || broken(get_ip(&prev_thread->ctx) == thread_proc), /* sometimes observed on win10 */
+               "unexpected instruction pointer %p\n", get_ip(&prev_thread->ctx));
+            prev_thread = ctx.current_thread;
+            exception_cnt++;
+        }
+
+        /* for some reason sometimes on Windows one thread has a different address. this is always the thread
+         * with the last reported exception, so we simply skip the check for the last exception unless it's the only one. */
+        if (exception_cnt == 1)
+            ok(get_ip(&prev_thread->ctx) == thread_proc + 2, "unexpected instruction pointer %p\n", get_ip(&prev_thread->ctx));
+
+        trace("received %u exceptions\n", exception_cnt);
+
+        for (;;)
+        {
+            ok(ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT
+               || broken(ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT), /* sometimes happens on vista */
+               "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+            if (ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT && !--worker_cnt) break;
+            next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        }
+    }
+
+    SetEvent(event);
+
+    do
+    {
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        ok (ctx.ev.dwDebugEventCode != EXCEPTION_DEBUG_EVENT, "got exception\n");
+    }
+    while (ctx.ev.dwDebugEventCode != EXIT_PROCESS_DEBUG_EVENT);
+
+    ret = CloseHandle(event);
+    ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+    ret = CloseHandle(pi.hThread);
+    ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+    ret = CloseHandle(pi.hProcess);
+    ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+}
+
 START_TEST(debugger)
 {
     HMODULE hdll;
 
     hdll=GetModuleHandleA("kernel32.dll");
     pCheckRemoteDebuggerPresent=(void*)GetProcAddress(hdll, "CheckRemoteDebuggerPresent");
-    pDebugActiveProcessStop=(void*)GetProcAddress(hdll, "DebugActiveProcessStop");
-    pDebugSetProcessKillOnExit=(void*)GetProcAddress(hdll, "DebugSetProcessKillOnExit");
-    pIsDebuggerPresent=(void*)GetProcAddress(hdll, "IsDebuggerPresent");
 
     ntdll = GetModuleHandleA("ntdll.dll");
     pDbgBreakPoint = (void*)GetProcAddress(ntdll, "DbgBreakPoint");
@@ -1045,7 +1448,7 @@ START_TEST(debugger)
     myARGC=winetest_get_mainargs(&myARGV);
     if (myARGC >= 3 && strcmp(myARGV[2], "crash") == 0)
     {
-        doCrash(myARGC, myARGV);
+        doCrash();
     }
     else if (myARGC >= 3 && strncmp(myARGV[2], "dbg,", 4) == 0)
     {
@@ -1059,14 +1462,22 @@ START_TEST(debugger)
     {
         doChildren(myARGC, myARGV);
     }
+    else if (myARGC >= 4 && !strcmp(myARGV[2], "wait"))
+    {
+        DWORD event;
+        sscanf(myARGV[3], "%x", &event);
+        wait_debugger((HANDLE)(DWORD_PTR)event);
+    }
     else
     {
         test_ExitCode();
         test_RemoteDebugger();
         test_debug_loop(myARGC, myARGV);
-        test_debug_children(myARGV[0], DEBUG_PROCESS, TRUE);
-        test_debug_children(myARGV[0], DEBUG_ONLY_THIS_PROCESS, FALSE);
-        test_debug_children(myARGV[0], DEBUG_PROCESS|DEBUG_ONLY_THIS_PROCESS, FALSE);
-        test_debug_children(myARGV[0], 0, FALSE);
+        test_debug_children(myARGV[0], DEBUG_PROCESS, TRUE, FALSE);
+        test_debug_children(myARGV[0], DEBUG_ONLY_THIS_PROCESS, FALSE, FALSE);
+        test_debug_children(myARGV[0], DEBUG_PROCESS|DEBUG_ONLY_THIS_PROCESS, FALSE, FALSE);
+        test_debug_children(myARGV[0], 0, FALSE, FALSE);
+        test_debug_children(myARGV[0], 0, FALSE, TRUE);
+        test_debugger(myARGV[0]);
     }
 }
