@@ -157,6 +157,31 @@ static COORD get_largest_console_window_size( HANDLE handle )
     return c;
 }
 
+/* helper function to replace OpenConsoleW */
+HANDLE open_console( BOOL output, DWORD access, SECURITY_ATTRIBUTES *sa, DWORD creation )
+{
+    HANDLE ret;
+
+    if (creation != OPEN_EXISTING)
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return INVALID_HANDLE_VALUE;
+    }
+
+    SERVER_START_REQ( open_console )
+    {
+        req->from       = wine_server_obj_handle( ULongToHandle( output ));
+        req->access     = access;
+        req->attributes = sa && sa->bInheritHandle ? OBJ_INHERIT : 0;
+        req->share      = FILE_SHARE_READ | FILE_SHARE_WRITE;
+        wine_server_call_err( req );
+        ret = wine_server_ptr_handle( reply->handle );
+    }
+    SERVER_END_REQ;
+    if (ret) ret = console_handle_map( ret );
+    return ret;
+}
+
 
 /******************************************************************
  *	AttachConsole   (kernelbase.@)
@@ -638,46 +663,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH PeekConsoleInputW( HANDLE handle, INPUT_RECORD *bu
 }
 
 
-/***********************************************************************
- *	ReadConsoleA   (kernelbase.@)
- */
-BOOL WINAPI DECLSPEC_HOTPATCH ReadConsoleA( HANDLE handle, LPVOID buffer, DWORD length,
-                                            DWORD *ret_count, void *reserved )
-{
-    LPWSTR strW = HeapAlloc( GetProcessHeap(), 0, length * sizeof(WCHAR) );
-    DWORD count = 0;
-    BOOL ret;
-
-    if (!strW)
-    {
-        SetLastError( ERROR_NOT_ENOUGH_MEMORY );
-        return FALSE;
-    }
-    if ((ret = ReadConsoleW( handle, strW, length, &count, NULL )))
-    {
-        count = WideCharToMultiByte( GetConsoleCP(), 0, strW, count, buffer, length, NULL, NULL );
-        if (ret_count) *ret_count = count;
-    }
-    HeapFree( GetProcessHeap(), 0, strW );
-    return ret;
-}
-
-
-/******************************************************************************
- *	ReadConsoleInputA   (kernelbase.@)
- */
-BOOL WINAPI DECLSPEC_HOTPATCH ReadConsoleInputA( HANDLE handle, INPUT_RECORD *buffer,
-                                                 DWORD length, DWORD *count )
-{
-    DWORD read;
-
-    if (!ReadConsoleInputW( handle, buffer, length, &read )) return FALSE;
-    input_records_WtoA( buffer, read );
-    if (count) *count = read;
-    return TRUE;
-}
-
-
 /******************************************************************************
  *	ReadConsoleOutputAttribute   (kernelbase.@)
  */
@@ -1264,27 +1249,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleWindowInfo( HANDLE handle, BOOL absolute
 }
 
 
-/***********************************************************************
- *	WriteConsoleA   (kernelbase.@)
- */
-BOOL WINAPI DECLSPEC_HOTPATCH WriteConsoleA( HANDLE handle, LPCVOID buffer, DWORD length,
-                                             DWORD *written, void *reserved )
-{
-    UINT cp = GetConsoleOutputCP();
-    LPWSTR strW;
-    DWORD lenW;
-    BOOL ret;
-
-    if (written) *written = 0;
-    lenW = MultiByteToWideChar( cp, 0, buffer, length, NULL, 0 );
-    if (!(strW = HeapAlloc( GetProcessHeap(), 0, lenW * sizeof(WCHAR) ))) return FALSE;
-    MultiByteToWideChar( cp, 0, buffer, length, strW, lenW );
-    ret = WriteConsoleW( handle, strW, lenW, written, 0 );
-    HeapFree( GetProcessHeap(), 0, strW );
-    return ret;
-}
-
-
 /******************************************************************************
  *	WriteConsoleInputA   (kernelbase.@)
  */
