@@ -129,8 +129,8 @@ static HANDLE open_vxd_handle( LPCWSTR name )
         return 0;
     }
     memcpy( nameW.Buffer, prefixW, sizeof(prefixW) );
-    MultiByteToWideChar( CP_UNIXCP, 0, dir, -1, nameW.Buffer + sizeof(prefixW)/sizeof(WCHAR), len );
-    len += sizeof(prefixW) / sizeof(WCHAR);
+    MultiByteToWideChar( CP_UNIXCP, 0, dir, -1, nameW.Buffer + ARRAY_SIZE(prefixW), len );
+    len += ARRAY_SIZE(prefixW);
     nameW.Buffer[len-1] = '/';
     strcpyW( nameW.Buffer + len, name );
 
@@ -201,7 +201,7 @@ HANDLE __wine_vxd_open( LPCWSTR filenameW, DWORD access, SECURITY_ATTRIBUTES *sa
 
     /* normalize the filename */
 
-    if (strlenW( filenameW ) >= sizeof(name)/sizeof(WCHAR) - 4 ||
+    if (strlenW( filenameW ) >= ARRAY_SIZE(name) - 4 ||
         strchrW( filenameW, '/' ) || strchrW( filenameW, '\\' ))
     {
         SetLastError( ERROR_FILE_NOT_FOUND );
@@ -1494,7 +1494,7 @@ void WINAPI __wine_vxd_win32s( CONTEXT *context )
          *   HANDLE32 SectionHandle       [in]     Section to be mapped
          *   DWORD    ProcessHandle       [in]     Process to be mapped into
          *   DWORD *  BaseAddress         [in/out] Address to be mapped at
-         *   DWORD    ZeroBits            [in]     (?? unclear ??)
+         *   DWORD    ZeroBits            [in]     Upper address bits that should be null, starting from bit 31
          *   DWORD    CommitSize          [in]     (?? unclear ??)
          *   LARGE_INTEGER *SectionOffset [in]     Offset within section
          *   DWORD *  ViewSize            [in]     Size of view
