@@ -18,8 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 
 #define COBJMACROS
@@ -33,8 +31,6 @@
 #include "winerror.h"
 #include "objbase.h"
 
-#include "wine/unicode.h"
-
 #include "msctf.h"
 #include "msctf_internal.h"
 
@@ -81,11 +77,15 @@ static inline EnumTfContext *impl_from_IEnumTfContexts(IEnumTfContexts *iface)
 
 static void DocumentMgr_Destructor(DocumentMgr *This)
 {
-    ITfThreadMgr *tm;
+    ITfThreadMgr *tm = NULL;
     TRACE("destroying %p\n", This);
 
     TF_GetThreadMgr(&tm);
-    ThreadMgr_OnDocumentMgrDestruction(tm, &This->ITfDocumentMgr_iface);
+    if (tm)
+    {
+        ThreadMgr_OnDocumentMgrDestruction(tm, &This->ITfDocumentMgr_iface);
+        ITfThreadMgr_Release(tm);
+    }
 
     if (This->contextStack[0])
         ITfContext_Release(This->contextStack[0]);
