@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 #include <stdio.h>
 #include <assert.h>
@@ -924,7 +922,6 @@ static HRESULT WINAPI OleControl_OnAmbientPropertyChange(IOleControl *iface, DIS
         if(V_VT(&res) == VT_BOOL) {
             if(V_BOOL(&res)) {
                 FIXME("silent mode is not supported\n");
-                hres = E_FAIL;
             }
         }else {
             FIXME("silent=%s\n", debugstr_variant(&res));
@@ -937,7 +934,6 @@ static HRESULT WINAPI OleControl_OnAmbientPropertyChange(IOleControl *iface, DIS
             return S_OK;
 
         FIXME("not supported AMBIENT_USERAGENT\n");
-        hres = E_FAIL;
         return S_OK;
     case DISPID_AMBIENT_PALETTE:
         TRACE("(%p)->(DISPID_AMBIENT_PALETTE)\n", This);
@@ -946,7 +942,6 @@ static HRESULT WINAPI OleControl_OnAmbientPropertyChange(IOleControl *iface, DIS
             return S_OK;
 
         FIXME("not supported AMBIENT_PALETTE\n");
-        hres = E_FAIL;
         return S_OK;
     }
 
@@ -1171,11 +1166,18 @@ static HRESULT WINAPI OleInPlaceObjectWindowless_UIDeactivate(IOleInPlaceObjectW
 }
 
 static HRESULT WINAPI OleInPlaceObjectWindowless_SetObjectRects(IOleInPlaceObjectWindowless *iface,
-        LPCRECT lprcPosRect, LPCRECT lprcClipRect)
+        const RECT *pos, const RECT *clip)
 {
     HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
-    FIXME("(%p)->(%p %p)\n", This, lprcPosRect, lprcClipRect);
-    return E_NOTIMPL;
+    RECT r;
+
+    TRACE("(%p)->(%s %s)\n", This, wine_dbgstr_rect(pos), wine_dbgstr_rect(clip));
+
+    if(clip && !EqualRect(clip, pos))
+        FIXME("Ignoring clip rect %s\n", wine_dbgstr_rect(clip));
+
+    r = *pos;
+    return IOleDocumentView_SetRect(&This->doc_obj->IOleDocumentView_iface, &r);
 }
 
 static HRESULT WINAPI OleInPlaceObjectWindowless_ReactivateAndUndo(IOleInPlaceObjectWindowless *iface)
