@@ -21,7 +21,12 @@ function guard(f) {
         try {
             f();
         }catch(e) {
-            ok(false, "Got exception " + ("message" in e ? e.message : e));
+            var msg = "Got exception ";
+            if(e && typeof(e) == "object" && "message")
+                msg += e.msg;
+            else
+                msg += e;
+            ok(false, msg);
         }
     };
 }
@@ -37,11 +42,11 @@ function run_tests() {
 }
 
 function ok(b,m) {
-    return external.ok(b, m);
+    return external.ok(b, format_message(m));
 }
 
 function trace(m) {
-    external.trace(m);
+    external.trace(format_message(m));
 }
 
 function win_skip(m) {
@@ -51,3 +56,22 @@ function win_skip(m) {
 function reportSuccess() {
     external.reportSuccess();
 }
+
+var todo_wine = {
+    ok: function(b,m) {
+        return external.todo_wine_ok(b, format_message(m));
+    }
+};
+
+function todo_wine_if(expr) {
+    return expr ? todo_wine : { ok: ok };
+}
+
+var file_prefix = document.location.pathname;
+if(document.location.search)
+    file_prefix += document.location.search;
+file_prefix += ": ";
+
+function format_message(msg) {
+    return file_prefix + msg;
+}
