@@ -33,7 +33,6 @@
 #include "query.h"
 
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msidb);
 
@@ -615,7 +614,7 @@ UINT msi_commit_streams( MSIDATABASE *db )
     for (i = 0; i < db->num_streams; i++)
     {
         name = msi_string_lookup( db->strings, db->streams[i].str_index, NULL );
-        if (!strcmpW( name, szSumInfo )) continue;
+        if (!wcscmp( name, szSumInfo )) continue;
 
         if (!(encname = encode_streamname( FALSE, name ))) return ERROR_OUTOFMEMORY;
         TRACE("saving stream %s as %s\n", debugstr_w(name), debugstr_w(encname));
