@@ -118,22 +118,22 @@ static inline unsigned char* u_strncat( unsigned char* dst, const unsigned char*
 
 static inline int u_strcmp( const unsigned char *s1, const unsigned char *s2 )
 {
-  return strcmp( (const char*)s1, (const char*)s2 );
+  return MSVCRT_strcmp( (const char*)s1, (const char*)s2 );
 }
 
 static inline int u_strcasecmp( const unsigned char *s1, const unsigned char *s2 )
 {
-  return strcasecmp( (const char*)s1, (const char*)s2 );
+  return MSVCRT__stricmp( (const char*)s1, (const char*)s2 );
 }
 
 static inline int u_strncmp( const unsigned char *s1, const unsigned char *s2, MSVCRT_size_t len )
 {
-  return strncmp( (const char*)s1, (const char*)s2, len );
+  return MSVCRT_strncmp( (const char*)s1, (const char*)s2, len );
 }
 
 static inline int u_strncasecmp( const unsigned char *s1, const unsigned char *s2, MSVCRT_size_t len )
 {
-  return strncasecmp( (const char*)s1, (const char*)s2, len );
+  return MSVCRT__strnicmp( (const char*)s1, (const char*)s2, len );
 }
 
 static inline unsigned char *u_strchr( const unsigned char *s, unsigned char x )
@@ -156,11 +156,6 @@ static inline unsigned char *u__strnset( unsigned char *s, unsigned char c, MSVC
   return (unsigned char*) MSVCRT__strnset( (char*)s, c, len );
 }
 
-static inline MSVCRT_size_t u_strcspn( const unsigned char *s, const unsigned char *rej )
-{
-  return strcspn( (const char *)s, (const char*)rej );
-}
-
 /*********************************************************************
  *		__p__mbctype (MSVCRT.@)
  */
@@ -404,7 +399,7 @@ unsigned int CDECL _mbctolower(unsigned int c)
       FIXME("Handle MBC chars\n");
       return c;
     }
-    return tolower(c); /* ASCII CP or SB char */
+    return MSVCRT__tolower_l(c, NULL); /* ASCII CP or SB char */
 }
 
 /*********************************************************************
@@ -417,7 +412,7 @@ unsigned int CDECL _mbctoupper(unsigned int c)
       FIXME("Handle MBC chars\n");
       return c;
     }
-    return toupper(c); /* ASCII CP or SB char */
+    return MSVCRT__toupper_l(c, NULL); /* ASCII CP or SB char */
 }
 
 /*********************************************************************
@@ -1216,7 +1211,7 @@ unsigned char* CDECL _mbscpy( unsigned char *dst, const unsigned char *src )
  */
 unsigned char * CDECL _mbsstr(const unsigned char *haystack, const unsigned char *needle)
 {
-    return (unsigned char *)strstr( (const char *)haystack, (const char *)needle );
+    return (unsigned char *)MSVCRT_strstr( (const char *)haystack, (const char *)needle );
 }
 
 /*********************************************************************
@@ -1551,13 +1546,35 @@ int CDECL _ismbblead(unsigned int c)
     return _ismbblead_l(c, NULL);
 }
 
+/*********************************************************************
+ *              _ismbbtrail_l(MSVCRT.@)
+ */
+int CDECL _ismbbtrail_l(unsigned int c, MSVCRT__locale_t locale)
+{
+    MSVCRT_pthreadmbcinfo mbcinfo;
+
+    if(!locale)
+        mbcinfo = get_mbcinfo();
+    else
+        mbcinfo = locale->mbcinfo;
+
+    return (mbcinfo->mbctype[(c&0xff) + 1] & _M2) != 0;
+}
 
 /*********************************************************************
  *		_ismbbtrail(MSVCRT.@)
  */
 int CDECL _ismbbtrail(unsigned int c)
 {
-  return (get_mbcinfo()->mbctype[(c&0xff) + 1] & _M2) != 0;
+    return _ismbbtrail_l(c, NULL);
+}
+
+/*********************************************************************
+ *              _ismbclegal_l(MSVCRT.@)
+ */
+int CDECL _ismbclegal_l(unsigned int c, MSVCRT__locale_t locale)
+{
+    return _ismbblead_l(HIBYTE(c), locale) && _ismbbtrail_l(LOBYTE(c), locale);
 }
 
 /*********************************************************************
@@ -1565,7 +1582,7 @@ int CDECL _ismbbtrail(unsigned int c)
  */
 int CDECL _ismbclegal(unsigned int c)
 {
-    return _ismbblead(HIBYTE(c)) && _ismbbtrail(LOBYTE(c));
+    return _ismbclegal_l(c, NULL);
 }
 
 /*********************************************************************
@@ -1887,7 +1904,7 @@ unsigned char* CDECL _mbslwr(unsigned char* s)
       *s++=c;
     }
   }
-  else for ( ; *s; s++) *s = tolower(*s);
+  else for ( ; *s; s++) *s = MSVCRT__tolower_l(*s, NULL);
   return ret;
 }
 
@@ -1921,7 +1938,7 @@ int CDECL _mbslwr_s(unsigned char* s, MSVCRT_size_t len)
       *s++=c;
     }
   }
-  else for ( ; *s && len > 0; s++, len--) *s = tolower(*s);
+  else for ( ; *s && len > 0; s++, len--) *s = MSVCRT__tolower_l(*s, NULL);
   if (*s)
   {
     *s = '\0';
@@ -1955,7 +1972,7 @@ unsigned char* CDECL _mbsupr(unsigned char* s)
       *s++=c;
     }
   }
-  else for ( ; *s; s++) *s = toupper(*s);
+  else for ( ; *s; s++) *s = MSVCRT__toupper_l(*s, NULL);
   return ret;
 }
 
@@ -1989,7 +2006,7 @@ int CDECL _mbsupr_s(unsigned char* s, MSVCRT_size_t len)
       *s++=c;
     }
   }
-  else for ( ; *s && len > 0; s++, len--) *s = toupper(*s);
+  else for ( ; *s && len > 0; s++, len--) *s = MSVCRT__toupper_l(*s, NULL);
   if (*s)
   {
     *s = '\0';
@@ -1999,11 +2016,11 @@ int CDECL _mbsupr_s(unsigned char* s, MSVCRT_size_t len)
   return 0;
 }
 
-
 /*********************************************************************
- *              _mbsspn (MSVCRT.@)
+ *              _mbsspn_l (MSVCRT.@)
  */
-MSVCRT_size_t CDECL _mbsspn(const unsigned char* string, const unsigned char* set)
+MSVCRT_size_t CDECL _mbsspn_l(const unsigned char* string,
+        const unsigned char* set, MSVCRT__locale_t locale)
 {
     const unsigned char *p, *q;
 
@@ -2011,8 +2028,11 @@ MSVCRT_size_t CDECL _mbsspn(const unsigned char* string, const unsigned char* se
     {
         for (q = set; *q; q++)
         {
-            if (_ismbblead(*q))
+            if (_ismbblead_l(*q, locale))
             {
+                /* duplicate a bug in native implementation */
+                if (!q[1]) break;
+
                 if (p[0] == q[0] && p[1] == q[1])
                 {
                     p++;
@@ -2030,6 +2050,14 @@ MSVCRT_size_t CDECL _mbsspn(const unsigned char* string, const unsigned char* se
     return p - string;
 }
 
+/*********************************************************************
+ *              _mbsspn (MSVCRT.@)
+ */
+MSVCRT_size_t CDECL _mbsspn(const unsigned char* string, const unsigned char* set)
+{
+    return _mbsspn_l(string, set, NULL);
+}
+
 /*********************************************************************
  *              _mbsspnp (MSVCRT.@)
  */
@@ -2040,13 +2068,39 @@ unsigned char* CDECL _mbsspnp(const unsigned char* string, const unsigned char*
 }
 
 /*********************************************************************
- *		_mbscspn(MSVCRT.@)
+ *		_mbscspn_l (MSVCRT.@)
+ */
+MSVCRT_size_t CDECL _mbscspn_l(const unsigned char* str,
+        const unsigned char* cmp, MSVCRT__locale_t locale)
+{
+    const unsigned char *p, *q;
+
+    for (p = str; *p; p++)
+    {
+        for (q = cmp; *q; q++)
+        {
+            if (_ismbblead_l(*q, locale))
+            {
+                /* duplicate a bug in native implementation */
+                if (!q[1]) return 0;
+
+                if (p[0] == q[0] && p[1] == q[1])
+                    return p - str;
+                q++;
+            }
+            else if (p[0] == q[0])
+                return p - str;
+        }
+    }
+    return p - str;
+}
+
+/*********************************************************************
+ *		_mbscspn (MSVCRT.@)
  */
 MSVCRT_size_t CDECL _mbscspn(const unsigned char* str, const unsigned char* cmp)
 {
-  if (get_mbcinfo()->ismbcodepage)
-    FIXME("don't handle double character case\n");
-  return u_strcspn(str, cmp);
+    return _mbscspn_l(str, cmp, NULL);
 }
 
 /*********************************************************************
@@ -2348,7 +2402,7 @@ MSVCRT_size_t CDECL MSVCRT__mbstowcs_l(MSVCRT_wchar_t *wcstr, const char *mbstr,
         }
     }
 
-    if(size<count && wcstr)
+    if(size<count)
         wcstr[size] = '\0';
 
     return size;
@@ -2371,6 +2425,7 @@ int CDECL MSVCRT__mbstowcs_s_l(MSVCRT_size_t *ret, MSVCRT_wchar_t *wcstr,
         MSVCRT_size_t count, MSVCRT__locale_t locale)
 {
     MSVCRT_size_t conv;
+    int err = 0;
 
     if(!wcstr && !size) {
         conv = MSVCRT__mbstowcs_l(NULL, mbstr, 0, locale);
@@ -2393,9 +2448,10 @@ int CDECL MSVCRT__mbstowcs_s_l(MSVCRT_size_t *ret, MSVCRT_wchar_t *wcstr,
     conv = MSVCRT__mbstowcs_l(wcstr, mbstr, conv, locale);
     if(conv<size)
         wcstr[conv++] = '\0';
-    else if(conv==size && (count==MSVCRT__TRUNCATE || wcstr[conv-1]=='\0'))
+    else if(conv==size && count==MSVCRT__TRUNCATE && wcstr[conv-1]!='\0') {
         wcstr[conv-1] = '\0';
-    else {
+        err = MSVCRT_STRUNCATE;
+    }else if(conv==size && wcstr[conv-1]!='\0') {
         MSVCRT_INVALID_PMT("wcstr[size] is too small", MSVCRT_ERANGE);
         if(size)
             wcstr[0] = '\0';
@@ -2404,7 +2460,7 @@ int CDECL MSVCRT__mbstowcs_s_l(MSVCRT_size_t *ret, MSVCRT_wchar_t *wcstr,
 
     if(ret)
         *ret = conv;
-    return 0;
+    return err;
 }
 
 /*********************************************************************
@@ -2504,16 +2560,89 @@ unsigned int CDECL _mbctokata(unsigned int c)
     return c;
 }
 
+/*********************************************************************
+ *		_ismbcl0_l (MSVCRT.@)
+ */
+int CDECL _ismbcl0_l(unsigned int c, MSVCRT__locale_t locale)
+{
+    MSVCRT_pthreadmbcinfo mbcinfo;
+
+    if(!locale)
+        mbcinfo = get_mbcinfo();
+    else
+        mbcinfo = locale->mbcinfo;
+
+    if(mbcinfo->mbcodepage == 932)
+    {
+        /* JIS non-Kanji */
+        return _ismbclegal_l(c, locale) && c >= 0x8140 && c <= 0x889e;
+    }
+
+    return 0;
+}
 
 /*********************************************************************
  *		_ismbcl0 (MSVCRT.@)
  */
 int CDECL _ismbcl0(unsigned int c)
 {
-  if(get_mbcinfo()->mbcodepage == 932)
-  {
-    /* JIS non-Kanji */
-    return (c >= 0x8140 && c <= 0x889e);
-  }
-  return 0;
+    return _ismbcl0_l(c, NULL);
+}
+
+/*********************************************************************
+ *		_ismbcl1_l (MSVCRT.@)
+ */
+int CDECL _ismbcl1_l(unsigned int c, MSVCRT__locale_t locale)
+{
+    MSVCRT_pthreadmbcinfo mbcinfo;
+
+    if(!locale)
+        mbcinfo = get_mbcinfo();
+    else
+        mbcinfo = locale->mbcinfo;
+
+    if(mbcinfo->mbcodepage == 932)
+    {
+        /* JIS level-1 */
+        return _ismbclegal_l(c, locale) && c >= 0x889f && c <= 0x9872;
+    }
+
+    return 0;
+}
+
+/*********************************************************************
+ *		_ismbcl1 (MSVCRT.@)
+ */
+int CDECL _ismbcl1(unsigned int c)
+{
+    return _ismbcl1_l(c, NULL);
+}
+
+/*********************************************************************
+ *		_ismbcl2_l (MSVCRT.@)
+ */
+int CDECL _ismbcl2_l(unsigned int c, MSVCRT__locale_t locale)
+{
+    MSVCRT_pthreadmbcinfo mbcinfo;
+
+    if(!locale)
+        mbcinfo = get_mbcinfo();
+    else
+        mbcinfo = locale->mbcinfo;
+
+    if(mbcinfo->mbcodepage == 932)
+    {
+        /* JIS level-2 */
+        return _ismbclegal_l(c, locale) && c >= 0x989f && c <= 0xeaa4;
+    }
+
+    return 0;
+}
+
+/*********************************************************************
+ *		_ismbcl2 (MSVCRT.@)
+ */
+int CDECL _ismbcl2(unsigned int c)
+{
+    return _ismbcl2_l(c, NULL);
 }
