@@ -36,6 +36,20 @@
 #define MINSPERHOUR        60
 #define HOURSPERDAY        24
 
+typedef struct {
+    const char *short_wday[7];
+    const char *wday[7];
+    const char *short_mon[12];
+    const char *mon[12];
+    const char *am;
+    const char *pm;
+    const char *short_date;
+    const char *date;
+    const char *time;
+    LCID lcid;
+    int  unk[2];
+} __lc_time_data;
+
 static __time32_t (__cdecl *p_mkgmtime32)(struct tm*);
 static struct tm* (__cdecl *p_gmtime32)(__time32_t*);
 static struct tm* (__cdecl *p_gmtime)(time_t*);
@@ -50,6 +64,7 @@ static long*      (__cdecl *p___p__dstbias)(void);
 static long*      (__cdecl *p__dstbias)(void);
 static long*      (__cdecl *p___p__timezone)(void);
 static size_t     (__cdecl *p_strftime)(char *, size_t, const char *, const struct tm *);
+static size_t (__cdecl *p__Strftime)(char*, size_t, const char*, const struct tm*, void*);
 static size_t     (__cdecl *p_wcsftime)(wchar_t *, size_t, const wchar_t *, const struct tm *);
 static char*      (__cdecl *p_asctime)(const struct tm *);
 
@@ -71,6 +86,7 @@ static void init(void)
     p__dstbias = (void*)GetProcAddress(hmod, "__dstbias");
     p___p__timezone = (void*)GetProcAddress(hmod, "__p__timezone");
     p_strftime = (void*)GetProcAddress(hmod, "strftime");
+    p__Strftime = (void*)GetProcAddress(hmod, "_Strftime");
     p_wcsftime = (void*)GetProcAddress(hmod, "wcsftime");
     p_asctime = (void*)GetProcAddress(hmod, "asctime");
 }
@@ -577,7 +593,7 @@ static void test_daylight(void)
 
     if (!p___p__daylight)
     {
-        win_skip("__p__daylight not available\n");
+        skip("__p__daylight not available\n");
         return;
     }
 
@@ -588,13 +604,137 @@ static void test_daylight(void)
 
 static void test_strftime(void)
 {
+    const struct {
+        const char *format;
+    } tests_einval[] = {
+        {"%C"},
+        {"%D"},
+        {"%e"},
+        {"%F"},
+        {"%h"},
+        {"%n"},
+        {"%R"},
+        {"%t"},
+        {"%T"},
+        {"%u"},
+    };
+
+    const struct {
+        const char *format;
+        const char *ret;
+        struct tm tm;
+        BOOL todo;
+    } tests[] = {
+        {"e%#%e", "e%e", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%c", "01/01/70 00:00:00", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%c", "02/30/70 00:00:00", { 0, 0, 0, 30, 1, 70, 4, 0, 0 }},
+        {"%#c", "Thursday, January 01, 1970 00:00:00", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%#c", "Thursday, February 30, 1970 00:00:00", { 0, 0, 0, 30, 1, 70, 4, 0, 0 }},
+        {"%x", "01/01/70", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%x", "02/30/70", { 0, 0, 0, 30, 1, 70, 4, 0, 0 }},
+        {"%#x", "Thursday, January 01, 1970", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%#x", "Thursday, February 30, 1970", { 0, 0, 0, 30, 1, 70, 4, 0, 0 }},
+        {"%X", "00:00:00", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%X", "14:00:00", { 0, 0, 14, 1, 0, 70, 4, 0, 0 }},
+        {"%a", "Thu", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%A", "Thursday", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%b", "Jan", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%B", "January", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%d", "01", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%#d", "1", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%H", "00", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%I", "12", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%j", "001", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%m", "01", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%#M", "0", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%p", "AM", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%U", "00", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%W", "00", { 0, 0, 0, 1, 0, 70, 4, 0, 0 }},
+        {"%U", "01", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        {"%W", "00", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        {"%U", "53", { 0, 0, 0, 1, 0, 70, 0, 365, 0 }},
+        {"%W", "52", { 0, 0, 0, 1, 0, 70, 0, 365, 0 }},
+    };
+
+    const struct {
+        const char *format;
+        const char *ret;
+        const char *short_date;
+        const char *date;
+        const char *time;
+        struct tm tm;
+        BOOL todo;
+    } tests_td[] = {
+        { "%c", "x z", "x", "y", "z", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#c", "y z", "x", "y", "z", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "m1", 0, 0, "MMM", { 0, 0, 1, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "1", 0, 0, "h", { 0, 0, 1, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "01", 0, 0, "hh", { 0, 0, 1, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "h01", 0, 0, "hhh", { 0, 0, 1, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "hh01", 0, 0, "hhhh", { 0, 0, 1, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "1", 0, 0, "H", { 0, 0, 1, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "01", 0, 0, "HH", { 0, 0, 1, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "H13", 0, 0, "HHH", { 0, 0, 13, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "0", 0, 0, "m", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "00", 0, 0, "mm", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "0", 0, 0, "s", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "00", 0, 0, "ss", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "s00", 0, 0, "sss", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "t", 0, 0, "t", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "tam", 0, 0, "tt", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "tam", 0, 0, "ttttttttt", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "tam", 0, 0, "a", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "tam", 0, 0, "aaaaa", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "tam", 0, 0, "A", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%X", "tam", 0, 0, "AAAAA", { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "1", "d", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "01", "dd", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "d1", "ddd", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "day1", "dddd", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "dday1", "ddddd", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "1", "M", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "01", "MM", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "m1", "MMM", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "mon1", "MMMM", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "Mmon1", "MMMMM", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "y", "y", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "70", "yy", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "y70", "yyy", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "1970", "yyyy", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "y1970", "yyyyy", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%x", "ggggggggggg", "ggggggggggg", 0, 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "1", 0, "d", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "01", 0, "dd", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "d1", 0, "ddd", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "day1", 0, "dddd", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "dday1", 0, "ddddd", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "1", 0, "M", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "01", 0, "MM", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "m1", 0, "MMM", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "mon1", 0, "MMMM", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "Mmon1", 0, "MMMMM", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "y", 0, "y", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "70", 0, "yy", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "y70", 0, "yyy", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "1970", 0, "yyyy", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+        { "%#x", "y1970", 0, "yyyyy", 0, { 0, 0, 0, 1, 0, 70, 0, 0, 0 }},
+    };
+
+    __lc_time_data time_data = {
+        { "d1", "d2", "d3", "d4", "d5", "d6", "d7" },
+        { "day1", "day2", "day3", "day4", "day5", "day6", "day7" },
+        { "m1", "m2", "m3", "m4", "m5", "m6", "m7", "m8", "m9", "m10", "m11", "m12" },
+        { "mon1", "mon2", "mon3", "mon4", "mon5", "mon6", "mon7", "mon8", "mon9", "mon10", "mon11", "mon12" },
+        "tam", "tpm"
+    };
+
     static const wchar_t cW[] = { '%','c',0 };
-    static const char expected[] = "01/01/70 00:00:00";
     time_t gmt;
     struct tm* gmt_tm;
     char buf[256], bufA[256];
     WCHAR bufW[256];
     long retA, retW;
+    int i;
 
     if (!p_strftime || !p_wcsftime || !p_gmtime)
     {
@@ -608,6 +748,14 @@ static void test_strftime(void)
     gmt_tm = p_gmtime(&gmt);
     ok(gmt_tm != NULL, "gmtime failed\n");
 
+    for (i=0; i<ARRAY_SIZE(tests_einval); i++)
+    {
+        errno = 0xdeadbeef;
+        retA = p_strftime(bufA, 256, tests_einval[i].format, gmt_tm);
+        ok(retA == 0, "%d) ret = %ld\n", i, retA);
+        ok(errno==EINVAL || broken(errno==0xdeadbeef), "%d) errno = %d\n", i, errno);
+    }
+
     errno = 0xdeadbeef;
     retA = p_strftime(NULL, 0, "copy", gmt_tm);
     ok(retA == 0, "expected 0, got %ld\n", retA);
@@ -641,14 +789,17 @@ static void test_strftime(void)
         ok(errno == EINVAL, "errno = %d\n", errno);
     }
 
-    retA = p_strftime(bufA, 256, "e%#%e", gmt_tm);
-    ok(retA == 3, "expected 3, got %ld\n", retA);
-    ok(!strcmp(bufA, "e%e"), "got %s\n", bufA);
+    for (i=0; i<ARRAY_SIZE(tests); i++)
+    {
+        retA = p_strftime(bufA, 256, tests[i].format, &tests[i].tm);
+        todo_wine_if(tests[i].todo) {
+            ok(retA == strlen(tests[i].ret), "%d) ret = %ld\n", i, retA);
+            ok(!strcmp(bufA, tests[i].ret), "%d) buf = \"%s\", expected \"%s\"\n",
+                    i, bufA, tests[i].ret);
+        }
+    }
 
     retA = p_strftime(bufA, 256, "%c", gmt_tm);
-    ok(retA == 17, "expected 17, got %ld\n", retA);
-    ok(strcmp(bufA, expected) == 0, "expected %s, got %s\n", expected, bufA);
-
     retW = p_wcsftime(bufW, 256, cW, gmt_tm);
     ok(retW == 17, "expected 17, got %ld\n", retW);
     ok(retA == retW, "expected %ld, got %ld\n", retA, retW);
@@ -657,96 +808,6 @@ static void test_strftime(void)
     buf[retA] = 0;
     ok(strcmp(bufA, buf) == 0, "expected %s, got %s\n", bufA, buf);
 
-    retA = p_strftime(bufA, 256, "%x", gmt_tm);
-    ok(retA == 8, "expected 8, got %ld\n", retA);
-    ok(!strcmp(bufA, "01/01/70"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%X", gmt_tm);
-    ok(retA == 8, "expected 8, got %ld\n", retA);
-    ok(!strcmp(bufA, "00:00:00"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%a", gmt_tm);
-    ok(retA == 3, "expected 3, got %ld\n", retA);
-    ok(!strcmp(bufA, "Thu"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%A", gmt_tm);
-    ok(retA == 8, "expected 8, got %ld\n", retA);
-    ok(!strcmp(bufA, "Thursday"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%b", gmt_tm);
-    ok(retA == 3, "expected 3, got %ld\n", retA);
-    ok(!strcmp(bufA, "Jan"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%B", gmt_tm);
-    ok(retA == 7, "expected 7, got %ld\n", retA);
-    ok(!strcmp(bufA, "January"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%d", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "01"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%#d", gmt_tm);
-    ok(retA == 1, "expected 1, got %ld\n", retA);
-    ok(!strcmp(bufA, "1"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%H", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "00"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%I", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "12"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%j", gmt_tm);
-    ok(retA == 3, "expected 3, got %ld\n", retA);
-    ok(!strcmp(bufA, "001"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%m", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "01"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%#M", gmt_tm);
-    ok(retA == 1, "expected 1, got %ld\n", retA);
-    ok(!strcmp(bufA, "0"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%p", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "AM"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%U", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "00"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%W", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "00"), "got %s\n", bufA);
-
-    gmt_tm->tm_wday = 0;
-    retA = p_strftime(bufA, 256, "%U", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "01"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%W", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "00"), "got %s\n", bufA);
-
-    gmt_tm->tm_yday = 365;
-    retA = p_strftime(bufA, 256, "%U", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "53"), "got %s\n", bufA);
-
-    retA = p_strftime(bufA, 256, "%W", gmt_tm);
-    ok(retA == 2, "expected 2, got %ld\n", retA);
-    ok(!strcmp(bufA, "52"), "got %s\n", bufA);
-
-    gmt_tm->tm_mon = 1;
-    gmt_tm->tm_mday = 30;
-    retA = p_strftime(bufA, 256, "%c", gmt_tm);
-    todo_wine {
-        ok(retA == 17, "expected 17, got %ld\n", retA);
-        ok(!strcmp(bufA, "02/30/70 00:00:00"), "got %s\n", bufA);
-    }
-
     if(!setlocale(LC_ALL, "Japanese_Japan.932")) {
         win_skip("Japanese_Japan.932 locale not available\n");
         return;
@@ -756,6 +817,27 @@ static void test_strftime(void)
     retA = p_strftime(bufA, 256, "\x82%c", gmt_tm);
     ok(retA == 3, "expected 3, got %ld\n", retA);
     ok(!strcmp(bufA, "\x82%c"), "got %s\n", bufA);
+
+    setlocale(LC_ALL, "C");
+    if(!p__Strftime) {
+        win_skip("_Strftime is not available\n");
+        return;
+    }
+
+    /* TODO: find meaning of unk[0] */
+    time_data.unk[0] = 1;
+    for (i=0; i<ARRAY_SIZE(tests_td); i++)
+    {
+        time_data.short_date = tests_td[i].short_date;
+        time_data.date = tests_td[i].date;
+        time_data.time = tests_td[i].time;
+        retA = p__Strftime(buf, sizeof(buf), tests_td[i].format, &tests_td[i].tm, &time_data);
+        ok(retA == strlen(buf), "%d) ret = %ld\n", i, retA);
+        todo_wine_if(tests_td[i].todo) {
+            ok(!strcmp(buf, tests_td[i].ret), "%d) buf = \"%s\", expected \"%s\"\n",
+                    i, buf, tests_td[i].ret);
+        }
+    }
 }
 
 static void test_asctime(void)
@@ -829,7 +911,7 @@ static void test__tzset(void)
     int ret;
 
     if(!p___p__daylight || !p___p__timezone || !p___p__dstbias) {
-        win_skip("__p__daylight, __p__timezone or __p__dstbias is not available\n");
+        skip("__p__daylight, __p__timezone or __p__dstbias is not available\n");
         return;
     }
 
@@ -873,23 +955,6 @@ static void test__tzset(void)
     _putenv(TZ_env);
 }
 
-static void test_clock(void)
-{
-    static const int THRESH = 100;
-    FILETIME start, cur;
-    int c, expect;
-    BOOL ret;
-
-    ret = GetProcessTimes(GetCurrentProcess(), &start, &cur, &cur, &cur);
-    ok(ret, "GetProcessTimes failed with error: %d\n", GetLastError());
-    GetSystemTimeAsFileTime(&cur);
-    expect = (((LONGLONG)cur.dwHighDateTime<<32)+cur.dwLowDateTime -
-            ((LONGLONG)start.dwHighDateTime<<32)-start.dwLowDateTime) / 10000;
-
-    c = clock();
-    ok(abs(c-expect) < THRESH, "clock() = %d, expected %d\n", c, expect);
-}
-
 START_TEST(time)
 {
     init();
@@ -908,5 +973,4 @@ START_TEST(time)
     test_localtime64_s();
     test_daylight();
     test_asctime();
-    test_clock();
 }
