@@ -36,12 +36,30 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
 
+BOOL WINAPI GetDaylightFlag(void);
+
+static LONGLONG init_time;
+
+void msvcrt_init_clock(void)
+{
+    LARGE_INTEGER systime;
+
+    NtQuerySystemTime(&systime);
+    init_time = systime.QuadPart;
+}
+
 static const int MonthLengths[2][12] =
 {
     { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
     { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
 };
 
+#if _MSVCR_VER>=140
+static const int MAX_SECONDS = 60;
+#else
+static const int MAX_SECONDS = 59;
+#endif
+
 static inline BOOL IsLeapYear(int Year)
 {
     return Year % 4 == 0 && (Year % 100 != 0 || Year % 400 == 0);
@@ -710,23 +728,10 @@ int CDECL _wstrtime_s(MSVCRT_wchar_t* time, MSVCRT_size_t size)
  */
 MSVCRT_clock_t CDECL MSVCRT_clock(void)
 {
-    static LONGLONG start_time;
     LARGE_INTEGER systime;
 
-    if(!start_time) {
-        KERNEL_USER_TIMES pti;
-
-        /* while Linux's clock returns user time, Windows' clock
-         * returns wall-clock time from process start.  cache the
-         * process start time since it won't change and to avoid
-         * wineserver round-trip overhead */
-        if(NtQueryInformationProcess(GetCurrentProcess(), ProcessTimes, &pti, sizeof(pti), NULL))
-            return -1;
-        start_time = pti.CreateTime.QuadPart;
-    }
-
     NtQuerySystemTime(&systime);
-    return (systime.QuadPart - start_time) * MSVCRT_CLOCKS_PER_SEC / TICKSPERSEC;
+    return (systime.QuadPart - init_time) / (TICKSPERSEC / MSVCRT_CLOCKS_PER_SEC);
 }
 
 /*********************************************************************
@@ -765,21 +770,19 @@ double CDECL MSVCRT_difftime(MSVCRT___time32_t time1, MSVCRT___time32_t time2)
  */
 void CDECL MSVCRT__ftime64(struct MSVCRT___timeb64 *buf)
 {
-  TIME_ZONE_INFORMATION tzinfo;
   FILETIME ft;
   ULONGLONG time;
 
-  DWORD tzid = GetTimeZoneInformation(&tzinfo);
+  _tzset_init();
+
   GetSystemTimeAsFileTime(&ft);
 
   time = ((ULONGLONG)ft.dwHighDateTime << 32) | ft.dwLowDateTime;
 
   buf->time = time / TICKSPERSEC - SECS_1601_TO_1970;
   buf->millitm = (time % TICKSPERSEC) / TICKSPERMSEC;
-  buf->timezone = tzinfo.Bias +
-      ( tzid == TIME_ZONE_ID_STANDARD ? tzinfo.StandardBias :
-      ( tzid == TIME_ZONE_ID_DAYLIGHT ? tzinfo.DaylightBias : 0 ));
-  buf->dstflag = (tzid == TIME_ZONE_ID_DAYLIGHT?1:0);
+  buf->timezone = MSVCRT___timezone / 60;
+  buf->dstflag = GetDaylightFlag();
 }
 
 /*********************************************************************
@@ -939,6 +942,11 @@ int CDECL MSVCRT__get_tzname(MSVCRT_size_t *ret, char *buf, MSVCRT_size_t bufsiz
     *ret = strlen(timezone)+1;
     if(!buf && !bufsize)
         return 0;
+    if(*ret > bufsize)
+    {
+        buf[0] = 0;
+        return MSVCRT_ERANGE;
+    }
 
     strcpy(buf, timezone);
     return 0;
@@ -952,111 +960,52 @@ char ** CDECL __p__tzname(void)
 	return MSVCRT__tzname;
 }
 
-static inline BOOL strftime_date(char *str, MSVCRT_size_t *pos, MSVCRT_size_t max,
-        BOOL alternate, const struct MSVCRT_tm *mstm, MSVCRT___lc_time_data *time_data)
-{
-    char *format;
-    SYSTEMTIME st;
-    MSVCRT_size_t ret;
-    LCID lcid;
-
-    st.wYear = mstm->tm_year + 1900;
-    st.wMonth = mstm->tm_mon + 1;
-    st.wDayOfWeek = mstm->tm_wday;
-    st.wDay = mstm->tm_mday;
-    st.wHour = mstm->tm_hour;
-    st.wMinute = mstm->tm_min;
-    st.wSecond = mstm->tm_sec;
-    st.wMilliseconds = 0;
-
-#if _MSVCR_VER < 110
-    lcid = time_data->lcid;
+#if _MSVCR_VER <= 90
+#define STRFTIME_CHAR char
+#define STRFTIME_TD(td, name) td->str.names.name
 #else
-    lcid = LocaleNameToLCID(time_data->locname, 0);
+#define STRFTIME_CHAR MSVCRT_wchar_t
+#define STRFTIME_TD(td, name) td->wstr.names.name
 #endif
 
-    format = alternate ? time_data->str.names.date : time_data->str.names.short_date;
-    ret = GetDateFormatA(lcid, 0, &st, format, NULL, 0);
-    if(ret && ret<max-*pos)
-        ret = GetDateFormatA(lcid, 0, &st, format, str+*pos, max-*pos);
-    if(!ret) {
-        *str = 0;
-        *MSVCRT__errno() = MSVCRT_EINVAL;
-        return FALSE;
-    }else if(ret > max-*pos) {
-        *str = 0;
-        *MSVCRT__errno() = MSVCRT_ERANGE;
-        return FALSE;
-    }
-    *pos += ret-1;
-    return TRUE;
-}
-
-static inline BOOL strftime_time(char *str, MSVCRT_size_t *pos, MSVCRT_size_t max,
-        const struct MSVCRT_tm *mstm, MSVCRT___lc_time_data *time_data)
+#define strftime_str(a,b,c,d) strftime_nstr(a,b,c,d,MSVCRT_SIZE_MAX)
+static inline BOOL strftime_nstr(STRFTIME_CHAR *str, MSVCRT_size_t *pos,
+        MSVCRT_size_t max, const STRFTIME_CHAR *src, MSVCRT_size_t len)
 {
-    SYSTEMTIME st;
-    MSVCRT_size_t ret;
-    LCID lcid;
+    while(*src && len)
+    {
+        if(*pos >= max) {
+            *str = 0;
+            *MSVCRT__errno() = MSVCRT_ERANGE;
+            return FALSE;
+        }
 
-    st.wYear = mstm->tm_year + 1900;
-    st.wMonth = mstm->tm_mon + 1;
-    st.wDayOfWeek = mstm->tm_wday;
-    st.wDay = mstm->tm_mday;
-    st.wHour = mstm->tm_hour;
-    st.wMinute = mstm->tm_min;
-    st.wSecond = mstm->tm_sec;
-    st.wMilliseconds = 0;
-
-#if _MSVCR_VER < 110
-    lcid = time_data->lcid;
-#else
-    lcid = LocaleNameToLCID(time_data->locname, 0);
-#endif
-
-    ret = GetTimeFormatA(lcid, 0, &st, time_data->str.names.time, NULL, 0);
-    if(ret && ret<max-*pos)
-        ret = GetTimeFormatA(lcid, 0, &st, time_data->str.names.time,
-                str+*pos, max-*pos);
-    if(!ret) {
-        *str = 0;
-        *MSVCRT__errno() = MSVCRT_EINVAL;
-        return FALSE;
-    }else if(ret > max-*pos) {
-        *str = 0;
-        *MSVCRT__errno() = MSVCRT_ERANGE;
-        return FALSE;
+        str[*pos] = *src;
+        src++;
+        *pos += 1;
+        len--;
     }
-    *pos += ret-1;
     return TRUE;
 }
 
-static inline BOOL strftime_str(char *str, MSVCRT_size_t *pos, MSVCRT_size_t max, char *src)
-{
-    MSVCRT_size_t len = strlen(src);
-    if(len > max-*pos) {
-        *str = 0;
-        *MSVCRT__errno() = MSVCRT_ERANGE;
-        return FALSE;
-    }
-
-    memcpy(str+*pos, src, len);
-    *pos += len;
-    return TRUE;
-}
-
-static inline BOOL strftime_int(char *str, MSVCRT_size_t *pos, MSVCRT_size_t max,
+static inline BOOL strftime_int(STRFTIME_CHAR *str, MSVCRT_size_t *pos, MSVCRT_size_t max,
         int src, int prec, int l, int h)
 {
+#if _MSVCR_VER > 90
+    static const WCHAR fmt[] = {'%','0','*','d',0};
+#endif
     MSVCRT_size_t len;
 
-    if(src<l || src>h) {
+    if(!MSVCRT_CHECK_PMT(src>=l && src<=h)) {
         *str = 0;
-        *MSVCRT__errno() = MSVCRT_EINVAL;
         return FALSE;
     }
 
+#if _MSVCR_VER <= 90
     len = MSVCRT__snprintf(str+*pos, max-*pos, "%0*d", prec, src);
+#else
+    len = MSVCRT__snwprintf(str+*pos, max-*pos, fmt, prec, src);
+#endif
     if(len == -1) {
         *str = 0;
         *MSVCRT__errno() = MSVCRT_ERANGE;
@@ -1067,13 +1016,188 @@ static inline BOOL strftime_int(char *str, MSVCRT_size_t *pos, MSVCRT_size_t max
     return TRUE;
 }
 
-static MSVCRT_size_t strftime_helper(char *str, MSVCRT_size_t max, const char *format,
-        const struct MSVCRT_tm *mstm, MSVCRT___lc_time_data *time_data, MSVCRT__locale_t loc)
+static inline BOOL strftime_format(STRFTIME_CHAR *str, MSVCRT_size_t *pos, MSVCRT_size_t max,
+        const struct MSVCRT_tm *mstm, MSVCRT___lc_time_data *time_data, const STRFTIME_CHAR *format)
+{
+    MSVCRT_size_t count;
+    BOOL ret = TRUE;
+
+    while(*format && ret)
+    {
+        count = 1;
+        while(format[0] == format[count]) count++;
+
+        switch(*format) {
+        case '\'':
+            if(count % 2 == 0) break;
+
+            format += count;
+            count = 0;
+            while(format[count] && format[count] != '\'') count++;
+
+            ret = strftime_nstr(str, pos, max, format, count);
+            if(!ret) return FALSE;
+            if(format[count] == '\'') count++;
+            break;
+        case 'd':
+            if(count > 2)
+            {
+                if(!MSVCRT_CHECK_PMT(mstm->tm_wday>=0 && mstm->tm_wday<=6))
+                {
+                    *str = 0;
+                    return FALSE;
+                }
+            }
+            switch(count) {
+            case 1:
+            case 2:
+                ret = strftime_int(str, pos, max, mstm->tm_mday, count==1 ? 0 : 2, 1, 31);
+                break;
+            case 3:
+                ret = strftime_str(str, pos, max, STRFTIME_TD(time_data, short_wday)[mstm->tm_wday]);
+                break;
+            default:
+                ret = strftime_nstr(str, pos, max, format, count-4);
+                if(ret)
+                    ret = strftime_str(str, pos, max, STRFTIME_TD(time_data, wday)[mstm->tm_wday]);
+                break;
+            }
+            break;
+        case 'M':
+            if(count > 2)
+            {
+                if(!MSVCRT_CHECK_PMT(mstm->tm_mon>=0 && mstm->tm_mon<=11))
+                {
+                    *str = 0;
+                    return FALSE;
+                }
+            }
+            switch(count) {
+            case 1:
+            case 2:
+                ret = strftime_int(str, pos, max, mstm->tm_mon+1, count==1 ? 0 : 2, 1, 12);
+                break;
+            case 3:
+                ret = strftime_str(str, pos, max, STRFTIME_TD(time_data, short_mon)[mstm->tm_mon]);
+                break;
+            default:
+                ret = strftime_nstr(str, pos, max, format, count-4);
+                if(ret)
+                    ret = strftime_str(str, pos, max, STRFTIME_TD(time_data, mon)[mstm->tm_mon]);
+                break;
+            }
+            break;
+        case 'y':
+            if(count > 1)
+            {
+#if _MSVCR_VER>=140
+                if(!MSVCRT_CHECK_PMT(mstm->tm_year >= -1900 && mstm->tm_year <= 8099))
+#else
+                if(!MSVCRT_CHECK_PMT(mstm->tm_year >= 0))
+#endif
+                {
+                    *str = 0;
+                    return FALSE;
+                }
+            }
+
+            switch(count) {
+            case 1:
+                ret = strftime_nstr(str, pos, max, format, 1);
+                break;
+            case 2:
+            case 3:
+                ret = strftime_nstr(str, pos, max, format, count-2);
+                if(ret)
+                    ret = strftime_int(str, pos, max, (mstm->tm_year+1900)%100, 2, 0, 99);
+                break;
+            default:
+                ret = strftime_nstr(str, pos, max, format, count-4);
+                if(ret)
+                    ret = strftime_int(str, pos, max, mstm->tm_year+1900, 4, 0, 9999);
+                break;
+            }
+            break;
+        case 'h':
+            if(!MSVCRT_CHECK_PMT(mstm->tm_hour>=0 && mstm->tm_hour<=23))
+            {
+                *str = 0;
+                return FALSE;
+            }
+            if(count > 2)
+                ret = strftime_nstr(str, pos, max, format, count-2);
+            if(ret)
+                ret = strftime_int(str, pos, max, (mstm->tm_hour + 11) % 12 + 1,
+                        count == 1 ? 0 : 2, 1, 12);
+            break;
+        case 'H':
+            if(count > 2)
+                ret = strftime_nstr(str, pos, max, format, count-2);
+            if(ret)
+                ret = strftime_int(str, pos, max, mstm->tm_hour, count == 1 ? 0 : 2, 0, 23);
+            break;
+        case 'm':
+            if(count > 2)
+                ret = strftime_nstr(str, pos, max, format, count-2);
+            if(ret)
+                ret = strftime_int(str, pos, max, mstm->tm_min, count == 1 ? 0 : 2, 0, 59);
+            break;
+        case 's':
+            if(count > 2)
+                ret = strftime_nstr(str, pos, max, format, count-2);
+            if(ret)
+                ret = strftime_int(str, pos, max, mstm->tm_sec, count == 1 ? 0 : 2, 0, MAX_SECONDS);
+            break;
+        case 'a':
+        case 'A':
+        case 't':
+            if(!MSVCRT_CHECK_PMT(mstm->tm_hour>=0 && mstm->tm_hour<=23))
+            {
+                *str = 0;
+                return FALSE;
+            }
+            ret = strftime_nstr(str, pos, max,
+                    mstm->tm_hour < 12 ? STRFTIME_TD(time_data, am) : STRFTIME_TD(time_data, pm),
+                    (*format == 't' && count == 1) ? 1 : MSVCRT_SIZE_MAX);
+            break;
+        default:
+            ret = strftime_nstr(str, pos, max, format, count);
+            break;
+        }
+        format += count;
+    }
+
+    return ret;
+}
+
+#if _MSVCR_VER>=140
+static inline BOOL strftime_tzdiff(STRFTIME_CHAR *str, MSVCRT_size_t *pos, MSVCRT_size_t max, BOOL is_dst)
+{
+    MSVCRT_long tz = MSVCRT___timezone + (is_dst ? MSVCRT__dstbias : 0);
+    char sign;
+
+    if(tz < 0) {
+        sign = '+';
+        tz = -tz;
+    }else {
+        sign = '-';
+    }
+
+    if(*pos < max)
+        str[(*pos)++] = sign;
+    if(!strftime_int(str, pos, max, tz/60/60, 2, 0, 99))
+        return FALSE;
+    return strftime_int(str, pos, max, tz/60%60, 2, 0, 59);
+}
+#endif
+
+static MSVCRT_size_t strftime_impl(STRFTIME_CHAR *str, MSVCRT_size_t max,
+        const STRFTIME_CHAR *format, const struct MSVCRT_tm *mstm,
+        MSVCRT___lc_time_data *time_data, MSVCRT__locale_t loc)
 {
     MSVCRT_size_t ret, tmp;
     BOOL alternate;
-
-    TRACE("(%p %ld %s %p %p)\n", str, max, format, mstm, time_data);
+    int year = mstm ? mstm->tm_year + 1900 : -1;
 
     if(!str || !format) {
         if(str && max)
@@ -1090,7 +1214,8 @@ static MSVCRT_size_t strftime_helper(char *str, MSVCRT_size_t max, const char *f
             if(MSVCRT__isleadbyte_l((unsigned char)*format, loc)) {
                 str[ret++] = *(format++);
                 if(ret == max) continue;
-                if(!str[ret]) goto einval_error;
+                if(!MSVCRT_CHECK_PMT(str[ret]))
+                    goto einval_error;
             }
             str[ret++] = *format;
             continue;
@@ -1104,65 +1229,148 @@ static MSVCRT_size_t strftime_helper(char *str, MSVCRT_size_t max, const char *f
             alternate = FALSE;
         }
 
-        if(!mstm)
+        if(!MSVCRT_CHECK_PMT(mstm))
             goto einval_error;
 
         switch(*format) {
         case 'c':
-            if(!strftime_date(str, &ret, max, alternate, mstm, time_data))
+#if _MSVCR_VER>=140
+            if(time_data == &cloc_time_data && !alternate)
+            {
+                static const WCHAR datetime_format[] =
+                        { '%','a',' ','%','b',' ','%','e',' ','%','T',' ','%','Y',0 };
+                tmp = strftime_impl(str+ret, max-ret, datetime_format, mstm, time_data, loc);
+                if(!tmp)
+                    return 0;
+                ret += tmp;
+                break;
+            }
+#endif
+            if(!strftime_format(str, &ret, max, mstm, time_data,
+                    alternate ? STRFTIME_TD(time_data, date) : STRFTIME_TD(time_data, short_date)))
                 return 0;
             if(ret < max)
                 str[ret++] = ' ';
-            if(!strftime_time(str, &ret, max, mstm, time_data))
+            if(!strftime_format(str, &ret, max, mstm, time_data, STRFTIME_TD(time_data, time)))
                 return 0;
             break;
         case 'x':
-            if(!strftime_date(str, &ret, max, alternate, mstm, time_data))
+            if(!strftime_format(str, &ret, max, mstm, time_data,
+                    alternate ? STRFTIME_TD(time_data, date) : STRFTIME_TD(time_data, short_date)))
                 return 0;
             break;
         case 'X':
-            if(!strftime_time(str, &ret, max, mstm, time_data))
+            if(!strftime_format(str, &ret, max, mstm, time_data, STRFTIME_TD(time_data, time)))
                 return 0;
             break;
         case 'a':
-            if(mstm->tm_wday<0 || mstm->tm_wday>6)
+            if(!MSVCRT_CHECK_PMT(mstm->tm_wday>=0 && mstm->tm_wday<=6))
                 goto einval_error;
-            if(!strftime_str(str, &ret, max, time_data->str.names.short_wday[mstm->tm_wday]))
+            if(!strftime_str(str, &ret, max, STRFTIME_TD(time_data, short_wday)[mstm->tm_wday]))
                 return 0;
             break;
         case 'A':
-            if(mstm->tm_wday<0 || mstm->tm_wday>6)
+            if(!MSVCRT_CHECK_PMT(mstm->tm_wday>=0 && mstm->tm_wday<=6))
                 goto einval_error;
-            if(!strftime_str(str, &ret, max, time_data->str.names.wday[mstm->tm_wday]))
+            if(!strftime_str(str, &ret, max, STRFTIME_TD(time_data, wday)[mstm->tm_wday]))
                 return 0;
             break;
         case 'b':
-            if(mstm->tm_mon<0 || mstm->tm_mon>11)
+#if _MSVCR_VER>=140
+        case 'h':
+#endif
+            if(!MSVCRT_CHECK_PMT(mstm->tm_mon>=0 && mstm->tm_mon<=11))
                 goto einval_error;
-            if(!strftime_str(str, &ret, max, time_data->str.names.short_mon[mstm->tm_mon]))
+            if(!strftime_str(str, &ret, max, STRFTIME_TD(time_data, short_mon)[mstm->tm_mon]))
                 return 0;
             break;
         case 'B':
-            if(mstm->tm_mon<0 || mstm->tm_mon>11)
+            if(!MSVCRT_CHECK_PMT(mstm->tm_mon>=0 && mstm->tm_mon<=11))
                 goto einval_error;
-            if(!strftime_str(str, &ret, max, time_data->str.names.mon[mstm->tm_mon]))
+            if(!strftime_str(str, &ret, max, STRFTIME_TD(time_data, mon)[mstm->tm_mon]))
                 return 0;
             break;
+#if _MSVCR_VER>=140
+        case 'C':
+            if(!MSVCRT_CHECK_PMT(year>=0 && year<=9999))
+                goto einval_error;
+            if(!strftime_int(str, &ret, max, year/100, alternate ? 0 : 2, 0, 99))
+                return 0;
+            break;
+#endif
         case 'd':
-            if(!strftime_int(str, &ret, max, mstm->tm_mday, alternate ? 0 : 2, 0, 31))
+            if(!strftime_int(str, &ret, max, mstm->tm_mday, alternate ? 0 : 2, 1, 31))
                 return 0;
             break;
+#if _MSVCR_VER>=140
+        case 'D':
+            if(!MSVCRT_CHECK_PMT(year>=0 && year<=9999))
+                goto einval_error;
+            if(!strftime_int(str, &ret, max, mstm->tm_mon+1, alternate ? 0 : 2, 1, 12))
+                return 0;
+            if(ret < max)
+                str[ret++] = '/';
+            if(!strftime_int(str, &ret, max, mstm->tm_mday, alternate ? 0 : 2, 1, 31))
+                return 0;
+            if(ret < max)
+                str[ret++] = '/';
+            if(!strftime_int(str, &ret, max, year%100, alternate ? 0 : 2, 0, 99))
+                return 0;
+            break;
+        case 'e':
+            if(!strftime_int(str, &ret, max, mstm->tm_mday, alternate ? 0 : 2, 1, 31))
+                return 0;
+            if(!alternate && str[ret-2] == '0')
+                str[ret-2] = ' ';
+            break;
+        case 'F':
+            if(!strftime_int(str, &ret, max, year, alternate ? 0 : 4, 0, 9999))
+                return 0;
+            if(ret < max)
+                str[ret++] = '-';
+            if(!strftime_int(str, &ret, max, mstm->tm_mon+1, alternate ? 0 : 2, 1, 12))
+                return 0;
+            if(ret < max)
+                str[ret++] = '-';
+            if(!strftime_int(str, &ret, max, mstm->tm_mday, alternate ? 0 : 2, 1, 31))
+                return 0;
+            break;
+        case 'g':
+        case 'G':
+            if(!MSVCRT_CHECK_PMT(year>=0 && year<=9999))
+                goto einval_error;
+            /* fall through */
+        case 'V':
+        {
+            int iso_year = year;
+            int iso_days = mstm->tm_yday - (mstm->tm_wday ? mstm->tm_wday : 7) + 4;
+            if (iso_days < 0)
+                iso_days += 365 + IsLeapYear(--iso_year);
+            else if(iso_days >= 365 + IsLeapYear(iso_year))
+                iso_days -= 365 + IsLeapYear(iso_year++);
+
+            if(*format == 'G') {
+                if(!strftime_int(str, &ret, max, iso_year, 4, 0, 9999))
+                    return 0;
+            } else if(*format == 'g') {
+                if(!strftime_int(str, &ret, max, iso_year%100, 2, 0, 99))
+                    return 0;
+            } else {
+                if(!strftime_int(str, &ret, max, iso_days/7 + 1, alternate ? 0 : 2, 0, 53))
+                    return 0;
+            }
+            break;
+        }
+#endif
         case 'H':
             if(!strftime_int(str, &ret, max, mstm->tm_hour, alternate ? 0 : 2, 0, 23))
                 return 0;
             break;
         case 'I':
-            tmp = mstm->tm_hour;
-            if(tmp > 12)
-                tmp -= 12;
-            else if(!tmp)
-                tmp = 12;
-            if(!strftime_int(str, &ret, max, tmp, alternate ? 0 : 2, 1, 12))
+            if(!MSVCRT_CHECK_PMT(mstm->tm_hour>=0 && mstm->tm_hour<=23))
+                goto einval_error;
+            if(!strftime_int(str, &ret, max, (mstm->tm_hour + 11) % 12 + 1,
+                        alternate ? 0 : 2, 1, 12))
                 return 0;
             break;
         case 'j':
@@ -1177,40 +1385,127 @@ static MSVCRT_size_t strftime_helper(char *str, MSVCRT_size_t max, const char *f
             if(!strftime_int(str, &ret, max, mstm->tm_min, alternate ? 0 : 2, 0, 59))
                 return 0;
             break;
+#if _MSVCR_VER>=140
+        case 'n':
+            str[ret++] = '\n';
+            break;
+#endif
         case 'p':
-            if(mstm->tm_hour<0 || mstm->tm_hour>23)
+            if(!MSVCRT_CHECK_PMT(mstm->tm_hour>=0 && mstm->tm_hour<=23))
                 goto einval_error;
             if(!strftime_str(str, &ret, max, mstm->tm_hour<12 ?
-                        time_data->str.names.am : time_data->str.names.pm))
+                        STRFTIME_TD(time_data, am) : STRFTIME_TD(time_data, pm)))
                 return 0;
             break;
+#if _MSVCR_VER>=140
+        case 'r':
+            if(time_data == &cloc_time_data)
+            {
+                if(!MSVCRT_CHECK_PMT(mstm->tm_hour>=0 && mstm->tm_hour<=23))
+                    goto einval_error;
+                if(!strftime_int(str, &ret, max, (mstm->tm_hour + 11) % 12 + 1,
+                            alternate ? 0 : 2, 1, 12))
+                    return 0;
+                if(ret < max)
+                    str[ret++] = ':';
+                if(!strftime_int(str, &ret, max, mstm->tm_min, alternate ? 0 : 2, 0, 59))
+                    return 0;
+                if(ret < max)
+                    str[ret++] = ':';
+                if(!strftime_int(str, &ret, max, mstm->tm_sec, alternate ? 0 : 2, 0, MAX_SECONDS))
+                    return 0;
+                if(ret < max)
+                    str[ret++] = ' ';
+                if(!strftime_str(str, &ret, max, mstm->tm_hour<12 ?
+                            STRFTIME_TD(time_data, am) : STRFTIME_TD(time_data, pm)))
+                    return 0;
+            }
+            else
+            {
+                if(!strftime_format(str, &ret, max, mstm, time_data, STRFTIME_TD(time_data, time)))
+                    return 0;
+            }
+            break;
+        case 'R':
+            if(!strftime_int(str, &ret, max, mstm->tm_hour, alternate ? 0 : 2, 0, 23))
+                return 0;
+            if(ret < max)
+                str[ret++] = ':';
+            if(!strftime_int(str, &ret, max, mstm->tm_min, alternate ? 0 : 2, 0, 59))
+                return 0;
+            break;
+#endif
         case 'S':
-            if(!strftime_int(str, &ret, max, mstm->tm_sec, alternate ? 0 : 2, 0, 59))
+            if(!strftime_int(str, &ret, max, mstm->tm_sec, alternate ? 0 : 2, 0, MAX_SECONDS))
                 return 0;
             break;
+#if _MSVCR_VER>=140
+        case 't':
+            str[ret++] = '\t';
+            break;
+        case 'T':
+            if(!strftime_int(str, &ret, max, mstm->tm_hour, alternate ? 0 : 2, 0, 23))
+                return 0;
+            if(ret < max)
+                str[ret++] = ':';
+            if(!strftime_int(str, &ret, max, mstm->tm_min, alternate ? 0 : 2, 0, 59))
+                return 0;
+            if(ret < max)
+                str[ret++] = ':';
+            if(!strftime_int(str, &ret, max, mstm->tm_sec, alternate ? 0 : 2, 0, MAX_SECONDS))
+                return 0;
+            break;
+        case 'u':
+            if(!MSVCRT_CHECK_PMT(mstm->tm_wday>=0 && mstm->tm_wday<=6))
+                goto einval_error;
+            tmp = mstm->tm_wday ? mstm->tm_wday : 7;
+            if(!strftime_int(str, &ret, max, tmp, 0, 1, 7))
+                return 0;
+            break;
+#endif
         case 'w':
             if(!strftime_int(str, &ret, max, mstm->tm_wday, 0, 0, 6))
                 return 0;
             break;
         case 'y':
-            if(!strftime_int(str, &ret, max, mstm->tm_year%100, alternate ? 0 : 2, 0, 99))
+#if _MSVCR_VER>=140
+            if(!MSVCRT_CHECK_PMT(year>=0 && year<=9999))
+#else
+            if(!MSVCRT_CHECK_PMT(year>=1900))
+#endif
+                goto einval_error;
+            if(!strftime_int(str, &ret, max, year%100, alternate ? 0 : 2, 0, 99))
                 return 0;
             break;
         case 'Y':
-            tmp = 1900+mstm->tm_year;
-            if(!strftime_int(str, &ret, max, tmp, alternate ? 0 : 4, 0, 9999))
+            if(!strftime_int(str, &ret, max, year, alternate ? 0 : 4, 0, 9999))
                 return 0;
             break;
         case 'z':
+#if _MSVCR_VER>=140
+            MSVCRT__tzset();
+            if(!strftime_tzdiff(str, &ret, max, mstm->tm_isdst))
+                return 0;
+            break;
+#endif
         case 'Z':
             MSVCRT__tzset();
+#if _MSVCR_VER <= 90
             if(MSVCRT__get_tzname(&tmp, str+ret, max-ret, mstm->tm_isdst ? 1 : 0))
                 return 0;
+#else
+                if(MSVCRT__mbstowcs_s_l(&tmp, str+ret, max-ret,
+                            mstm->tm_isdst ? tzname_dst : tzname_std,
+                            MSVCRT__TRUNCATE, loc) == MSVCRT_STRUNCATE)
+                    ret = max;
+#endif
             ret += tmp-1;
             break;
         case 'U':
         case 'W':
-            if(mstm->tm_wday<0 || mstm->tm_wday>6 || mstm->tm_yday<0 || mstm->tm_yday>365)
+            if(!MSVCRT_CHECK_PMT(mstm->tm_wday>=0 && mstm->tm_wday<=6))
+                goto einval_error;
+            if(!MSVCRT_CHECK_PMT(mstm->tm_yday>=0 && mstm->tm_yday<=365))
                 goto einval_error;
             if(*format == 'U')
                 tmp = mstm->tm_wday;
@@ -1228,6 +1523,7 @@ static MSVCRT_size_t strftime_helper(char *str, MSVCRT_size_t max, const char *f
             break;
         default:
             WARN("unknown format %c\n", *format);
+            MSVCRT_INVALID_PMT("unknown format", MSVCRT_EINVAL);
             goto einval_error;
         }
     }
@@ -1244,10 +1540,44 @@ static MSVCRT_size_t strftime_helper(char *str, MSVCRT_size_t max, const char *f
 
 einval_error:
     *str = 0;
-    *MSVCRT__errno() = MSVCRT_EINVAL;
     return 0;
 }
 
+static MSVCRT_size_t strftime_helper(char *str, MSVCRT_size_t max, const char *format,
+        const struct MSVCRT_tm *mstm, MSVCRT___lc_time_data *time_data, MSVCRT__locale_t loc)
+{
+#if _MSVCR_VER <= 90
+    TRACE("(%p %ld %s %p %p %p)\n", str, max, format, mstm, time_data, loc);
+    return strftime_impl(str, max, format, mstm, time_data, loc);
+#else
+    MSVCRT_wchar_t *s, *fmt;
+    MSVCRT_size_t len;
+
+    TRACE("(%p %ld %s %p %p %p)\n", str, max, format, mstm, time_data, loc);
+
+    if (!MSVCRT_CHECK_PMT(str != NULL)) return 0;
+    if (!MSVCRT_CHECK_PMT(max != 0)) return 0;
+    *str = 0;
+    if (!MSVCRT_CHECK_PMT(format != NULL)) return 0;
+
+    len = MSVCRT__mbstowcs_l( NULL, format, 0, loc ) + 1;
+    if (!len || !(fmt = MSVCRT_malloc( len*sizeof(MSVCRT_wchar_t) ))) return 0;
+    MSVCRT__mbstowcs_l(fmt, format, len, loc);
+
+    if ((s = MSVCRT_malloc( max*sizeof(MSVCRT_wchar_t) )))
+    {
+        len = strftime_impl( s, max, fmt, mstm, time_data, loc );
+        if (len)
+            len = MSVCRT__wcstombs_l( str, s, max, loc );
+        MSVCRT_free( s );
+    }
+    else len = 0;
+
+    MSVCRT_free( fmt );
+    return len;
+#endif
+}
+
 #if _MSVCR_VER >= 80
 /********************************************************************
  *     _strftime_l (MSVCR80.@)
@@ -1281,6 +1611,7 @@ static MSVCRT_size_t wcsftime_helper( MSVCRT_wchar_t *str, MSVCRT_size_t max,
         const MSVCRT_wchar_t *format, const struct MSVCRT_tm *mstm,
         MSVCRT___lc_time_data *time_data, MSVCRT__locale_t loc )
 {
+#if _MSVCR_VER <= 90
     char *s, *fmt;
     MSVCRT_size_t len;
 
@@ -1292,7 +1623,7 @@ static MSVCRT_size_t wcsftime_helper( MSVCRT_wchar_t *str, MSVCRT_size_t max,
 
     if ((s = MSVCRT_malloc( max*4 )))
     {
-        if (!strftime_helper( s, max*4, fmt, mstm, time_data, loc )) s[0] = 0;
+        if (!strftime_impl( s, max*4, fmt, mstm, time_data, loc )) s[0] = 0;
         len = MSVCRT__mbstowcs_l( str, s, max, loc );
         MSVCRT_free( s );
     }
@@ -1300,6 +1631,10 @@ static MSVCRT_size_t wcsftime_helper( MSVCRT_wchar_t *str, MSVCRT_size_t max,
 
     MSVCRT_free( fmt );
     return len;
+#else
+    TRACE("%p %ld %s %p %p %p\n", str, max, debugstr_w(format), mstm, time_data, loc);
+    return strftime_impl(str, max, format, mstm, time_data, loc);
+#endif
 }
 
 /*********************************************************************
