@@ -452,7 +452,7 @@ static BSTR alloc_str_from_narrow(const char *str)
 
 static BSTR _bstr_(const char *str)
 {
-    assert(alloced_bstrs_count < sizeof(alloced_bstrs)/sizeof(alloced_bstrs[0]));
+    assert(alloced_bstrs_count < ARRAY_SIZE(alloced_bstrs));
     alloced_bstrs[alloced_bstrs_count] = alloc_str_from_narrow(str);
     return alloced_bstrs[alloced_bstrs_count++];
 }
@@ -983,7 +983,7 @@ static void test_collection_content(void)
         content[i] = bstr;
 
         for (j = 0; j < i; ++j)
-            ok(winetest_strcmpW(content[j], bstr), "got duplicate entry\n");
+            ok(wcscmp(content[j], bstr), "got duplicate entry\n");
     }
 
     for (i = 0; i < 3; ++i)
@@ -1001,7 +1001,7 @@ static void test_collection_content(void)
             ok(bstr != NULL && *bstr, "expected non-empty string\n");
 
             for (j = 0; j < i; ++j)
-                ok(winetest_strcmpW(content[j], bstr), "got duplicate entry\n");
+                ok(wcscmp(content[j], bstr), "got duplicate entry\n");
             content[i] = bstr;
         }
 
@@ -1371,10 +1371,10 @@ static void test_XDR_datatypes(void)
             break;
         case VT_R8:
             if (!strcmp(ptr->typename, "float"))
-                ok(V_R8(&v) == (double)3.14159, "got %f\n", V_R8(&v));
+                ok(V_R8(&v) == 3.14159, "got %f\n", V_R8(&v));
             else
             todo_wine
-                ok(V_R8(&v) == (double)3.14159265358979323846, "got %.20f\n", V_R8(&v));
+                ok(V_R8(&v) == 3.14159265358979323846, "got %.20f\n", V_R8(&v));
             break;
         case VT_UI1:
             ok(V_UI1(&v) == 0xFF, "got %02x\n", V_UI1(&v));
