@@ -171,6 +171,9 @@ static void test_SQLWritePrivateProfileString(void)
    {
         HKEY hkey;
 
+        ret = SQLWritePrivateProfileString("wineodbc", "testing" , NULL, "odbc.ini");
+        ok(ret, "SQLWritePrivateProfileString failed\n");
+
         reg_ret = RegOpenKeyExW(HKEY_CURRENT_USER, odbc_key, 0, KEY_READ, &hkey);
         ok(reg_ret == ERROR_SUCCESS, "RegOpenKeyExW failed\n");
         if(reg_ret == ERROR_SUCCESS)
@@ -640,6 +643,11 @@ static void test_SQLGetInstalledDrivers(void)
     ok(!ret, "got %d\n", ret);
     check_error(ODBC_ERROR_INVALID_BUFF_LEN);
 
+    ret = SQLGetInstalledDrivers(buffer, 10, NULL);
+    ok(ret, "got %d\n", ret);
+    ok(strlen(buffer) == 8, "got len %u\n", lstrlenA(buffer));
+    ok(!buffer[9], "buffer not doubly null-terminated\n");
+
     ret = SQLGetInstalledDrivers(buffer, 10, &written);
     ok(ret, "got %d\n", ret);
     ok(strlen(buffer) == 8, "got len %u\n", lstrlenA(buffer));
@@ -679,6 +687,90 @@ static void test_SQLGetInstalledDrivers(void)
     SQLRemoveDriver("Wine test", TRUE, NULL);
 }
 
+static void test_SQLValidDSN(void)
+{
+    static const char *invalid = "[]{}(),;?*=!@\\";
+    char str[10];
+    int i;
+    BOOL ret;
+
+    strcpy(str, "wine10");
+    for(i = 0; i < strlen(invalid); i++)
+    {
+        str[4] = invalid[i];
+        ret = SQLValidDSN(str);
+        ok(!ret, "got %d\n", ret);
+    }
+
+    /* Too large */
+    ret = SQLValidDSN("Wine123456789012345678901234567890");
+    ok(!ret, "got %d\n", ret);
+
+    /* Valid with a space */
+    ret = SQLValidDSN("Wine Vinegar");
+    ok(ret, "got %d\n", ret);
+
+    /* Max DSN name value */
+    ret = SQLValidDSN("12345678901234567890123456789012");
+    ok(ret, "got %d\n", ret);
+}
+
+static void test_SQLValidDSNW(void)
+{
+    static const WCHAR invalid[] = {'[',']','{','}','(',')',',',';','?','*','=','!','@','\\',0};
+    static const WCHAR value[] = { 'w','i','n','e','1','0',0};
+    static const WCHAR too_large[] = { 'W','i','n','e','1','2','3','4','5','6','7','8','9','0','1','2','3','4','5',
+                                   '6','7','8','9','0','1','2','3','4','5','6','7','8','9','0', 0};
+    static const WCHAR with_space[] = { 'W','i','n','e',' ','V','i','n','e','g','a','r', 0};
+    static const WCHAR max_dsn[] = { '1','2','3','4','5','6','7','8','9','0','1','2','3','4','5','6','7','8','9','0',
+                                   '1','2','3','4','5','6','7','8','9','0','1','2', 0};
+    WCHAR str[10];
+    int i;
+    BOOL ret;
+
+    lstrcpyW(str, value);
+    for(i = 0; i < lstrlenW(invalid); i++)
+    {
+        str[4] = invalid[i];
+        ret = SQLValidDSNW(str);
+        ok(!ret, "got %d\n", ret);
+    }
+
+    ret = SQLValidDSNW(too_large);
+    ok(!ret, "got %d\n", ret);
+
+    ret = SQLValidDSNW(with_space);
+    ok(ret, "got %d\n", ret);
+
+    ret = SQLValidDSNW(max_dsn);
+    ok(ret, "got %d\n", ret);
+}
+
+static void test_SQLConfigDataSource(void)
+{
+    BOOL ret;
+
+    ret = SQLConfigDataSource(0, ODBC_ADD_DSN, "SQL Server", "DSN=WINEMQIS\0Database=MQIS\0\0");
+    todo_wine ok(ret, "got %d\n", ret);
+
+    ret = SQLConfigDataSource(0, ODBC_REMOVE_DSN, "SQL Server", "DSN=WINEMQIS\0\0");
+    todo_wine ok(ret, "got %d\n", ret);
+
+    ret = SQLConfigDataSource(0, ODBC_REMOVE_DSN, "SQL Server", "DSN=WINEMQIS\0\0");
+    if(!ret)
+    {
+        RETCODE ret;
+        DWORD err;
+        ret = SQLInstallerError(1, &err, NULL, 0, NULL);
+        ok(ret == SQL_SUCCESS_WITH_INFO, "got %d\n", ret);
+        todo_wine ok(err == ODBC_ERROR_INVALID_DSN, "got %u\n", err);
+    }
+
+    ret = SQLConfigDataSource(0, ODBC_ADD_DSN, "ODBC driver", "DSN=ODBC data source\0\0");
+    ok(!ret, "got %d\n", ret);
+    check_error(ODBC_ERROR_COMPONENT_NOT_FOUND);
+}
+
 START_TEST(misc)
 {
     test_SQLConfigMode();
@@ -690,4 +782,7 @@ START_TEST(misc)
     test_SQLInstallDriverEx();
     test_SQLInstallTranslatorEx();
     test_SQLGetInstalledDrivers();
+    test_SQLValidDSN();
+    test_SQLValidDSNW();
+    test_SQLConfigDataSource();
 }
