@@ -153,14 +153,21 @@ struct apartment
   BOOL main;               /* is this a main-threaded-apartment? (RO) */
 };
 
+struct init_spy
+{
+    struct list entry;
+    IInitializeSpy *spy;
+    unsigned int id;
+};
+
 /* this is what is stored in TEB->ReservedForOle */
 struct oletls
 {
     struct apartment *apt;
     IErrorInfo       *errorinfo;   /* see errorinfo.c */
-    IUnknown         *state;       /* see CoSetState */
+    DWORD             thread_seqid;/* returned with CoGetCurrentProcess */
     DWORD             apt_mask;    /* apartment mask (+0Ch on x86) */
-    IInitializeSpy   *spy;         /* The "SPY" from CoInitializeSpy */
+    void            *unknown0;
     DWORD            inits;        /* number of times CoInitializeEx called */
     DWORD            ole_inits;    /* number of times OleInitialize called */
     GUID             causality_id; /* unique identifier for each COM call */
@@ -171,6 +178,9 @@ struct oletls
     IUnknown        *call_state;    /* current call context (+3Ch on x86) */
     DWORD            unknown2[46];
     IUnknown        *cancel_object; /* cancel object set by CoSetCancelObject (+F8h on x86) */
+    IUnknown        *state;       /* see CoSetState */
+    struct list      spies;         /* Spies installed with CoRegisterInitializeSpy */
+    DWORD            spies_lock;
 };
 
 
@@ -262,7 +272,12 @@ APARTMENT *apartment_get_current_or_mta(void) DECLSPEC_HIDDEN;
 static inline struct oletls *COM_CurrentInfo(void)
 {
     if (!NtCurrentTeb()->ReservedForOle)
-        NtCurrentTeb()->ReservedForOle = heap_alloc_zero(sizeof(struct oletls));
+    {
+        struct oletls *oletls = heap_alloc_zero(sizeof(*oletls));
+        if (oletls)
+            list_init(&oletls->spies);
+        NtCurrentTeb()->ReservedForOle = oletls;
+    }
 
     return NtCurrentTeb()->ReservedForOle;
 }
@@ -341,4 +356,6 @@ static inline HRESULT copy_formatetc(FORMATETC *dst, const FORMATETC *src)
 extern HRESULT EnumSTATDATA_Construct(IUnknown *holder, ULONG index, DWORD array_len, STATDATA *data,
                                       BOOL copy, IEnumSTATDATA **ppenum) DECLSPEC_HIDDEN;
 
+extern DWORD rpcss_get_next_seqid(void) DECLSPEC_HIDDEN;
+
 #endif /* __WINE_OLE_COMPOBJ_H */
