@@ -23,8 +23,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <assert.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -44,7 +42,6 @@
 #include "ole2.h"
 #include "ole2ver.h"
 
-#include "wine/unicode.h"
 #include "compobj_private.h"
 #include "olestd.h"
 #include "wine/list.h"
@@ -697,7 +694,7 @@ HRESULT WINAPI OleRegGetUserType(REFCLSID clsid, DWORD form, LPOLESTR *usertype)
   {
     HKEY auxkey;
 
-    sprintfW(auxkeynameW, auxusertypeW, form);
+    swprintf(auxkeynameW, ARRAY_SIZE(auxkeynameW), auxusertypeW, form);
     if (COM_OpenKeyForCLSID(clsid, auxkeynameW, KEY_READ, &auxkey) == S_OK)
     {
       if (!RegQueryValueExW(auxkey, emptyW, NULL, &valuetype, NULL, &valuelen) && valuelen)
@@ -883,7 +880,7 @@ HRESULT WINAPI OleRegGetMiscStatus(
   /*
    * Open the key specific to the requested aspect.
    */
-  sprintfW(keyName, dfmtW, dwAspect);
+  swprintf(keyName, ARRAY_SIZE(keyName), dfmtW, dwAspect);
 
   result = open_classes_key(miscStatusKey, keyName, KEY_READ, &aspectKey);
   if (result == ERROR_SUCCESS)
@@ -967,7 +964,7 @@ static HRESULT WINAPI EnumOLEVERB_Next(
         LPWSTR pwszOLEVERB;
         LPWSTR pwszMenuFlags;
         LPWSTR pwszAttribs;
-        LONG res = RegEnumKeyW(This->hkeyVerb, This->index, wszSubKey, sizeof(wszSubKey)/sizeof(wszSubKey[0]));
+        LONG res = RegEnumKeyW(This->hkeyVerb, This->index, wszSubKey, ARRAY_SIZE(wszSubKey));
         if (res == ERROR_NO_MORE_ITEMS)
         {
             hr = S_FALSE;
@@ -1002,7 +999,7 @@ static HRESULT WINAPI EnumOLEVERB_Next(
         }
 
         TRACE("verb string: %s\n", debugstr_w(pwszOLEVERB));
-        pwszMenuFlags = strchrW(pwszOLEVERB, ',');
+        pwszMenuFlags = wcschr(pwszOLEVERB, ',');
         if (!pwszMenuFlags)
         {
             hr = OLEOBJ_E_INVALIDVERB;
@@ -1012,7 +1009,7 @@ static HRESULT WINAPI EnumOLEVERB_Next(
         /* nul terminate the name string and advance to first character */
         *pwszMenuFlags = '\0';
         pwszMenuFlags++;
-        pwszAttribs = strchrW(pwszMenuFlags, ',');
+        pwszAttribs = wcschr(pwszMenuFlags, ',');
         if (!pwszAttribs)
         {
             hr = OLEOBJ_E_INVALIDVERB;
@@ -1024,10 +1021,10 @@ static HRESULT WINAPI EnumOLEVERB_Next(
         pwszAttribs++;
 
         /* fill out structure for this verb */
-        rgelt->lVerb = atolW(wszSubKey);
+        rgelt->lVerb = wcstol(wszSubKey, NULL, 10);
         rgelt->lpszVerbName = pwszOLEVERB; /* user should free */
-        rgelt->fuFlags = atolW(pwszMenuFlags);
-        rgelt->grfAttribs = atolW(pwszAttribs);
+        rgelt->fuFlags = wcstol(pwszMenuFlags, NULL, 10);
+        rgelt->grfAttribs = wcstol(pwszAttribs, NULL, 10);
 
         if (pceltFetched)
             (*pceltFetched)++;
@@ -2409,7 +2406,7 @@ static void OLEUTL_ReadRegistryDWORDValue(
       case REG_EXPAND_SZ:
       case REG_MULTI_SZ:
       case REG_SZ:
-	*pdwValue = (DWORD)strtoulW(buffer, NULL, 10);
+	*pdwValue = wcstoul(buffer, NULL, 10);
 	break;
     }
   }
@@ -2601,7 +2598,7 @@ HRESULT WINAPI OleSetAutoConvert(REFCLSID clsidOld, REFCLSID clsidNew)
     if (FAILED(res))
         goto done;
     StringFromGUID2(clsidNew, szClsidNew, CHARS_IN_GUID);
-    if (RegSetValueW(hkey, wszAutoConvertTo, REG_SZ, szClsidNew, (strlenW(szClsidNew)+1) * sizeof(WCHAR)))
+    if (RegSetValueW(hkey, wszAutoConvertTo, REG_SZ, szClsidNew, (lstrlenW(szClsidNew)+1) * sizeof(WCHAR)))
     {
         res = REGDB_E_WRITEREGDB;
 	goto done;
@@ -2797,6 +2794,25 @@ static inline HRESULT PROPVARIANT_ValidateType(VARTYPE vt)
     case VT_FILETIME|VT_VECTOR:
     case VT_CF|VT_VECTOR:
     case VT_CLSID|VT_VECTOR:
+    case VT_ARRAY|VT_I1:
+    case VT_ARRAY|VT_UI1:
+    case VT_ARRAY|VT_I2:
+    case VT_ARRAY|VT_UI2:
+    case VT_ARRAY|VT_I4:
+    case VT_ARRAY|VT_UI4:
+    case VT_ARRAY|VT_INT:
+    case VT_ARRAY|VT_UINT:
+    case VT_ARRAY|VT_R4:
+    case VT_ARRAY|VT_R8:
+    case VT_ARRAY|VT_CY:
+    case VT_ARRAY|VT_DATE:
+    case VT_ARRAY|VT_BSTR:
+    case VT_ARRAY|VT_BOOL:
+    case VT_ARRAY|VT_DECIMAL:
+    case VT_ARRAY|VT_DISPATCH:
+    case VT_ARRAY|VT_UNKNOWN:
+    case VT_ARRAY|VT_ERROR:
+    case VT_ARRAY|VT_VARIANT:
         return S_OK;
     }
     WARN("Bad type %d\n", vt);
@@ -2908,6 +2924,8 @@ HRESULT WINAPI PropVariantClear(PROPVARIANT * pvar) /* [in/out] */
                 CoTaskMemFree(pvar->u.capropvar.pElems);
             }
         }
+        else if (pvar->vt & VT_ARRAY)
+            hr = SafeArrayDestroy(pvar->u.parray);
         else
         {
             WARN("Invalid/unsupported type %d\n", pvar->vt);
@@ -3088,6 +3106,11 @@ HRESULT WINAPI PropVariantCopy(PROPVARIANT *pvarDest,      /* [out] */
             else
                 CopyMemory(pvarDest->u.capropvar.pElems, pvarSrc->u.capropvar.pElems, len * elemSize);
         }
+        else if (pvarSrc->vt & VT_ARRAY)
+        {
+            pvarDest->u.uhVal.QuadPart = 0;
+            return SafeArrayCopy(pvarSrc->u.parray, &pvarDest->u.parray);
+        }
         else
             WARN("Invalid/unsupported type %d\n", pvarSrc->vt);
     }
