@@ -59,7 +59,6 @@ DEFINE_EXPECT(ReceiveConnection);
 DEFINE_EXPECT(GetAllocatorRequirements);
 DEFINE_EXPECT(NotifyAllocator);
 DEFINE_EXPECT(Reconnect);
-DEFINE_EXPECT(Read_FccHandler);
 DEFINE_EXPECT(MediaSeeking_GetPositions);
 DEFINE_EXPECT(MemAllocator_GetProperties);
 DEFINE_EXPECT(MemInputPin_QueryInterface_IStream);
@@ -74,28 +73,6 @@ DEFINE_EXPECT(MediaSample_GetActualDataLength);
 DEFINE_EXPECT(MediaSample_GetSize);
 DEFINE_EXPECT(MediaSample_GetMediaTime);
 
-static int strcmp_wa(LPCWSTR strw, const char *stra)
-{
-    CHAR buf[512];
-    WideCharToMultiByte(CP_ACP, 0, strw, -1, buf, sizeof(buf), NULL, NULL);
-    return lstrcmpA(stra, buf);
-}
-
-static BSTR a2bstr(const char *str)
-{
-    BSTR ret;
-    int len;
-
-    if(!str)
-        return NULL;
-
-    len = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
-    ret = SysAllocStringLen(NULL, len-1);
-    MultiByteToWideChar(CP_ACP, 0, str, -1, ret, len);
-
-    return ret;
-}
-
 typedef enum {
     SOURCE_FILTER,
     SINK_FILTER,
@@ -1154,46 +1131,6 @@ static void init_test_filter(test_filter *This, PIN_DIRECTION dir, filter_type t
     This->filter_type = type;
 }
 
-static void test_AviMux_QueryInterface(void)
-{
-    IUnknown *avimux, *unk;
-    HRESULT hr;
-
-    hr = CoCreateInstance(&CLSID_AviDest, NULL, CLSCTX_INPROC_SERVER, &IID_IUnknown, (void**)&avimux);
-    ok(hr == S_OK || broken(hr == REGDB_E_CLASSNOTREG),
-            "couldn't create AVI Mux filter, hr = %08x\n", hr);
-    if(hr != S_OK) {
-        win_skip("AVI Mux filter is not registered\n");
-        return;
-    }
-
-    hr = IUnknown_QueryInterface(avimux, &IID_IBaseFilter, (void**)&unk);
-    ok(hr == S_OK, "QueryInterface(IID_IBaseFilter) failed: %x\n", hr);
-    IUnknown_Release(unk);
-
-    hr = IUnknown_QueryInterface(avimux, &IID_IConfigAviMux, (void**)&unk);
-    ok(hr == S_OK, "QueryInterface(IID_IConfigAviMux) failed: %x\n", hr);
-    IUnknown_Release(unk);
-
-    hr = IUnknown_QueryInterface(avimux, &IID_IConfigInterleaving, (void**)&unk);
-    ok(hr == S_OK, "QueryInterface(IID_IConfigInterleaving) failed: %x\n", hr);
-    IUnknown_Release(unk);
-
-    hr = IUnknown_QueryInterface(avimux, &IID_IMediaSeeking, (void**)&unk);
-    ok(hr == S_OK, "QueryInterface(IID_IMediaSeeking) failed: %x\n", hr);
-    IUnknown_Release(unk);
-
-    hr = IUnknown_QueryInterface(avimux, &IID_IPersistMediaPropertyBag, (void**)&unk);
-    ok(hr == S_OK, "QueryInterface(IID_IPersistMediaPropertyBag) failed: %x\n", hr);
-    IUnknown_Release(unk);
-
-    hr = IUnknown_QueryInterface(avimux, &IID_ISpecifyPropertyPages, (void**)&unk);
-    ok(hr == S_OK, "QueryInterface(IID_ISpecifyPropertyPages) failed: %x\n", hr);
-    IUnknown_Release(unk);
-
-    IUnknown_Release(avimux);
-}
-
 static HRESULT WINAPI MemAllocator_QueryInterface(IMemAllocator *iface, REFIID riid, void **ppvObject)
 {
     if(IsEqualIID(riid, &IID_IUnknown)) {
@@ -1428,11 +1365,9 @@ static void test_AviMux(char *arg)
     VIDEOINFO videoinfo;
     IPin *avimux_in, *avimux_out, *pin;
     AM_MEDIA_TYPE source_media_type;
-    AM_MEDIA_TYPE *media_type;
     PIN_DIRECTION dir;
     IBaseFilter *avimux;
     IEnumPins *ep;
-    IEnumMediaTypes *emt;
     IMemInputPin *memin;
     ALLOCATOR_PROPERTIES props;
     IMemAllocator *memalloc;
@@ -1468,35 +1403,6 @@ static void test_AviMux(char *arg)
     ok(dir == PINDIR_INPUT, "dir = %d\n", dir);
     IEnumPins_Release(ep);
 
-    hr = IPin_EnumMediaTypes(avimux_out, &emt);
-    ok(hr == S_OK, "EnumMediaTypes returned %x\n", hr);
-    hr = IEnumMediaTypes_Next(emt, 1, &media_type, NULL);
-    ok(hr == S_OK, "Next returned %x\n", hr);
-    ok(IsEqualIID(&media_type->majortype, &MEDIATYPE_Stream), "majortype = %s\n",
-            wine_dbgstr_guid(&media_type->majortype));
-    ok(IsEqualIID(&media_type->subtype, &MEDIASUBTYPE_Avi), "subtype = %s\n",
-            wine_dbgstr_guid(&media_type->subtype));
-    ok(media_type->bFixedSizeSamples, "bFixedSizeSamples = %x\n", media_type->bFixedSizeSamples);
-    ok(!media_type->bTemporalCompression, "bTemporalCompression = %x\n", media_type->bTemporalCompression);
-    ok(media_type->lSampleSize == 1, "lSampleSize = %d\n", media_type->lSampleSize);
-    ok(IsEqualIID(&media_type->formattype, &GUID_NULL), "formattype = %s\n",
-            wine_dbgstr_guid(&media_type->formattype));
-    ok(!media_type->pUnk, "pUnk = %p\n", media_type->pUnk);
-    ok(!media_type->cbFormat, "cbFormat = %d\n", media_type->cbFormat);
-    ok(!media_type->pbFormat, "pbFormat = %p\n", media_type->pbFormat);
-    CoTaskMemFree(media_type);
-    hr = IEnumMediaTypes_Next(emt, 1, &media_type, NULL);
-    ok(hr == S_FALSE, "Next returned %x\n", hr);
-    IEnumMediaTypes_Release(emt);
-
-    hr = IPin_EnumMediaTypes(avimux_in, &emt);
-    ok(hr == S_OK, "EnumMediaTypes returned %x\n", hr);
-    hr = IEnumMediaTypes_Reset(emt);
-    ok(hr == S_OK, "Reset returned %x\n", hr);
-    hr = IEnumMediaTypes_Next(emt, 1, &media_type, NULL);
-    ok(hr == S_FALSE, "Next returned %x\n", hr);
-    IEnumMediaTypes_Release(emt);
-
     hr = IPin_ReceiveConnection(avimux_in, &source_filter.IPin_iface, NULL);
     ok(hr == E_POINTER, "ReceiveConnection returned %x\n", hr);
 
@@ -1737,119 +1643,6 @@ static void test_AviMux(char *arg)
     ok(ref == 0, "IStream was not destroyed (%d)\n", ref);
 }
 
-static HRESULT WINAPI PropertyBag_QueryInterface(IPropertyBag *iface, REFIID riid, void **ppv)
-{
-    if(IsEqualGUID(&IID_IUnknown, riid) || IsEqualGUID(&IID_IPropertyBag, riid)) {
-        *ppv = iface;
-        return S_OK;
-    }
-
-    ok(0, "unexpected call %s\n", wine_dbgstr_guid(riid));
-    *ppv = NULL;
-    return E_NOINTERFACE;
-}
-
-static ULONG WINAPI PropertyBag_AddRef(IPropertyBag *iface)
-{
-    return 2;
-}
-
-static ULONG WINAPI PropertyBag_Release(IPropertyBag *iface)
-{
-    return 1;
-}
-
-static HRESULT WINAPI PropertyBag_Read(IPropertyBag *iface, LPCOLESTR pszPropName, VARIANT *pVar, IErrorLog *pErrorLog)
-{
-    ok(!pErrorLog, "pErrorLog = %p\n", pErrorLog);
-
-    if(!strcmp_wa(pszPropName, "FccHandler")) {
-        CHECK_EXPECT(Read_FccHandler);
-        V_VT(pVar) = VT_BSTR;
-        V_BSTR(pVar) = a2bstr("mrle");
-        return S_OK;
-    }
-
-    ok(0, "unexpected call: %s\n", wine_dbgstr_w(pszPropName));
-    return E_NOTIMPL;
-}
-
-static HRESULT WINAPI PropertyBag_Write(IPropertyBag *iface, LPCOLESTR pszPropName, VARIANT *pVar)
-{
-    ok(0, "unexpected call: %s\n", wine_dbgstr_w(pszPropName));
-    return E_NOTIMPL;
-}
-
-static const IPropertyBagVtbl PropertyBagVtbl = {
-    PropertyBag_QueryInterface,
-    PropertyBag_AddRef,
-    PropertyBag_Release,
-    PropertyBag_Read,
-    PropertyBag_Write
-};
-
-static IPropertyBag PropertyBag = { &PropertyBagVtbl };
-
-static void test_AviCo(void)
-{
-    IPersistPropertyBag *persist_bag;
-    IPin *pin, *in_pin, *out_pin;
-    IEnumPins *enum_pins;
-    IBaseFilter *avico;
-    PIN_INFO pin_info;
-    HRESULT hres;
-
-    static const WCHAR inputW[] = {'I','n','p','u','t',0};
-    static const WCHAR outputW[] = {'O','u','t','p','u','t',0};
-
-    hres = CoCreateInstance(&CLSID_AVICo, NULL, CLSCTX_INPROC_SERVER, &IID_IBaseFilter, (void**)&avico);
-    if(hres == REGDB_E_CLASSNOTREG || hres == CLASS_E_CLASSNOTAVAILABLE) {
-        win_skip("CLSID_AVICo not registered/available\n");
-        return;
-    }
-    ok(hres == S_OK, "Could not create CLSID_AVICo class: %08x\n", hres);
-
-    hres = IBaseFilter_QueryInterface(avico, &IID_IPin, (void**)&pin);
-    ok(hres == E_NOINTERFACE, "QueryInterface(IID_IPin) returned: %08x\n", hres);
-
-    hres = IBaseFilter_QueryInterface(avico, &IID_IPersistPropertyBag, (void**)&persist_bag);
-    ok(hres == S_OK, "QueryInterface(IID_IPersistPropertyBag) returned: %08x\n", hres);
-
-    SET_EXPECT(Read_FccHandler);
-    hres = IPersistPropertyBag_Load(persist_bag, &PropertyBag, NULL);
-    ok(hres == S_OK, "Load failed: %08x\n", hres);
-    CHECK_CALLED(Read_FccHandler);
-
-    IPersistPropertyBag_Release(persist_bag);
-
-    hres = IBaseFilter_EnumPins(avico, &enum_pins);
-    ok(hres == S_OK, "EnumPins failed: %08x\n", hres);
-
-    hres = IEnumPins_Next(enum_pins, 1, &in_pin, NULL);
-    ok(hres == S_OK, "Next failed: %08x\n", hres);
-
-    hres = IPin_QueryPinInfo(in_pin, &pin_info);
-    ok(hres == S_OK, "QueryPinInfo failed: %08x\n", hres);
-    ok(pin_info.pFilter == avico, "pin_info.pFilter != avico\n");
-    ok(pin_info.dir == PINDIR_INPUT, "pin_info.dir = %d\n", pin_info.dir);
-    ok(!lstrcmpW(pin_info.achName, inputW), "pin_info.achName = %s\n", wine_dbgstr_w(pin_info.achName));
-
-    hres = IEnumPins_Next(enum_pins, 1, &out_pin, NULL);
-    ok(hres == S_OK, "Next failed: %08x\n", hres);
-
-    hres = IPin_QueryPinInfo(out_pin, &pin_info);
-    ok(hres == S_OK, "QueryPinInfo failed: %08x\n", hres);
-    ok(pin_info.pFilter == avico, "pin_info.pFilter != avico\n");
-    ok(pin_info.dir == PINDIR_OUTPUT, "pin_info.dir = %d\n", pin_info.dir);
-    ok(!lstrcmpW(pin_info.achName, outputW), "pin_info.achName = %s\n", wine_dbgstr_w(pin_info.achName));
-
-    IEnumPins_Release(enum_pins);
-
-    IPin_Release(in_pin);
-    IPin_Release(out_pin);
-    IBaseFilter_Release(avico);
-}
-
 /* Outer IUnknown for COM aggregation tests */
 struct unk_impl {
     IUnknown IUnknown_iface;
@@ -1907,9 +1700,9 @@ static void test_COM_vfwcapture(void)
     /* COM aggregation */
     hr = CoCreateInstance(&CLSID_VfwCapture, &unk_obj.IUnknown_iface, CLSCTX_INPROC_SERVER,
             &IID_IUnknown, (void**)&unk_obj.inner_unk);
-    if (hr == REGDB_E_CLASSNOTREG)
+    if ((hr == REGDB_E_CLASSNOTREG) || (hr == CLASS_E_CLASSNOTAVAILABLE))
     {
-        win_skip("CLSID_VfwCapture not registered\n");
+        win_skip("CLSID_VfwCapture not supported (0x%x)\n", hr);
         return;
     }
     ok(hr == S_OK, "VfwCapture create failed: %08x\n", hr);
@@ -1985,7 +1778,7 @@ static void test_COM_vfwcapture(void)
 
     /* Unsupported interfaces */
     hr = IBaseFilter_QueryInterface(bf, &IID_IAMStreamConfig, (void**)&unk);
-    todo_wine ok(hr == E_NOINTERFACE, "QueryInterface for IID_IAMStreamConfig failed: %08x\n", hr);
+    ok(hr == E_NOINTERFACE, "QueryInterface for IID_IAMStreamConfig failed: %08x\n", hr);
     hr = IBaseFilter_QueryInterface(bf, &IID_IAMVideoProcAmp, (void**)&unk);
     todo_wine ok(hr == E_NOINTERFACE, "QueryInterface for IID_IAMVideoProcAmp failed: %08x\n", hr);
     hr = IBaseFilter_QueryInterface(bf, &IID_IOverlayNotify, (void**)&unk);
@@ -2003,9 +1796,7 @@ START_TEST(qcap)
 
         arg_c = winetest_get_mainargs(&arg_v);
 
-        test_AviMux_QueryInterface();
         test_AviMux(arg_c>2 ? arg_v[2] : NULL);
-        test_AviCo();
         test_COM_vfwcapture();
 
         CoUninitialize();
