@@ -46,27 +46,22 @@ WINE_DEFAULT_DEBUG_CHANNEL(qcap);
 
 typedef struct VfwCapture
 {
-    BaseFilter filter;
+    struct strmbase_filter filter;
     IAMStreamConfig IAMStreamConfig_iface;
     IAMVideoProcAmp IAMVideoProcAmp_iface;
     IPersistPropertyBag IPersistPropertyBag_iface;
     BOOL init;
     Capture *driver_info;
 
-    BaseOutputPin source;
+    struct strmbase_source source;
     IKsPropertySet IKsPropertySet_iface;
 } VfwCapture;
 
-static inline VfwCapture *impl_from_BaseFilter(BaseFilter *iface)
+static inline VfwCapture *impl_from_strmbase_filter(struct strmbase_filter *iface)
 {
     return CONTAINING_RECORD(iface, VfwCapture, filter);
 }
 
-static inline VfwCapture *impl_from_IBaseFilter(IBaseFilter *iface)
-{
-    return CONTAINING_RECORD(iface, VfwCapture, filter.IBaseFilter_iface);
-}
-
 static inline VfwCapture *impl_from_IAMStreamConfig(IAMStreamConfig *iface)
 {
     return CONTAINING_RECORD(iface, VfwCapture, IAMStreamConfig_iface);
@@ -82,35 +77,30 @@ static inline VfwCapture *impl_from_IPersistPropertyBag(IPersistPropertyBag *ifa
     return CONTAINING_RECORD(iface, VfwCapture, IPersistPropertyBag_iface);
 }
 
-static inline VfwCapture *impl_from_IPin(IPin *iface)
+static struct strmbase_pin *vfw_capture_get_pin(struct strmbase_filter *iface, unsigned int index)
 {
-    return CONTAINING_RECORD(iface, VfwCapture, source.pin.IPin_iface);
-}
-
-static IPin *vfw_capture_get_pin(BaseFilter *iface, unsigned int index)
-{
-    VfwCapture *This = impl_from_BaseFilter(iface);
+    VfwCapture *This = impl_from_strmbase_filter(iface);
 
     if (index >= 1)
         return NULL;
 
-    return &This->source.pin.IPin_iface;
+    return &This->source.pin;
 }
 
-static void vfw_capture_destroy(BaseFilter *iface)
+static void vfw_capture_destroy(struct strmbase_filter *iface)
 {
-    VfwCapture *filter = impl_from_BaseFilter(iface);
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
 
     if (filter->init)
     {
         if (filter->filter.state != State_Stopped)
-            qcap_driver_stop(filter->driver_info, &filter->filter.state);
+            qcap_driver_stop_stream(filter->driver_info);
         qcap_driver_destroy(filter->driver_info);
     }
 
-    if (filter->source.pin.pConnectedTo)
+    if (filter->source.pin.peer)
     {
-        IPin_Disconnect(filter->source.pin.pConnectedTo);
+        IPin_Disconnect(filter->source.pin.peer);
         IPin_Disconnect(&filter->source.pin.IPin_iface);
     }
     strmbase_source_cleanup(&filter->source);
@@ -119,9 +109,9 @@ static void vfw_capture_destroy(BaseFilter *iface)
     ObjectRefCount(FALSE);
 }
 
-static HRESULT vfw_capture_query_interface(BaseFilter *iface, REFIID iid, void **out)
+static HRESULT vfw_capture_query_interface(struct strmbase_filter *iface, REFIID iid, void **out)
 {
-    VfwCapture *filter = impl_from_BaseFilter(iface);
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
 
     if (IsEqualGUID(iid, &IID_IPersistPropertyBag))
         *out = &filter->IPersistPropertyBag_iface;
@@ -134,54 +124,47 @@ static HRESULT vfw_capture_query_interface(BaseFilter *iface, REFIID iid, void *
     return S_OK;
 }
 
-static const BaseFilterFuncTable BaseFuncTable = {
+static HRESULT vfw_capture_init_stream(struct strmbase_filter *iface)
+{
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
+
+    qcap_driver_init_stream(filter->driver_info);
+    return VFW_S_CANT_CUE;
+}
+
+static HRESULT vfw_capture_start_stream(struct strmbase_filter *iface, REFERENCE_TIME time)
+{
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
+
+    qcap_driver_start_stream(filter->driver_info);
+    return S_OK;
+}
+
+static HRESULT vfw_capture_stop_stream(struct strmbase_filter *iface)
+{
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
+
+    qcap_driver_stop_stream(filter->driver_info);
+    return VFW_S_CANT_CUE;
+}
+
+static HRESULT vfw_capture_cleanup_stream(struct strmbase_filter *iface)
+{
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
+
+    qcap_driver_cleanup_stream(filter->driver_info);
+    return S_OK;
+}
+
+static const struct strmbase_filter_ops filter_ops =
+{
     .filter_get_pin = vfw_capture_get_pin,
     .filter_destroy = vfw_capture_destroy,
     .filter_query_interface = vfw_capture_query_interface,
-};
-
-/** IMediaFilter methods **/
-
-static HRESULT WINAPI VfwCapture_Stop(IBaseFilter * iface)
-{
-    VfwCapture *This = impl_from_IBaseFilter(iface);
-
-    TRACE("()\n");
-    return qcap_driver_stop(This->driver_info, &This->filter.state);
-}
-
-static HRESULT WINAPI VfwCapture_Pause(IBaseFilter * iface)
-{
-    VfwCapture *This = impl_from_IBaseFilter(iface);
-
-    TRACE("()\n");
-    return qcap_driver_pause(This->driver_info, &This->filter.state);
-}
-
-static HRESULT WINAPI VfwCapture_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
-{
-    VfwCapture *This = impl_from_IBaseFilter(iface);
-    TRACE("(%s)\n", wine_dbgstr_longlong(tStart));
-    return qcap_driver_run(This->driver_info, &This->filter.state);
-}
-
-static const IBaseFilterVtbl VfwCapture_Vtbl =
-{
-    BaseFilterImpl_QueryInterface,
-    BaseFilterImpl_AddRef,
-    BaseFilterImpl_Release,
-    BaseFilterImpl_GetClassID,
-    VfwCapture_Stop,
-    VfwCapture_Pause,
-    VfwCapture_Run,
-    BaseFilterImpl_GetState,
-    BaseFilterImpl_SetSyncSource,
-    BaseFilterImpl_GetSyncSource,
-    BaseFilterImpl_EnumPins,
-    BaseFilterImpl_FindPin,
-    BaseFilterImpl_QueryFilterInfo,
-    BaseFilterImpl_JoinFilterGraph,
-    BaseFilterImpl_QueryVendorInfo
+    .filter_init_stream = vfw_capture_init_stream,
+    .filter_start_stream = vfw_capture_start_stream,
+    .filter_stop_stream = vfw_capture_stop_stream,
+    .filter_cleanup_stream = vfw_capture_cleanup_stream,
 };
 
 /* AMStreamConfig interface, we only need to implement {G,S}etFormat */
@@ -209,7 +192,8 @@ AMStreamConfig_SetFormat(IAMStreamConfig *iface, AM_MEDIA_TYPE *pmt)
     HRESULT hr;
     VfwCapture *This = impl_from_IAMStreamConfig(iface);
 
-    TRACE("(%p): %p->%p\n", iface, pmt, pmt ? pmt->pbFormat : NULL);
+    TRACE("filter %p, mt %p.\n", This, pmt);
+    strmbase_dump_media_type(pmt);
 
     if (This->filter.state != State_Stopped)
     {
@@ -223,18 +207,16 @@ AMStreamConfig_SetFormat(IAMStreamConfig *iface, AM_MEDIA_TYPE *pmt)
         return E_POINTER;
     }
 
-    dump_AM_MEDIA_TYPE(pmt);
-
-    if (This->source.pin.pConnectedTo)
+    if (This->source.pin.peer)
     {
-        hr = IPin_QueryAccept(This->source.pin.pConnectedTo, pmt);
+        hr = IPin_QueryAccept(This->source.pin.peer, pmt);
         TRACE("Would accept: %d\n", hr);
         if (hr == S_FALSE)
             return VFW_E_INVALIDMEDIATYPE;
     }
 
     hr = qcap_driver_set_format(This->driver_info, pmt);
-    if (SUCCEEDED(hr) && This->filter.filterInfo.pGraph && This->source.pin.pConnectedTo)
+    if (SUCCEEDED(hr) && This->filter.filterInfo.pGraph && This->source.pin.peer)
     {
         hr = IFilterGraph_Reconnect(This->filter.filterInfo.pGraph, &This->source.pin.IPin_iface);
         if (SUCCEEDED(hr))
@@ -248,9 +230,13 @@ static HRESULT WINAPI
 AMStreamConfig_GetFormat( IAMStreamConfig *iface, AM_MEDIA_TYPE **pmt )
 {
     VfwCapture *This = impl_from_IAMStreamConfig(iface);
+    HRESULT hr;
 
     TRACE("%p -> (%p)\n", iface, pmt);
-    return qcap_driver_get_format(This->driver_info, pmt);
+    hr = qcap_driver_get_format(This->driver_info, pmt);
+    if (SUCCEEDED(hr))
+        strmbase_dump_media_type(*pmt);
+    return hr;
 }
 
 static HRESULT WINAPI
@@ -506,25 +492,24 @@ static const IKsPropertySetVtbl IKsPropertySet_VTable =
    KSP_QuerySupported
 };
 
-static inline VfwCapture *impl_from_BasePin(BasePin *pin)
+static inline VfwCapture *impl_from_strmbase_pin(struct strmbase_pin *pin)
 {
     return CONTAINING_RECORD(pin, VfwCapture, source.pin);
 }
 
-static HRESULT WINAPI VfwPin_CheckMediaType(BasePin *pin, const AM_MEDIA_TYPE *mt)
+static HRESULT source_query_accept(struct strmbase_pin *pin, const AM_MEDIA_TYPE *mt)
 {
-    VfwCapture *filter = impl_from_BasePin(pin);
+    VfwCapture *filter = impl_from_strmbase_pin(pin);
     return qcap_driver_check_format(filter->driver_info, mt);
 }
 
-static HRESULT WINAPI VfwPin_GetMediaType(BasePin *pin, int iPosition, AM_MEDIA_TYPE *pmt)
+static HRESULT source_get_media_type(struct strmbase_pin *pin,
+        unsigned int iPosition, AM_MEDIA_TYPE *pmt)
 {
-    VfwCapture *filter = impl_from_BasePin(pin);
+    VfwCapture *filter = impl_from_strmbase_pin(pin);
     AM_MEDIA_TYPE *vfw_pmt;
     HRESULT hr;
 
-    if (iPosition < 0)
-        return E_INVALIDARG;
     if (iPosition > 0)
         return VFW_S_NO_MORE_ITEMS;
 
@@ -536,7 +521,23 @@ static HRESULT WINAPI VfwPin_GetMediaType(BasePin *pin, int iPosition, AM_MEDIA_
     return hr;
 }
 
-static HRESULT WINAPI VfwPin_DecideBufferSize(BaseOutputPin *iface, IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *ppropInputRequest)
+static HRESULT source_query_interface(struct strmbase_pin *iface, REFIID iid, void **out)
+{
+    VfwCapture *filter = impl_from_strmbase_pin(iface);
+
+    if (IsEqualGUID(iid, &IID_IKsPropertySet))
+        *out = &filter->IKsPropertySet_iface;
+    else if (IsEqualGUID(iid, &IID_IAMStreamConfig))
+        *out = &filter->IAMStreamConfig_iface;
+    else
+        return E_NOINTERFACE;
+
+    IUnknown_AddRef((IUnknown *)*out);
+    return S_OK;
+}
+
+static HRESULT WINAPI VfwPin_DecideBufferSize(struct strmbase_source *iface,
+        IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *ppropInputRequest)
 {
     ALLOCATOR_PROPERTIES actual;
 
@@ -552,89 +553,20 @@ static HRESULT WINAPI VfwPin_DecideBufferSize(BaseOutputPin *iface, IMemAllocato
     return IMemAllocator_SetProperties(pAlloc, ppropInputRequest, &actual);
 }
 
-static const BaseOutputPinFuncTable output_BaseOutputFuncTable = {
-    {
-        VfwPin_CheckMediaType,
-        VfwPin_GetMediaType
-    },
-    BaseOutputPinImpl_AttemptConnection,
-    VfwPin_DecideBufferSize,
-    BaseOutputPinImpl_DecideAllocator,
-};
-
-static HRESULT WINAPI VfwPin_QueryInterface(IPin * iface, REFIID riid, LPVOID * ppv)
+static const struct strmbase_source_ops source_ops =
 {
-    VfwCapture *filter = impl_from_IPin(iface);
-
-    TRACE("%s %p\n", debugstr_guid(riid), ppv);
-
-    *ppv = NULL;
-    if (IsEqualIID(riid, &IID_IUnknown) || IsEqualIID(riid, &IID_IPin))
-        *ppv = &filter->source.pin.IPin_iface;
-    else if (IsEqualIID(riid, &IID_IKsPropertySet))
-        *ppv = &filter->IKsPropertySet_iface;
-    else if (IsEqualIID(riid, &IID_IAMStreamConfig))
-        *ppv = &filter->IAMStreamConfig_iface;
-
-    if (*ppv)
-    {
-        IUnknown_AddRef((IUnknown *)(*ppv));
-        return S_OK;
-    }
-
-    FIXME("No interface for %s!\n", debugstr_guid(riid));
-    return E_NOINTERFACE;
-}
-
-static HRESULT WINAPI
-VfwPin_EnumMediaTypes(IPin * iface, IEnumMediaTypes ** ppEnum)
-{
-    VfwCapture *filter = impl_from_IPin(iface);
-    AM_MEDIA_TYPE *pmt;
-    HRESULT hr;
-
-    hr = qcap_driver_get_format(filter->driver_info, &pmt);
-    if (SUCCEEDED(hr)) {
-        hr = BasePinImpl_EnumMediaTypes(iface, ppEnum);
-        DeleteMediaType(pmt);
-    }
-    return hr;
-}
-
-static HRESULT WINAPI
-VfwPin_QueryInternalConnections(IPin * iface, IPin ** apPin, ULONG * cPin)
-{
-    TRACE("(%p)->(%p, %p)\n", iface, apPin, cPin);
-    return E_NOTIMPL;
-}
-
-static const IPinVtbl VfwPin_Vtbl =
-{
-    VfwPin_QueryInterface,
-    BasePinImpl_AddRef,
-    BasePinImpl_Release,
-    BaseOutputPinImpl_Connect,
-    BaseOutputPinImpl_ReceiveConnection,
-    BaseOutputPinImpl_Disconnect,
-    BasePinImpl_ConnectedTo,
-    BasePinImpl_ConnectionMediaType,
-    BasePinImpl_QueryPinInfo,
-    BasePinImpl_QueryDirection,
-    BasePinImpl_QueryId,
-    BasePinImpl_QueryAccept,
-    VfwPin_EnumMediaTypes,
-    VfwPin_QueryInternalConnections,
-    BaseOutputPinImpl_EndOfStream,
-    BaseOutputPinImpl_BeginFlush,
-    BaseOutputPinImpl_EndFlush,
-    BasePinImpl_NewSegment
+    .base.pin_query_accept = source_query_accept,
+    .base.pin_get_media_type = source_get_media_type,
+    .base.pin_query_interface = source_query_interface,
+    .pfnAttemptConnection = BaseOutputPinImpl_AttemptConnection,
+    .pfnDecideBufferSize = VfwPin_DecideBufferSize,
+    .pfnDecideAllocator = BaseOutputPinImpl_DecideAllocator,
 };
 
 IUnknown * WINAPI QCAP_createVFWCaptureFilter(IUnknown *outer, HRESULT *phr)
 {
     static const WCHAR source_name[] = {'O','u','t','p','u','t',0};
     VfwCapture *object;
-    PIN_INFO pin_info;
 
     if (!(object = CoTaskMemAlloc(sizeof(*object))))
     {
@@ -642,19 +574,14 @@ IUnknown * WINAPI QCAP_createVFWCaptureFilter(IUnknown *outer, HRESULT *phr)
         return NULL;
     }
 
-    strmbase_filter_init(&object->filter, &VfwCapture_Vtbl, outer, &CLSID_VfwCapture,
-            (DWORD_PTR)(__FILE__ ": VfwCapture.csFilter"), &BaseFuncTable);
+    strmbase_filter_init(&object->filter, outer, &CLSID_VfwCapture, &filter_ops);
 
     object->IAMStreamConfig_iface.lpVtbl = &IAMStreamConfig_VTable;
     object->IAMVideoProcAmp_iface.lpVtbl = &IAMVideoProcAmp_VTable;
     object->IPersistPropertyBag_iface.lpVtbl = &IPersistPropertyBag_VTable;
     object->init = FALSE;
 
-    pin_info.dir = PINDIR_OUTPUT;
-    pin_info.pFilter = &object->filter.IBaseFilter_iface;
-    lstrcpyW(pin_info.achName, source_name);
-    strmbase_source_init(&object->source, &VfwPin_Vtbl, &pin_info,
-            &output_BaseOutputFuncTable, &object->filter.csFilter);
+    strmbase_source_init(&object->source, &object->filter, source_name, &source_ops);
 
     object->IKsPropertySet_iface.lpVtbl = &IKsPropertySet_VTable;
 
